package ru.crp.lvms.chart;

import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

import java.util.List;

@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY)
@JsonSubTypes({
        @JsonSubTypes.Type(value = ConstChart.class, name = "ConstChart"),
        @JsonSubTypes.Type(value = LineChart.class, name = "LineChart")}
)
abstract public class Chart {
    public enum Type {
        CONST_X, CONST_Y, CURVE, DOTS
    }

    protected Type type;

    protected String title;
    protected List<String> description;

    protected String xAxisLabel;
    protected String yAxisLabel;

    protected String xDimension;
    protected String yDimension;

    protected AxisDomainType domainX;
    protected AxisDomainType domainY;

    protected Borders borders = new Borders();


    // for json deserialization
    public Chart() {
    }

    public Chart(Type type) {
        this.type = type;
    }

    public Chart(Type type, String title, String xAxisLabel, String xDimension, String yAxisLabel, String yDimension) {
        this(type);
        this.title = title;

        this.xAxisLabel = xAxisLabel;
        this.xDimension = xDimension;

        this.yAxisLabel = yAxisLabel;
        this.yDimension = yDimension;
    }

    public Chart(Type type, String title, String xAxisLabel, String xDimension, AxisDomainType domainX,
                 String yAxisLabel, String yDimension, AxisDomainType domainY) {
        this(type, title, xAxisLabel, xDimension, yAxisLabel, yDimension);
        this.domainX = domainX;
        this.domainY = domainY;
    }

    public ChartsSet toChartsSet() {
        ChartsSet chartsSet = new ChartsSet(this.title, this.xAxisLabel, this.xDimension, this.yAxisLabel, this.yDimension);
        if (this.domainX != null) {
            chartsSet.setDomainX(this.domainX);
        }
        if (this.domainY != null) {
            chartsSet.setDomainX(this.domainY);
        }
        chartsSet.setDescription(this.description);
        chartsSet.addChart(this);
        return chartsSet;
    }

    public void addBorderPoint(Point point) {
        borders.addPoint(point);
    }

    public void addXLineBorder(LineDescription lineDescription) {
        borders.addXLineBorder(lineDescription);
    }

    public void addYLineBorder(LineDescription lineDescription) {
        borders.addYLineBorder(lineDescription);
    }

    public void addPoint(Point point) {
        throw new IllegalStateException("Method is not expected to be called");
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public List<String> getDescription() {
        return description;
    }

    public void setDescription(List<String> description) {
        this.description = description;
    }

    public String getxAxisLabel() {
        return xAxisLabel;
    }

    public void setxAxisLabel(String xAxisLabel) {
        this.xAxisLabel = xAxisLabel;
    }

    public String getyAxisLabel() {
        return yAxisLabel;
    }

    public void setyAxisLabel(String yAxisLabel) {
        this.yAxisLabel = yAxisLabel;
    }

    public String getxDimension() {
        return xDimension;
    }

    public void setxDimension(String xDimension) {
        this.xDimension = xDimension;
    }

    public String getyDimension() {
        return yDimension;
    }

    public void setyDimension(String yDimension) {
        this.yDimension = yDimension;
    }

    public AxisDomainType getDomainX() {
        return domainX;
    }

    public void setDomainX(AxisDomainType domainX) {
        this.domainX = domainX;
    }

    public AxisDomainType getDomainY() {
        return domainY;
    }

    public void setDomainY(AxisDomainType domainY) {
        this.domainY = domainY;
    }

    public Borders getBorders() {
        return borders;
    }

    public void setBorders(Borders borders) {
        this.borders = borders;
    }
}
