package ru.crp.lvms.events.listeners;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.lvms.events.TaskStateChangedEvent;
import ru.crp.lvms.model.entities.tasks.TaskEntity;

@Component
public class TaskProgressEventListener {
    private static final Logger LOGGER = LoggerFactory.getLogger(TaskProgressEventListener.class);

    @EventListener
    public void onTaskUpdated(TaskStateChangedEvent taskStateChangedEvent) {
        TaskEntity newState = taskStateChangedEvent.getNewState();
        CalculationStatus oldStatus = CalculationStatus.valueOf(taskStateChangedEvent.getOldState().getStatusId());
        Integer oldProgress = taskStateChangedEvent.getOldState().getProgress();
        if (oldProgress == null) {
            oldProgress = 0;
        }
        Integer newProgress = newState.getProgress();
        if (newProgress == null) {
            newProgress = 0;
        }
        CalculationStatus newStatus = CalculationStatus.valueOf(newState.getStatusId());
        LOGGER.info("Task(id = '{}' for simulation = '{}') state changed '{}({})' -> '{}({})'",
                newState.getId(), newState.getParentName(), oldStatus, oldProgress, newStatus, newProgress);
    }
}
