package ru.crp.lvms.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Service;
import ru.crp.lvms.exceptions.EntityDeleteException;
import ru.crp.lvms.managers.construction.elements.ConstructionElementManager;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.dao.HelpChapterDao;
import ru.crp.lvms.model.dao.UserDao;
import ru.crp.lvms.model.dao.construction.elements.ConstructionElementDocsDao;
import ru.crp.lvms.model.entities.HelpChapterEntity;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementDocsEntity;
import ru.crp.lvms.optimizations.base.dao.OptimizationDao;
import ru.crp.lvms.utils.URLs;
import ru.crp.lvms.utils.ValidationHelper;

import javax.annotation.Nullable;
import javax.servlet.http.HttpServletResponse;

import java.net.URL;

import static ru.crp.lvms.utils.MessageUtils.*;

@Service
public class HelpManager extends BaseManagerSuggest<HelpChapterEntity>{

    @Autowired
    private HelpChapterDao helpChapterDao;

    @Autowired
    private OptimizationDao optimizationDao;

    @Override
    protected BaseSpaceDao<HelpChapterEntity> aliveDao() {
        return helpChapterDao;
    }

    public String getBookmarkName(String pageUrl) {
         String newPageUrl = modifiUrl(pageUrl);
         String bookmark = helpChapterDao.findOne(newPageUrl).getChapter();
         return (bookmark);
    }

    public String modifiUrl(String pageUrl) {
        String[] pathElements = pageUrl.split("/");
        if (pathElements[0].equals("profile")) {
            return pathElements[0];
        }
        if (pathElements[1].contains("?id=")) {
            return pageUrl.substring(0,pageUrl.indexOf("?id="));
        }
        if (pathElements.length <= 2) {
            return pageUrl;
        }
        else if (pathElements.length == 3 && isInteger(pathElements[2])){
            return (pathElements[0]+"/"+pathElements[1]);
        }
        else if (pathElements.length == 3 && !isInteger(pathElements[2])){
            return pageUrl;
        }
        else if (pathElements.length == 6 && pathElements[3].equals("settings")){
            if (optimizationDao.findOne(Integer.parseInt(pathElements[2])).getCaoName().equals("Tosca"))
            return (pathElements[0]+"/"+pathElements[1]+"/"+pathElements[3]+
                    "/"+optimizationDao.findOne(Integer.parseInt(pathElements[2])).getCaoName()+
                    "/"+optimizationDao.findOne(Integer.parseInt(pathElements[2])).getOptimizationTypeId()+
                    "/"+pathElements[5]);
            else if (optimizationDao.findOne(Integer.parseInt(pathElements[2])).getCaoName().equals("Optimus"))
                return "defaultPage";
            else
                return (pathElements[0]+"/"+pathElements[1]+"/"+pathElements[3]+
                        "/"+optimizationDao.findOne(Integer.parseInt(pathElements[2])).getCaoName()+
                        "/"+pathElements[5]);
        }
        else if (pathElements.length > 3 && isInteger(pathElements[2])){
            return (pathElements[0]+"/"+pathElements[1]+"/"+pathElements[3]);
        }
        else return "defaultPage";
    }

    public static boolean isInteger(String input){
        boolean parsable = true;
        try{
            Integer.parseInt(input);
        }catch(NumberFormatException e){
            parsable = false;
        }
        return parsable;
    }
}