package ru.crp.lvms.managers.misc;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Service;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.managers.NotificationManager;
import ru.crp.lvms.managers.UserManager;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.web.dto.misc.Auditable;

import static ru.crp.lvms.utils.MessageUtils.*;

@Service
public class EventManager {
    @Autowired
    private MessageSource msgSrc;

    @Autowired
    private NotificationManager notificationManager;

    @Autowired
    private UserManager userManager;

    public <E extends Entity & Auditable> void entityDeleted(E entity, UserEntity user) {
        String message = getMessage(ENTITY_DELETED, entity, user);
        notifyOwnerIfNeed(message, entity.getOwnerId(), user);
    }

    public <E extends Entity & Auditable> void entityUpdated(E entity, UserEntity user) {
        String message = getMessage(ENTITY_UPDATED, entity, user);
        notifyOwnerIfNeed(message, entity.getOwnerId(), user);
    }


    private void notifyOwnerIfNeed(String message, int ownerId, UserEntity user) {
        if (ownerId != user.getId()) {
            notificationManager.notifyUser(message, userManager.findOne(ownerId, null));
        }
    }


    private <E extends Entity & Auditable> String getMessage(String messageCode, E entity, UserEntity user) {
        String entityType = getEntityType(entity.getClass());
        String entityName = getEntityName(entity);
        return msgSrc.getMessage(messageCode, new Object[]{entityType, entityName, user.getId()}, null);
    }


    private <E extends Entity & Auditable> String getEntityName(E entity) {
        return String.valueOf(entity.getId());
    }
}
