package ru.crp.lvms.managers.tasks.resultsHandlers;

import ru.crp.cmls.calculations.solvers.xml.config.ResultType;
import ru.crp.lvms.exceptions.solving.ResultProcessingException;
import ru.crp.lvms.managers.tasks.ResultsParent;
import ru.crp.lvms.model.entities.ResultEntity;
import ru.crp.lvms.model.entities.misc.CreatedBy;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Date;
import java.util.List;

abstract public class KeyResultHandler {
    protected static final String DESCRIPTION_FILE_EXT = "descr";

    private Path pathToStorage;

    public KeyResultHandler(Path pathToStorage) {
        this.pathToStorage = pathToStorage;
    }

    public List<ResultEntity> handleResultsFile(ResultType resultType, Path pathToFile, ResultsParent resultsParent, CreatedBy createdBy) {
        if (!canHandle(resultType, pathToFile)) {
            throw new ResultProcessingException("Can't extract result type '" + resultType + "' from file " + pathToFile);
        }
        return doHandleResultsFile(pathToFile, resultsParent, createdBy);
    }

    public abstract boolean canHandle(ResultType resultType, Path pathToFile);

    protected abstract List<ResultEntity> doHandleResultsFile(Path pathToFile, ResultsParent resultsParent, CreatedBy createdBy);

    protected ResultEntity initCommonResultProperties(Path pathToFile, ResultsParent resultsParent, CreatedBy createdBy) {
        ResultEntity newResult = new ResultEntity();
        newResult.setPathToFile(pathToStorage.relativize(pathToFile).toString());
        newResult.setParentId(resultsParent.getId());
        newResult.setParentType(resultsParent.getType());
        newResult.setOwnerId(resultsParent.getOwnerId());
        newResult.setCreatedBy(createdBy);
        newResult.setCreationDate(new Date());
        return newResult;
    }

    protected void extractDescription(Path keyResultFile, ResultEntity keyResultEntity) {
        String descriptionFileName = keyResultFile.getFileName().toString() + "." + DESCRIPTION_FILE_EXT;
        Path descriptionFile = keyResultFile.resolveSibling(descriptionFileName);
        if (!Files.isRegularFile(descriptionFile)) {
            return;
        }
        try {
            String description = new String(Files.readAllBytes(descriptionFile), StandardCharsets.UTF_8);
            keyResultEntity.setDescription(description);
            Files.delete(descriptionFile);
        } catch (IOException ex) {
            throw new ResultProcessingException("Can't extract description for key result", ex);
        }
    }
}
