package ru.crp.lvms.managers.tasks.resultsHandlers;

import org.apache.commons.io.FilenameUtils;
import ru.crp.cmls.calculations.solvers.xml.config.ResultType;
import ru.crp.lvms.managers.tasks.ResultsParent;
import ru.crp.lvms.model.entities.ResultEntity;
import ru.crp.lvms.model.entities.misc.CreatedBy;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

import static java.util.Collections.singletonList;

public class PresentationResultHandler extends KeyResultHandler {
    public PresentationResultHandler(Path pathToStorage) {
        super(pathToStorage);
    }

    @Override
    public boolean canHandle(ResultType resultType, Path pathToFile) {
        return resultType == ResultType.presentations && Files.isRegularFile(pathToFile) && isPresentation(pathToFile);
    }

    @Override
    protected List<ResultEntity> doHandleResultsFile(Path pathToFile, ResultsParent resultsParent, CreatedBy createdBy) {
        ResultEntity newPresentation = initCommonResultProperties(pathToFile, resultsParent, createdBy);
        if (createdBy == CreatedBy.Auto) {
            extractDescription(pathToFile, newPresentation);
        }

        newPresentation.setName(FilenameUtils.removeExtension(pathToFile.getFileName().toString()));
        newPresentation.setType(ResultType.presentations);

        return singletonList(newPresentation);
    }

    public static boolean isPresentation(Path presentationFile) {
        String extension = FilenameUtils.getExtension(presentationFile.toString());
        return "ppt".equalsIgnoreCase(extension) || "pptx".equalsIgnoreCase(extension);
    }
}
