package ru.crp.lvms.model.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Repository;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntity;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntityTable;

import java.util.List;
import java.util.Map;

@Repository
@Primary
public class CompositeMaterialDao extends BaseCompositeDao<CompositeMaterialEntityTable, CompositeMaterialEntity> {
    @Component
    public static class TableDao extends BaseSpaceDao<CompositeMaterialEntityTable> {

    }

    @Autowired
    private TableDao tableDao;

    @Override
    protected TableDao tableDao() {
        return tableDao;
    }

    public List<CompositeMaterialEntity> getCompositeMaterialsUsingMaterial(int materialId) {
        return find("WHERE matrix_id = ? OR fiber_id = ?", materialId, materialId);
    }

    public List<CompositeMaterialEntity> getCompositeMaterialsUsingTechnology(int technologyId) {
        return find("WHERE technology_id = ?", technologyId);
    }

    public PageImpl<CompositeMaterialEntity> getCompositeMaterialsPageUsingTechnology(Map<String, Object> filters,
                                                                                      Pageable pageRequest, int technologyId) {
        return find("WHERE technology_id = ?", filters, pageRequest, technologyId);
    }

    public void removeTechnologyEquipment(int technologyId, int equipmentId) {
        jdbcTemplate().update("UPDATE composite_materials SET equipment_id = NULL " +
                "WHERE technology_id = ? AND equipment_id = ?", technologyId, equipmentId);
    }

    public CompositeMaterialEntity getCompositeMaterialUsingSimulation(int simulationId) {
        return findOne("JOIN virtual_simulations vs ON vs.composite_material_id = c.id WHERE vs.id = ?", simulationId);
    }
}
