package ru.crp.lvms.model.dao;

import org.springframework.stereotype.Repository;
import ru.crp.lvms.model.entities.Layer;

import java.util.ArrayList;
import java.util.List;

@Repository
public class LayersDao extends BaseSpaceDao<Layer> {
    private static final String DELETE_FOR_MATERIAL_SQL = "DELETE FROM layers WHERE composite_material_id = ?";
    private static final String BATCH_INSERT_SQL = "INSERT INTO layers (position, thickness, angle, "
            + "composite_material_id) VALUES (?, ?, ?, ?)";

    public List<Layer> findByMaterialId(int materialId) {
        return find("WHERE composite_material_id = ? ORDER BY position", materialId);
    }


    public void deleteMaterialLayers(int materialId) {
        getJdbcTemplate().update(DELETE_FOR_MATERIAL_SQL, materialId);
    }


    public void batchInsertLayers(List<Layer> layers) {
        if (layers.isEmpty()) {
            return;
        }
        List<Object[]> paramsInsert = new ArrayList<>();
        layers.forEach(layer -> paramsInsert.add(new Object[]{
                layer.getPosition(), layer.getThickness(), layer.getAngle(), layer.getCompositeMaterialId()
        }));
        getJdbcTemplate().batchUpdate(BATCH_INSERT_SQL, paramsInsert);
    }
}
