package ru.crp.lvms.model.dao;

import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.cmlb.model.utils.EntityUtils;
import ru.crp.lvms.model.entities.nodes.NodeEntity;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static java.util.stream.Collectors.toList;

@Transactional(propagation = Propagation.REQUIRED)
@Repository
@Primary
public class NodesDao extends BaseSpaceDao<NodeEntity> {
    private static final String NODES_SOLVERS_TABLE = "nodes_solver_types";

    public void updateNodeSolvers(int nodeId, Set<String> solverNames) {
        getJdbcTemplate().update("DELETE FROM " + NODES_SOLVERS_TABLE + " WHERE node_id= ? ", nodeId);
        List<Object[]> paramsInsert = new ArrayList<>();
        for (String solverName : solverNames) {
            paramsInsert.add(new Object[]{nodeId, solverName});
        }
        if (!paramsInsert.isEmpty()) {
            getJdbcTemplate().batchUpdate(
                    "INSERT INTO " + NODES_SOLVERS_TABLE + " (node_id, solver_name) values (?, ?)",
                    paramsInsert);
        }
    }

    public List<NodeEntity> findByUrl(String url) {
        return find("WHERE url=?", url);
    }

    public NodeEntity findByNameAndCluster(String nodeName, int clusterId) {
        List<NodeEntity> nodes = find("WHERE cluster_id = ? AND name = ?", clusterId, nodeName);
        return nodes.isEmpty() ? null : nodes.get(0);
    }


    @Nullable
    public NodeEntity findOneFreeWithRestrictions(@Nullable Long memory,
                                                  @Nullable Integer numOfCores,
                                                  String solverName) {
        Set<CalculationStatus> workingStatuses = CalculationStatus.getWorkingStatuses();
        String InSql = EntityUtils.sqlForListParams(workingStatuses);
        List<Object> params = new ArrayList<>(workingStatuses.stream().map(CalculationStatus::getId).collect(toList()));

        String memoryCond = "";
        String numOfCoresCond = "";
        params.add(0, solverName);
        if (numOfCores != null) {
            numOfCoresCond = "AND num_cores >= ?";
            params.add(0, numOfCores);
        }
        if (memory != null) {
            memoryCond = "AND memory >= ?";
            params.add(0, memory);
        }

        List<NodeEntity> suitableNodes = find("INNER JOIN nodes_solver_types nst ON nst.node_id = " + getAlias() + ".id" +
                        " WHERE alive = 'TRUE' " + memoryCond + " " + numOfCoresCond + " AND nst.solver_name = ?" +
                        " AND NOT EXISTS(SELECT * FROM tasks t WHERE t.node_id = " + getAlias() + ".id" +
                        " AND t.status_id IN " + InSql + ")",
                params.toArray());
        return suitableNodes.isEmpty() ? null : suitableNodes.get(0);
    }

    public List<String> findNodeSolvers(int nodeId) {
        return jdbcTemplate().queryForList("SELECT solver_name FROM nodes_solver_types WHERE node_id = ?", String.class, nodeId);
    }

    public boolean getNodeAliveStatus(int nodeId) {
        return getJdbcTemplate().queryForObject("SELECT alive FROM nodes WHERE id = ?", Boolean.class, nodeId);
    }

    public void updateAllNodesAliveStatus(int clusterId, boolean newState) {
        getJdbcTemplate().update("UPDATE nodes SET alive = ? WHERE cluster_id = ?", newState, clusterId);
    }

    public List<NodeEntity> getAllForCluster(int clusterId) {
        return find("WHERE cluster_id = ?", clusterId);
    }
}
