package ru.crp.lvms.model.dao;

import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import ru.crp.lvms.model.entities.ReferenceGraphEntry;

import java.util.List;

@Repository
public class ReferenceGraphDao extends BaseSpaceDao<ReferenceGraphEntry> {
    private static final String TABLE_NAME_PARAM = "<TABLE_NAME>";
    private static final String DESCENDANTS_AND_ME = "WITH RECURSIVE descending_branch(id, reference_id, name, path) AS (" +
            " SELECT rgd.id, rgd.reference_id, rgd.name, 'root' AS path" +
            " FROM " + TABLE_NAME_PARAM + " rgd WHERE id=?" +
            " UNION ALL" +
            " SELECT rgd.id, rgd.reference_id, rgd.name, descending_branch.path || '/' || rgd.reference_id AS path" +
            " FROM " + TABLE_NAME_PARAM + " rgd, descending_branch" +
            " WHERE rgd.reference_id=descending_branch.id" +
            ") SELECT * from descending_branch ORDER BY path DESC";

    private static final String ASCENDANTS_AND_ME = "WITH RECURSIVE ascending_branch(id, reference_id, name, path) AS (" +
            " SELECT rgd.id, rgd.reference_id, rgd.name, 'root' AS path" +
            " FROM " + TABLE_NAME_PARAM + " rgd WHERE id=?" +
            " UNION ALL" +
            " SELECT rgd.id, rgd.reference_id, rgd.name, ascending_branch.path || '/' || rgd.id AS path" +
            " FROM " + TABLE_NAME_PARAM + " rgd, ascending_branch" +
            " WHERE rgd.id=ascending_branch.reference_id" +
            ") SELECT * FROM ascending_branch ORDER BY path;";


    public List<ReferenceGraphEntry> getDescendantBranch(int id, String tableName) {
        String query = DESCENDANTS_AND_ME.replaceAll(TABLE_NAME_PARAM, tableName);
        return getJdbcTemplate().query(query, getRowMapper(), id);
    }


    public List<ReferenceGraphEntry> getAscendentBranchAndMe(int id, String tableName) {
        RowMapper<ReferenceGraphEntry> mapper = getRowMapper();
        String query = ASCENDANTS_AND_ME.replaceAll(TABLE_NAME_PARAM, tableName);
        return getJdbcTemplate().query(query, mapper, id);
    }
}
