package ru.crp.lvms.model.entities.tasks;

import com.fasterxml.jackson.annotation.JsonGetter;
import com.fasterxml.jackson.annotation.JsonIgnore;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmls.api.TaskParentType;
import ru.crp.cmls.calculations.solvers.xml.config.Type;
import ru.crp.lvms.cluster.model.ClusterType;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.misc.AuditableImpl;
import ru.crp.lvms.utils.Mapper;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;

import javax.annotation.Nullable;
import javax.validation.constraints.NotNull;
import java.util.Date;

@Table("tasks_join")
public class TaskEntity extends AuditableImpl implements Cloneable {
    @Column("status_id")
    private int statusId;

    @Column("queue_priority")
    private int queuePriority;

    @Column(value = "task_position", onlySelect = true)
    private Integer taskPosition;

    @Nullable
    @Column("node_id")
    private Integer nodeId;

    @Nullable
    @Column("additional_startup_arguments")
    private String additionalStartupArguments;

    @Nullable
    @Column("progress")
    private Integer progress;

    @Nullable
    @Column(value = "start_exec_date", onlySelect = true)
    private Date startExecDate;

    @NotNull(message = "Объект расчета должен быть задан")
    @Column("parent_id")
    private int parentId;

    @NotNull(message = "Тип объекта расчета должен быть задан")
    @Column("parent_type")
    private TaskParentType parentType;

    @GridColumn(name = "parentName", label = "Расчетный объект", cell = GridColumnType.uri, order = 1)
    @Column(value = "parent_name", onlySelect = true)
    private String parentName;

    @GridColumn(name = "status", label = "Статус расчета", order = 2)
    private String status;

    @GridColumn(name = "solverName", label = "Расчетный пакет", order = 3)
    @Column("solver_name")
    private String solverName;

    @GridColumn(name = "type", label = "Тип расчета", order = 4)
    @Column("type")
    private Type type;

    @GridColumn(name = "numOfCores", label = "Количество ядер процессора", cell = GridColumnType.integer, order = 5, isExcluded = true)
    @Nullable
    @Column("num_cores")
    private Integer numOfCores;

    @GridColumn(name = "memory", label = "RAM MB", cell = GridColumnType.integer, order = 6, isExcluded = true)
    @Nullable
    @Column("memory")
    private Long memory;

    @GridColumn(name = "node", label = "Расчетный узел", order = 7)
    @Nullable
    @Column(value = "node", onlySelect = true)
    private String node;

    @GridColumn(name = "owner", label = "Автор", cell = GridColumnType.uri, order = 102)
    @Column(value = "owner", onlySelect = true)
    private String owner;

    @Column("cluster_id")
    private int clusterId;

    @Column("auto_select_node")
    private boolean autoSelectNode;

    @Column("calculation_finished_with_exception")
    private boolean calculationFinishedWithException;

    @Nullable
    @Column("prev_task_id")
    private Integer prevTaskId;

    private boolean isStoppable;

    public static TaskEntity fromDTO(TaskIn taskIn, UserEntity owner) {
        TaskEntity newTask = Mapper.map(taskIn, TaskEntity.class);
        if (newTask.getNodeId() == null) {
            newTask.setAutoSelectNode(true);
        }
        if (newTask.getClusterId() == 0) {
            newTask.setClusterId(ClusterType.Space.getId());
        }
        if (newTask.getClusterId() == ClusterType.Slurm.getId()) {
            newTask.setAutoSelectNode(true);
            newTask.setNodeId(null);
        }
        newTask.setOwnerId(owner.getId());
        newTask.setStatusId(CalculationStatus.Waiting.getId());
        newTask.setQueuePriority(0);

        return newTask;
    }


    public int getStatusId() {
        return statusId;
    }

    public void setStatusId(int statusId) {
        this.statusId = statusId;
        CalculationStatus calculationStatus = CalculationStatus.valueOf(statusId);
        this.status = calculationStatus.getDisplayName();
        this.isStoppable = CalculationStatus.getStoppableStatuses().contains(calculationStatus);
    }

    public int getQueuePriority() {
        return queuePriority;
    }

    public void setQueuePriority(int queuePriority) {
        this.queuePriority = queuePriority;
    }

    public Integer getTaskPosition() {
        return taskPosition;
    }

    public void setTaskPosition(Integer taskPosition) {
        this.taskPosition = taskPosition;
    }

    public String getSolverName() {
        return solverName;
    }

    public void setSolverName(String solverName) {
        this.solverName = solverName;
    }

    @Nullable
    public String getAdditionalStartupArguments() {
        return additionalStartupArguments;
    }

    public void setAdditionalStartupArguments(@Nullable String additionalStartupArguments) {
        this.additionalStartupArguments = additionalStartupArguments;
    }

    @Nullable
    public Integer getNodeId() {
        return nodeId;
    }

    public void setNodeId(@Nullable Integer nodeId) {
        this.nodeId = nodeId;
    }

    @Nullable
    public Integer getNumOfCores() {
        return numOfCores;
    }

    public void setNumOfCores(@Nullable Integer numOfCores) {
        this.numOfCores = numOfCores;
    }

    @Nullable
    public Long getMemory() {
        return memory;
    }

    public void setMemory(@Nullable Long memory) {
        this.memory = memory;
    }

    @Nullable
    public Integer getProgress() {
        return progress;
    }

    public void setProgress(@Nullable Integer progress) {
        this.progress = progress;
    }

    @Nullable
    public Date getStartExecDate() {
        return startExecDate;
    }

    public void setStartExecDate(@Nullable Date startExecDate) {
        this.startExecDate = startExecDate;
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    @Nullable
    public String getNode() {
        return node;
    }

    public void setNode(@Nullable String node) {
        this.node = node;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public int getParentId() {
        return parentId;
    }

    public void setParentId(int parentId) {
        this.parentId = parentId;
    }

    public TaskParentType getParentType() {
        return parentType;
    }

    public void setParentType(TaskParentType parentType) {
        this.parentType = parentType;
    }

    @JsonIgnore
    public Type getType() {
        return type;
    }

    @JsonGetter("type")
    public String getTypeDisplayName() {
        return type.getDisplayName();
    }

    public void setType(Type type) {
        this.type = type;
    }

    public String getParentName() {
        return parentName;
    }

    public void setParentName(String parentName) {
        this.parentName = parentName;
    }

    public int getClusterId() {
        return clusterId;
    }

    public void setClusterId(int clusterId) {
        this.clusterId = clusterId;
    }

    public boolean getAutoSelectNode() {
        return autoSelectNode;
    }

    public void setAutoSelectNode(boolean autoSelectNode) {
        this.autoSelectNode = autoSelectNode;
    }

    public boolean getCalculationFinishedWithException() {
        return calculationFinishedWithException;
    }

    public void setCalculationFinishedWithException(boolean calculationFinishedWithException) {
        this.calculationFinishedWithException = calculationFinishedWithException;
    }

    @JsonGetter("stoppable")
    public boolean getIsStoppable() {
        return isStoppable;
    }

    @Nullable
    public Integer getPrevTaskId() {
        return prevTaskId;
    }

    public void setPrevTaskId(@Nullable Integer prevTaskId) {
        this.prevTaskId = prevTaskId;
    }

    @Override
    public TaskEntity clone() throws CloneNotSupportedException {
        return (TaskEntity) super.clone();
    }
}
