package ru.crp.lvms.model.entities.tasks;

import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.cmls.api.TaskParentType;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.utils.AuthUtils;

import java.util.Date;

public class TaskShort {
    private int id;
    private int parentId;
    private String parentName;
    private TaskParentType parentType;
    private String owner;
    private String type;
    private String status;
    private Integer progress;
    private Date modificationDate;
    private Date startExecDate;
    private Boolean isStoppable;

    public TaskShort(TaskEntity taskFull) {
        this.id = taskFull.getId();
        this.parentId = taskFull.getParentId();
        this.parentName = taskFull.getParentName();
        this.parentType = taskFull.getParentType();
        this.owner = taskFull.getOwner();
        this.type = taskFull.getType().getDisplayName();
        this.status = taskFull.getStatus();
        this.progress = taskFull.getProgress();

        this.modificationDate = taskFull.getModificationDate();
        this.startExecDate = taskFull.getStartExecDate();

        this.isStoppable = canStopTask(taskFull);
    }

    public static boolean canStopTask(TaskEntity taskEntity) {
        // if it is just ModelMapper test task entity
        if (taskEntity.getId() == 0) {
            return false;
        }
        CalculationStatus taskStatus = CalculationStatus.valueOf(taskEntity.getStatusId());

        boolean taskCanBeStopped = CalculationStatus.getStoppableStatuses().contains(taskStatus);

        boolean userHasEnoughRights = true;
        PortalUser currentUser = PortalUser.getCurrentUser();
        if (currentUser != null) {
            userHasEnoughRights = AuthUtils.isOwnerOrAdmin(taskEntity.getOwnerId(), currentUser.getUser());
        }
        return taskCanBeStopped && userHasEnoughRights;
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public int getParentId() {
        return parentId;
    }

    public void setParentId(int parentId) {
        this.parentId = parentId;
    }

    public String getParentName() {
        return parentName;
    }

    public void setParentName(String parentName) {
        this.parentName = parentName;
    }

    public TaskParentType getParentType() {
        return parentType;
    }

    public void setParentType(TaskParentType parentType) {
        this.parentType = parentType;
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public Integer getProgress() {
        return progress;
    }

    public void setProgress(Integer progress) {
        this.progress = progress;
    }

    public Date getModificationDate() {
        return modificationDate;
    }

    public void setModificationDate(Date modificationDate) {
        this.modificationDate = modificationDate;
    }

    public Date getStartExecDate() {
        return startExecDate;
    }

    public void setStartExecDate(Date startExecDate) {
        this.startExecDate = startExecDate;
    }

    public Boolean getStoppable() {
        return isStoppable;
    }

    public void setStoppable(Boolean stoppable) {
        isStoppable = stoppable;
    }
}
