package ru.crp.lvms.model.entities.tree;

import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.lvms.model.FolderColor;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementEntity;
import ru.crp.lvms.model.entities.misc.ItemType;
import ru.crp.lvms.optimizations.base.entities.OptimizationType;
import ru.crp.lvms.optimizations.base.enums.OptimizationTypes;
import ru.crp.lvms.validators.EntityExists;

import javax.validation.constraints.NotNull;

@Table("optimizations_tree_join")
public class OptimizationsTreeNode extends TreeNode {
    @EntityExists(entityClass = OptimizationType.class, message = "Тип оптимизации не найден")
    @NotNull(message = "Тип оптимизации должен быть задан")
    @Column("optimization_type_id")
    private Integer optimizationTypeId;

    @EntityExists(entityClass = ConstructionElementEntity.class, message = "Элемент конструкции не найден")
    @NotNull(message = "Элемент конструкции должен быть задан")
    @Column("construction_element_id")
    private Integer constructionElementId;

    @Column("base_optimization_id")
    private Integer baseOptimizationId;

    public Integer getOptimizationTypeId() {
        return optimizationTypeId;
    }

    public void setOptimizationTypeId(Integer optimizationTypeId) {
        this.optimizationTypeId = optimizationTypeId;
    }

    public Integer getConstructionElementId() {
        return constructionElementId;
    }

    public void setConstructionElementId(Integer constructionElementId) {
        this.constructionElementId = constructionElementId;
    }

    public Integer getBaseOptimizationId() {
        return baseOptimizationId;
    }

    public void setBaseOptimizationId(Integer baseOptimizationId) {
        this.baseOptimizationId = baseOptimizationId;
    }

    @Override
    public String getAltText() {
        if (optimizationTypeId == null) {
            return String.format("Оптимизация - %s", getName());
        } else {
            return String.format("%s - %s", OptimizationTypes.valueOf(optimizationTypeId).getDisplayName(), getName());
        }
    }

    @Override
    public FolderColor getColor() {
        if (optimizationTypeId == null || getItemType() != ItemType.OptimizationFolder) {
            return FolderColor.BLACK;
        } else {
            return FolderColor.fromOptimizationType(OptimizationTypes.valueOf(optimizationTypeId));
        }
    }
}
