package ru.crp.lvms.optimizations.base.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.optimizations.base.dao.OptimizationJsonSettingsDao;
import ru.crp.lvms.optimizations.base.entities.OptimizationJsonSetting;
import ru.crp.lvms.optimizations.base.entities.OptimizationListSetting;

public class JsonSettingsController {

    @Autowired
    protected OptimizationJsonSettingsDao jsonSettingsDao;

    protected OptimizationJsonSetting updateParam(int optimizationId, int paramId, OptimizationJsonSetting updatedModel) {
        validateJsonSetting(updatedModel);
        OptimizationJsonSetting existedParam = jsonSettingsDao.findById(optimizationId, paramId);
        if (existedParam == null) {
            throw new BadRequestException("Параметр с номером " + paramId + " не найден");
        }
        existedParam.setPlainValue(updatedModel.getPlainValue());
        existedParam.setBoolValue(updatedModel.getBoolValue());
        existedParam.setListValue(updatedModel.getListValue());
        existedParam.setCompositeValue(updatedModel.getCompositeValue());
        return jsonSettingsDao.updateAndRead(existedParam);
    }

    private void validateJsonSetting(OptimizationJsonSetting jsonSetting) {
        switch (jsonSetting.getType()) {
            case plain:
                if (jsonSetting.getPlainValue() == null) {
                    throw new BadRequestException("Значение параметра не задано");
                }
                break;
            case bool:
                if (jsonSetting.getBoolValue() == null) {
                    throw new BadRequestException("Значение параметра не задано");
                }
                break;
            case list:
                if (jsonSetting.getListValue() == null) {
                    throw new BadRequestException("Значение параметра не задано");
                }
                long selectedObjects = jsonSetting.getListValue().stream()
                        .filter(OptimizationListSetting::isSelected)
                        .count();
                if (selectedObjects > 1) {
                    throw new BadRequestException("Только одно из допустимых значений может быть выбрано");
                }
                break;
            case composite:
                if (jsonSetting.getCompositeValue() == null) {
                    throw new BadRequestException("Значение параметра не задано");
                }
                break;
            default:
                throw new IllegalArgumentException("Unexepected enum value: " + jsonSetting.getType());
        }
    }

}
