package ru.crp.lvms.optimizations.base.dao;

import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.cmlb.model.utils.EntityUtils;
import ru.crp.cmls.api.TaskParentType;
import ru.crp.cmls.calculations.solvers.xml.config.ResultType;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.entities.ResultEntity;
import ru.crp.lvms.model.entities.misc.CreatedBy;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

@Repository
// overriding 'mandatory' transaction to be able to use this class in background jobs
@Transactional(propagation = Propagation.SUPPORTS)
public class ResultsDao extends BaseSpaceDao<ResultEntity> {

    public List<ResultEntity> findAllByParent(int parentId, TaskParentType parentType) {
        return find("WHERE parent_id = ? AND parent_type = ?", parentId, parentType.toString());
    }

    public List<ResultEntity> findCustomResultsByParent(int parentId, TaskParentType parentType,ResultType type) {
        return find("WHERE parent_id = ? AND parent_type = ? AND type = ?", parentId, parentType.toString(),type.toString());
    }

    public PageImpl<ResultEntity> findPageByParent(int parentId, TaskParentType parentType, Map<String, Object> filters, Pageable page) {
        return find("WHERE parent_id = ? AND parent_type = ?", filters, page, parentId, parentType.toString());
    }

    public PageImpl<ResultEntity> findCustomResultsPageByParent(int parentId, TaskParentType parentType, Map<String, Object> filters, Pageable page, ResultType type) {
        return find("WHERE parent_id = ? AND parent_type = ? AND type = ?", filters, page, parentId, parentType.toString(),type.toString());
    }

    public void deleteWithPaths(List<String> paths) {
        String inSql = EntityUtils.sqlForListParams(paths);
        jdbcTemplate().update("DELETE FROM results WHERE path_to_file IN " + inSql, paths.toArray());
    }

    public void batchInsert(List<ResultEntity> extractedResults) {
        if (extractedResults.isEmpty()) {
            return;
        }
        List<Object[]> toInsert = extractedResults.stream()
                .map(res -> new Object[]{res.getParentId(), res.getParentType().toString(),
                        res.getCreatedBy().toString(), res.getType().toString(),
                        res.getName(), res.getDescription(), res.getPathToFile(), res.getOwnerId(),
                        res.getValue(),res.getTargetValue()})
                .collect(toList());
        jdbcTemplate().batchUpdate("INSERT INTO " +
                "results_join(parent_id, parent_type, created_by, type, name, description, path_to_file, owner_id, value, target_value)" +
                "VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", toInsert);
    }

    public List<ResultEntity> getResultByParentAndType(int parentId, TaskParentType parentType, ResultType resultType) {
        return find("WHERE parent_id = ? AND parent_type = ? AND type = ?", parentId, parentType.toString(), resultType.toString());
    }

    public void deleteResults(int parentId, TaskParentType parentType, List<CreatedBy> createdBy) {
        if (createdBy.isEmpty()) {
            return;
        }
        String inSql = EntityUtils.sqlForListParams(createdBy);

        List<Object> params = new ArrayList<>(createdBy.stream().map(CreatedBy::toString).collect(toList()));
        params.add(0, parentType.toString());
        params.add(0, parentId);

        jdbcTemplate().update("DELETE FROM results WHERE " + parentType.getColName() + " = ? AND parent_type = ? AND created_by IN " + inSql,
                params.toArray());
    }
}
