package ru.crp.lvms.optimizations.base.entities;

import com.fasterxml.jackson.annotation.JsonIgnore;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmlb.model.entities.base.Named;
import ru.crp.lvms.model.entities.misc.BaseEntityTable;
import ru.crp.lvms.model.entities.misc.PostprocessingStatus;
import ru.crp.lvms.model.entities.misc.StatusFlag;
import ru.crp.lvms.validators.Unique;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;

import javax.annotation.Nullable;
import javax.validation.constraints.NotNull;

@Table("optimizations")
//@Unique(entityClass = OptimizationEntity.class, fieldName = "name", columnNane = "name", message = "Имя должно быть уникально")
public class OptimizationEntityTable extends BaseEntityTable implements Named {
    @GridColumn(name = "name", label = "Название", order = 0, cell = GridColumnType.uri)
    @Column("name")
    @NotNull(message = "Имя должно быть задано")
    private String name;

    @Column("pid")
    @NotNull(message = "Родительская директория должна быть задана")
    private Integer pid;

    @Column("optimization_type_id")
    private Integer optimizationTypeId;

    @Column("reference_id")
    private Integer referenceId;

    @Column("flag")
    private StatusFlag flag = StatusFlag.none;

    private String flagDisplayName = StatusFlag.none.getDisplayName();

    @Column("status_id")
    //todo rename
    private int solvingStatusId;

    //todo remove
    private Integer postprocessingStatusId = PostprocessingStatus.NotExecuted.getId();

    @Column(value = "image_file_id", onlySelect = true)
    @JsonIgnore
    private Integer imageFileId;

    @Column("description")
    private String description;

    @Column("solver_cao_id")
    private Integer solverCaoId;

    @Column("solver_cae_id")
    private Integer solverCaeId;

    @Column("solver_cad_id")
    private Integer solverCadId;

    @Override
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Integer getPid() {
        return pid;
    }

    public void setPid(Integer pid) {
        this.pid = pid;
    }

    public Integer getOptimizationTypeId() {
        return optimizationTypeId;
    }

    public void setOptimizationTypeId(Integer optimizationTypeId) {
        this.optimizationTypeId = optimizationTypeId;
    }

    public Integer getReferenceId() {
        return referenceId;
    }

    public void setReferenceId(Integer referenceId) {
        this.referenceId = referenceId;
    }

    public StatusFlag getFlag() {
        return flag;
    }

    public void setFlag(@Nullable StatusFlag flag) {
        this.flag = flag;
        if (flag == null) {
            this.flagDisplayName = null;
        } else {
            this.flagDisplayName = flag.getDisplayName();
        }
    }

    public Integer getSolvingStatusId() {
        return solvingStatusId;
    }

    public void setSolvingStatusId(Integer solvingStatusId) {
        this.solvingStatusId = solvingStatusId;
    }

    public Integer getPostprocessingStatusId() {
        return postprocessingStatusId;
    }

    public void setPostprocessingStatusId(Integer postprocessingStatusId) {
        this.postprocessingStatusId = postprocessingStatusId;
    }

    public Integer getImageFileId() {
        return imageFileId;
    }

    public void setImageFileId(Integer imageFileId) {
        this.imageFileId = imageFileId;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    @Nullable
    public Integer getSolverCaoId() {
        return solverCaoId;
    }

    public void setSolverCaoId(Integer solverCaoId) {
        this.solverCaoId = solverCaoId;
    }

    public String getFlagDisplayName() {
        return flagDisplayName;
    }

    public Integer getSolverCaeId() {
        return solverCaeId;
    }

    public void setSolverCaeId(Integer solverCaeId) {
        this.solverCaeId = solverCaeId;
    }

    public Integer getSolverCadId() {
        return solverCadId;
    }

    public void setSolverCadId(Integer solverCadId) {
        this.solverCadId = solverCadId;
    }
}
