package ru.crp.lvms.optimizations.base.enums;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;

import javax.annotation.Nonnull;
import java.util.Objects;

@JsonFormat(shape = JsonFormat.Shape.OBJECT)
public enum OptimizationCae implements OptimizationSolver {
    ANSYS("ANSYS", "Ansys"),
    ABAQUS("ABAQUS", "Abaqus"),
    NASTRAN("NASTRAN", "Nastran"),
    LSDYNA("LS-DYNA", "LS-Dyna double"),
    NXNASTRAN("NX-NASTRAN", "NX-Nastran");

    private int id;

    private String uniqueName;

    private String solverConfigName;

    OptimizationCae(String uniqueName, String solverConfigName) {
        this.uniqueName = uniqueName;
        this.solverConfigName = solverConfigName;
    }

    public String getSolverConfigName() {
        return solverConfigName;
    }

    public void setSolverConfigName(String solverConfigName) {
        this.solverConfigName = solverConfigName;
    }

    @Override
    public int getId() {
        return id;
    }

    @Override
    public void setId(int id) {
        this.id = id;
    }

    @JsonIgnore
    public String getUniqueName() {
        return uniqueName;
    }

    public void setUniqueName(String uniqueName) {
        this.uniqueName = uniqueName;
    }

    @Nonnull
    public static OptimizationCae valueOf(Integer id) {
        for (OptimizationCae type : values()) {
            if (type.getId() == id) {
                return type;
            }
        }
        throw new IllegalArgumentException("There is no cae with such id: " + id);
    }

    @Nonnull
    public static OptimizationCae fromUniqueName(String uniqueName) {
        for (OptimizationCae type : values()) {
            if (Objects.equals(type.getUniqueName(), uniqueName)) {
                return type;
            }
        }
        throw new IllegalArgumentException("There is no cae with such unique name: " + uniqueName);
    }

    public String getName() {
        return this.name();
    }
}
