package ru.crp.lvms.optimizations.base.enums;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;

import javax.annotation.Nonnull;
import java.util.Objects;

@JsonFormat(shape = JsonFormat.Shape.OBJECT)
public enum OptimizationTypes {
    EngineeringCalculation("ENGINEERING_CALCULATION"),
    TopologicalOptimization("TOPOLOGICAL_OPTIMIZATION"),
    TopographicOptimization("TOPOGRAPHIC_OPTIMIZATION"),
    ParametricShapeOptimization("PARAMETRIC_SHAPE_OPTIMIZATION"),
    //TopologicalOptimizationOfMicrostructure("TOPOLOGICAL OPTIMIZATION OF MICROSTRUCTURE"),
    ParametricOptimizationOfMicrostructure("PARAMETRIC OPTIMIZATION OF MICROSTRUCTURE"),
    ShapeOptimization("SHAPE_OPTIMIZATION");

    private int id;

    private String uniqueName;

    private String displayName;

    OptimizationTypes(String uniqueName) {
        this.uniqueName = uniqueName;
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    @JsonIgnore
    public String getUniqueName() {
        return uniqueName;
    }

    public void setUniqueName(String uniqueName) {
        this.uniqueName = uniqueName;
    }

    @Nonnull
    public static OptimizationTypes valueOf(Integer id) {
        for (OptimizationTypes type : values()) {
            if (type.getId() == id) {
                return type;
            }
        }
        throw new IllegalArgumentException("There is no OptimizationType with such id: " + id);
    }

    @Nonnull
    public static OptimizationTypes fromUniqueName(String uniqueName) {
        for (OptimizationTypes type : values()) {
            if (Objects.equals(type.getUniqueName(), uniqueName)) {
                return type;
            }
        }
        throw new IllegalArgumentException("There is no OptimizationType with such unique name: " + uniqueName);
    }

    public String getName() {
        return this.name();
    }

    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    public String getDisplayName() {
        return displayName;
    }
}
