package ru.crp.lvms.optimizations.optimus.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import ru.crp.lvms.optimizations.base.dao.OptimizationJsonSettingsDao;
import ru.crp.lvms.optimizations.base.enums.OptimizationCao;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettingsController;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettingsManager;
import ru.crp.lvms.optimizations.optimus.entities.OptimusOptimizationBaseSetting;
import ru.crp.lvms.optimizations.optimus.enums.OptimusBaseSettingType;
import ru.crp.lvms.optimizations.optimus.enums.OptimusSettingsType;
import ru.crp.lvms.optimizations.optimus.managers.OptimusOptimizationSettingsManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.utils.URLs;

import static ru.crp.lvms.optimizations.optimus.enums.OptimusBaseSettingType.NUM_CAO_VARYING_PARAMS;
import static ru.crp.lvms.optimizations.optimus.enums.OptimusBaseSettingType.NUM_TARGET_FUNCS;
//import static ru.crp.lvms.optimizations.mf.enums.MFBaseSettingType.NUM_CAO_VARYING_PARAMS;
//import static ru.crp.lvms.optimizations.mf.enums.MFBaseSettingType.NUM_TARGET_FUNCS;

@RestController
@RequestMapping(value = URLs.OPTIMUS_BASE_SETTINGS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class OptimusBaseSettingsController extends AbstractOptimizationSettingsController<OptimusOptimizationBaseSetting, OptimusSettingsType> {

    @Autowired
    private OptimusOptimizationSettingsManager optimusOptimizationSettingsManager;

    @Autowired
    private OptimizationJsonSettingsDao optimizationJsonSettingsDao;


    @Override
    protected OptimusSettingsType getSettingsType() {
        return OptimusSettingsType.BaseSetting;
    }

    @Override
    protected Integer getSettingsCount(int optimizationId) {
        return OptimusBaseSettingType.values().length;
    }

    @Override
    protected AbstractOptimizationSettingsManager<OptimusSettingsType> getSettingsManager() {
        return optimusOptimizationSettingsManager;
    }

    @Override
    protected OptimizationCao getOptimizationSolverType() {
        return OptimizationCao.Optimus;
    }

    @Override
    public OptimusOptimizationBaseSetting updateSettings(@PathVariable("optimizationId") int optimizationId,
                                                    @PathVariable("id") int settingsId,
                                                    @RequestBody OptimusOptimizationBaseSetting updatedSettings,
                                                    @CurrentUser PortalUser currentUser) {
        OptimusOptimizationBaseSetting resp = (OptimusOptimizationBaseSetting) super.updateSettings(optimizationId, settingsId, updatedSettings, currentUser);
        if (resp.getTypeId() == NUM_TARGET_FUNCS.getId()) {
            optimusOptimizationSettingsManager.selectDefaultAlgorithm(optimizationId, resp.getCurrentValue());
        }
        if (resp.getTypeId() == NUM_TARGET_FUNCS.getId() || resp.getTypeId() == NUM_CAO_VARYING_PARAMS.getId()) {
            Integer selectedAlgorithmId = optimusOptimizationSettingsManager.findSelectedAlgorithmId(optimizationId);
            optimizationJsonSettingsDao.resetOptimusDefaultDoeParams(optimizationId, selectedAlgorithmId);
        }
        return resp;
    }
}
