package ru.crp.lvms.optimizations.optimus.managers;

import org.modelmapper.Converter;
import org.modelmapper.spi.MappingContext;
import org.springframework.stereotype.Component;
import ru.crp.lvms.optimizations.base.entities.OptimizationJsonSetting;
import ru.crp.lvms.optimizations.base.entities.OptimizationListSetting;
import ru.crp.lvms.optimizations.base.enums.JsonSettingsType;
//import ru.crp.lvms.optimizations.mf.config.xml.PropertyType;
import ru.crp.lvms.optimizations.optimus.config.xml.FactorsType;

import javax.annotation.Nullable;
import java.util.List;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.joining;

@Component
public class OptimusPropertyTypeConverter implements Converter<OptimizationJsonSetting, FactorsType> {

    @Override
    public FactorsType convert(MappingContext<OptimizationJsonSetting, FactorsType> context) {
        OptimizationJsonSetting source = context.getSource();
        FactorsType dest = context.getDestination();
//        dest.setName(source.getXmlName());
//       // dest.setValue(getOptimizationJsonSettingValue(source));
//        if (source.getType() == JsonSettingsType.list) {
//            String keys = source.getListValue().stream().map(OptimizationListSetting::getXmlName).collect(joining(","));
//            dest.setRange(keys);
//        }
//        if (source.getType() == JsonSettingsType.composite) {
//            String keys = source.getCompositeValue().stream().map(OptimizationJsonSetting::getXmlName).collect(joining(","));
//            dest.setRange(keys);
//        }
        return dest;
    }

    private String getOptimizationJsonSettingValue(OptimizationJsonSetting setting) {
        switch (setting.getType()) {
            case plain:
                return withDefault(setting.getPlainValue());
            case bool:
                return withDefault(String.valueOf(setting.getBoolValue()));
            case list:
                List<OptimizationListSetting> values = setting.getListValue();
                String selectedValue = values.stream().filter(OptimizationListSetting::isSelected).findFirst()
                        .map(OptimizationListSetting::getXmlName).orElse("");
                return withDefault(selectedValue);
            case composite:
                return withDefault(setting.getCompositeValue().stream()
                        .map(this::getOptimizationJsonSettingValue).collect(Collectors.joining(",")));
            default:
                throw new IllegalArgumentException("Unexepected enum value: " + setting.getType());
        }
    }

    private String withDefault(@Nullable String val) {
        return (val == null || val.equals("null")) ? "" : val;
    }

}


