package ru.crp.lvms.optimizations.optistruct.controllers;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettings;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructCollector;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationLoadStep;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructBaseSettingType;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructCollectorType;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructLoadStepCalculationType;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructSettingsType;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;

import java.util.List;

import static ru.crp.lvms.utils.URLs.OPTISTRUCT_LOAD_STEP_URL;

@RestController
@RequestMapping(value = OPTISTRUCT_LOAD_STEP_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class OptistructLoadStepController extends OptistructAbstractOptimizationSettingsController<OptistructOptimizationLoadStep> {

    @Override
    protected OptistructBaseSettingType getOptistructOptimizationBaseSettingType() {
        return OptistructBaseSettingType.NUM_LOAD_STEPS;
    }

    @Override
    protected OptistructSettingsType getSettingsType() {
        return OptistructSettingsType.LoadStep;
    }

    @RequestMapping(method = RequestMethod.GET, path = "/loads")
    public List<OptistructCollector> getPossibleCollectorsForStep(@PathVariable("optimizationId") int optimizationId,
                                                                  @CurrentUser PortalUser currentUser) {
        checkOptimizationSolverType(optimizationId, currentUser);
        return getSettingsManager().findCollectorsForOptimizationByType(optimizationId, OptistructCollectorType.Load);
    }

    @RequestMapping(method = RequestMethod.GET, path = "/calculationTypes")
    public OptistructLoadStepCalculationType[] getPossibleLoadStepTypes(@PathVariable("optimizationId") int optimizationId,
                                                                        @CurrentUser PortalUser currentUser) {
        return OptistructLoadStepCalculationType.values();
    }

    @Override
    public AbstractOptimizationSettings updateSettings(@PathVariable("optimizationId") int optimizationId,
                                                       @PathVariable("id") int settingsId,
                                                       @RequestBody OptistructOptimizationLoadStep updatedSettings,
                                                       @CurrentUser PortalUser currentUser) {
        if (updatedSettings.getCalculationType() == OptistructLoadStepCalculationType.STATIC) {
            updatedSettings.setIsFixed(true);
        }
        return super.updateSettings(optimizationId, settingsId, updatedSettings, currentUser);
    }
}
