package ru.crp.lvms.optimizations.optistruct.dao;

import org.postgresql.jdbc4.Jdbc4Array;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettignsDao;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationLoadStep;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructLoadStepCalculationType;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.stream.Collectors;

import static java.util.Arrays.asList;

@Repository
@Transactional
public class OptistructLoadStepDao extends AbstractOptimizationSettignsDao<OptistructOptimizationLoadStep> {

    @PostConstruct
    public void init() {
        setFindSql("SELECT * FROM " + getTableName());
        setFindAllSql("SELECT * FROM " + getTableName());
        setFindOneSql("SELECT * FROM " + getTableName() + " WHERE id = ?");
        setRowMapper((resultSet, rowNum) -> {
            Jdbc4Array pgArray = (Jdbc4Array) resultSet.getArray("selected_load_ids");
            Integer[] loads = new Integer[0];
            if (pgArray != null) {
                loads = (Integer[]) pgArray.getArray();
            }
            OptistructOptimizationLoadStep loadStep = new OptistructOptimizationLoadStep();
            loadStep.setId(resultSet.getInt("id"));
            loadStep.setOptimizationId(resultSet.getInt("optimization_id"));
            loadStep.setOptimizationTypeId(resultSet.getInt("optimization_type_id"));
            loadStep.setName(resultSet.getString("name"));
            loadStep.setScriptId(resultSet.getInt("script_id"));
            loadStep.setIsFixed(resultSet.getBoolean("is_fixed"));
            loadStep.setCalculationType(OptistructLoadStepCalculationType.valueOf(resultSet.getString("calculation_type")));
            loadStep.setSelectedLoadIds(new ArrayList<>(asList(loads)));
            return loadStep;
        });
    }

    @Override
    public int insert(OptistructOptimizationLoadStep optistructOptimizationLoadStep) {
        int id = super.insert(optistructOptimizationLoadStep);
        updateSelectedLoadIds(findOne(id));
        return id;
    }

    @Override
    public void update(OptistructOptimizationLoadStep optistructOptimizationLoadStep) {
        super.update(optistructOptimizationLoadStep);
        updateSelectedLoadIds(optistructOptimizationLoadStep);
    }

    private void updateSelectedLoadIds(OptistructOptimizationLoadStep optistructOptimizationLoadStep) {
        if (!optistructOptimizationLoadStep.getSelectedLoadIds().isEmpty()) {
            String array = "{" + optistructOptimizationLoadStep
                    .getSelectedLoadIds()
                    .stream()
                    .map(String::valueOf)
                    .collect(Collectors.joining(", ")) + "}";
            jdbcTemplate().update("UPDATE " + getTableName() + " SET selected_load_ids = \'" + array +
                    "\' WHERE id = ?", optistructOptimizationLoadStep.getId());
        }
    }
}
