package ru.crp.lvms.optimizations.optistruct.dao;

import org.springframework.stereotype.Repository;
import ru.crp.cmlb.model.utils.EntityUtils;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

import static java.util.Arrays.asList;

@Repository
public class OptistructTechnologicalConstraintTreeValueDao extends BaseSpaceDao<TechnologicalConstraintTreeNodeValue> {
    public OptistructTechnologicalConstraintTreeValueDao() {
        setTableName("optistruct_technological_constraints_tree_values_join");
    }

    public void clearSavedValues(int optimizationTcId) {
        jdbcTemplate().update("DELETE FROM optistruct_technological_constraints_tree_values WHERE tc_id = ?", optimizationTcId);
    }

    public List<TechnologicalConstraintTreeNodeValue> findForConstraint(Integer optimizationTcId) {
        return find("WHERE tc_id = ?", optimizationTcId);
    }


    public void deleteByGroupAndParent(int optimizationTcId, String groupName, Integer parentId, int skippedId) {
        List<Object> params = new ArrayList<>(asList(optimizationTcId, optimizationTcId, parentId, groupName, skippedId));
        List<Integer> nodesToDelete = getJdbcTemplate().queryForList("SELECT tc_tree_id FROM optistruct_technological_constraints_tree_values_join WHERE EXISTS " +
                "(WITH RECURSIVE recs(id, parent_id) AS " +
                "(SELECT values.tc_tree_id AS id, values.parent_id AS parent_id FROM optistruct_technological_constraints_tree_values_join values WHERE values.tc_id = ? " +
                "UNION ALL select tree.id AS id, tree.parent_id AS parent_id FROM optistruct_technological_constraints_tree tree,  recs AS r " +
                "WHERE tree.id = r.parent_id) " +
                "SELECT * from recs r WHERE r.id IN (" +
                "SELECT otct.tc_tree_id FROM optistruct_technological_constraints_tree_values_join otct " +
                " WHERE otct.tc_id = ? AND otct.parent_id = ? AND otct.type = 'select' AND otct.group_name = ?  AND otct.id <> ?))", params.toArray(), Integer.class);

        if (nodesToDelete.isEmpty()) {
            return;
        }
        String inSQL = EntityUtils.sqlForListParams(nodesToDelete);
        params = new ArrayList<>(nodesToDelete);
        params.add(0, optimizationTcId);
        getJdbcTemplate().update("DELETE FROM optistruct_technological_constraints_tree_values WHERE tc_id = ? AND tc_tree_id IN " + inSQL, params.toArray());
    }


    public void deleteByTreeId(Integer optimizationTcId, Integer treeId) {
        getJdbcTemplate().update("DELETE FROM optistruct_technological_constraints_tree_values WHERE tc_id = ? AND tc_tree_id = ?", optimizationTcId, treeId);
    }


    @Nullable
    public TechnologicalConstraintTreeNodeValue findByTreeId(Integer optimizationTCId, Integer technologicalConstraintTreeId) {
        List<TechnologicalConstraintTreeNodeValue> values = find("WHERE tc_id = ? AND tc_tree_id = ?", optimizationTCId, technologicalConstraintTreeId);
        if (values.isEmpty()) {
            return null;
        }
        return values.get(0);
    }


    public void updateValue(TechnologicalConstraintTreeNodeValue entity) {
        jdbcTemplate().update("UPDATE optistruct_technological_constraints_tree_values_join SET value = ? WHERE tc_tree_id = ? AND tc_id = ?",
                entity.getValue(), entity.getTechnologicalConstraintTreeId(), entity.getTechnologicalConstraintId());
    }


    public void deleteSeveralFieldsByFieldIds(int optimizationTCId, List<Integer> treeIds) {
        if (treeIds.isEmpty()) {
            return;
        }
        List<Object> params = new ArrayList<>(treeIds);
        params.add(0, optimizationTCId);
        String inSql = EntityUtils.sqlForListParams(treeIds);
        getJdbcTemplate().update("DELETE FROM optistruct_technological_constraints_tree_values WHERE tc_id = ? AND tc_tree_id IN " + inSql,
                params.toArray());
    }


    public void deleteWithChildrenByTreeId(int optimizationTCId, List<Integer> fieldsToDelete) {
        if (fieldsToDelete.isEmpty()) {
            return;
        }
        List<Object> params = new ArrayList<>(fieldsToDelete);
        params.add(0, optimizationTCId);
        params.add(optimizationTCId);
        String inSQL = EntityUtils.sqlForListParams(fieldsToDelete);
        List<Integer> nodesToDelete = getJdbcTemplate().queryForList("WITH RECURSIVE recs(tc_tree_id) AS (SELECT otct.tc_tree_id " +
                "                            FROM optistruct_technological_constraints_tree_values_join otct " +
                "                            WHERE otct.tc_id = ? AND otct.tc_tree_id IN " + inSQL +
                "                            UNION ALL SELECT otct2.tc_tree_id " +
                "                                      FROM optistruct_technological_constraints_tree_values_join otct2, recs AS r " +
                "                                      WHERE otct2.parent_id = r.tc_tree_id AND otct2.tc_id = ?) " +
                "SELECT * FROM recs", params.toArray(), Integer.class);

        if (nodesToDelete.isEmpty()) {
            return;
        }
        params = new ArrayList<>(nodesToDelete);
        params.add(0, optimizationTCId);
        inSQL = EntityUtils.sqlForListParams(nodesToDelete);
        getJdbcTemplate().update("DELETE FROM optistruct_technological_constraints_tree_values WHERE tc_id = ? AND tc_tree_id IN " + inSQL, params.toArray());
    }
}
