package ru.crp.lvms.optimizations.optistruct.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.CannotSerializeTransactionException;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.lvms.optimizations.optistruct.dao.*;
import ru.crp.lvms.optimizations.optistruct.entities.*;

import java.util.Comparator;
import java.util.List;

@Service
@Transactional(isolation = Isolation.SERIALIZABLE)
// Set retry on transaction isolation fail, use random exponential back off
@Retryable(include = CannotSerializeTransactionException.class, maxAttempts = 5,
        backoff = @Backoff(delay = 100, multiplier = 2, random = true))
public class OptistructSuggestManager {

    @Autowired
    private OptistructTFAttaDao optistructTFAttaDao;

    @Autowired
    private OptistructTFAttbDao optistructTFAttbDao;

    @Autowired
    private OptistructTFAttiDao optistructTFAttiDao;

    @Autowired
    private OptistructTFPtypeDao optistructTFPtypeDao;

    @Autowired
    private OptistructTFDao optistructTFDao;

    @Autowired
    private OptistructTechConstraintDao optistructTechConstraintDao;

    @Autowired
    private OptistructTFOptimizationTypeDao optistructTFOptimizationTypeDao;

    public List<OptistructTFAtta> findAllowedAtta(int tfId) {
        return optistructTFAttaDao.findAllowed(tfId);
    }

    public List<OptistructTFAttb> findAllowedAttb(int tfId) {
        return optistructTFAttbDao.findAllowed(tfId);
    }

    public List<OptistructTFAtti> findAllowedAtti(int tfId) {
        return optistructTFAttiDao.findAllowed(tfId);
    }

    public List<OptistructTFPtype> findAllowedPtype(int tfId) {
        return optistructTFPtypeDao.findAllowed(tfId);
    }

    public List<OptistructTF> findAllTF() {
        return optistructTFDao.findAll();
    }

    public List<OptistructTFOptimizationType> findAllTFOptimizationTypes() {
        return optistructTFOptimizationTypeDao.findAll();
    }

    public List<OptistructTechnologicalConstraint> findAllTechnologicalConstraints() {
        List<OptistructTechnologicalConstraint> all = optistructTechConstraintDao.findAll();
        all.sort(Comparator.comparing(OptistructTechnologicalConstraint::getName));
        return all;
    }
}
