package ru.crp.lvms.optimizations.optistruct.tcl;

import org.apache.commons.lang3.text.StrSubstitutor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.optistruct.dao.*;
import ru.crp.lvms.optimizations.optistruct.entities.*;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructBaseSettingType;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructSettingsType;
import ru.crp.lvms.optimizations.optistruct.managers.OptistructOptimizationSettingsManager;
import ru.crp.lvms.utils.NumericalInputConverter;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.Arrays.asList;
import static java.util.stream.Collectors.toList;
import static org.apache.commons.lang3.StringUtils.isEmpty;
import static ru.crp.lvms.optimizations.optistruct.managers.OptistructAdditionalParametersManager.PLAIN_TYPE_ID;
import static ru.crp.lvms.utils.Utils.zipWithIndex;

@Component
public class OptistructTFAndConstraintsGenerator {

    @Autowired
    private OptistructOptimizationSettingsManager optistructOptimizationSettingsManager;

    @Autowired
    private OptistructTFAttaDao optistructTFAttaDao;

    @Autowired
    private NumericalInputConverter numericalInputConverter;

    @Autowired
    private OptistructTFOptimizationTypeDao optistructTFOptimizationTypeDao;

    @Autowired
    private OptistructTFDao optistructTFDao;

    @Autowired
    private OptistructOptimizationLnSetDao optistructOptimizationLnSetDao;

    @Autowired
    private OptistructTFPtypeDao optistructTFPtypeDao;


    public String formTF(OptimizationEntity optimization, Map<OptistructBaseSettingType, OptistructOptimizationBaseSetting> baseSettings) {
        Integer tfCount = baseSettings.get(OptistructBaseSettingType.NUM_TARGET_FUNCS).getCurrentValue();
        Stream<OptistructOptimizationTF> tfs = optistructOptimizationSettingsManager
                .findOptimizationSettings(optimization.getId(), OptistructSettingsType.TargetFunction, null)
                .stream()
                .limit(tfCount)
                .map(abstr -> (OptistructOptimizationTF) abstr);
        return tfs
                .map(tf -> {

                    String atta = getAtta(tf.getAttaId(), tf.getAttaValue());

                    List<Integer> attiIdsWithZeros = Optional.ofNullable(tf.getAttiId())
                            .map(unused -> numericalInputConverter.convert(tf.getAttiValue()))
                            .orElse(new ArrayList<>());

                    List<Integer> attiIds = new ArrayList<>(attiIdsWithZeros);
                    attiIdsWithZeros.addAll(asList(0, 0, 0, 0, 0, 0));

                    String createArray = "*createarray " + attiIdsWithZeros.size() + " " +
                            attiIdsWithZeros.stream().map(String::valueOf).collect(Collectors.joining(" ")) + "\n";

                    String optType = " " + optistructTFOptimizationTypeDao.findOne(tf.getOptimizationTypeId()).getId();

                    String loadCond = " " + optistructTFDao.findOne(tf.getTfId()).getLcDefault();
                    String ptype = "0";
                    List<OptistructTFPtype> optistructTFPtypes = optistructTFPtypeDao.findAllowed(tf.getTfId());
                    String optiResponseCreate = String.join(" ", "*optiresponsecreate", "\"" + tf.getName() + "\"",
                            tf.getTfResponse(), ptype, "0", atta, "0", String.valueOf(attiIds.size()), "6 0 0 0 1",
                            String.valueOf(attiIdsWithZeros.size()), "1 6");
                    if (tf.getAttiId()!=null && tf.getAttiValue()!= null) {
                        OptistructLnSet lnSet = optistructOptimizationLnSetDao.findOne(Integer.parseInt(tf.getAttiValue()));
                        return  "set temparraysize [llength [hm_getentityarray sets " + lnSet.getName() + " " + lnSet.getLnSetTypeName() + "]]\n" +
                                "set temparraysizewithzeros [expr 6+$temparraysize]\n"+
                                "eval *createarray $temparraysizewithzeros [hm_getentityarray sets " + lnSet.getName() + " " +
                                lnSet.getLnSetTypeName() + "] 0 0 0 0 0 0\n"+
                                "*createdoublearray 6 0 0 0 0 0 0\n"+
                                "*optiresponsecreate " + "\"" + tf.getName() + "\" " + tf.getTfResponse()+ " " +
                                optistructTFPtypes.get(0).getPropId()+" 0 "+ atta +
                                " 0 $temparraysize 6 0 0 0 1 $temparraysizewithzeros 1 6\n"+
                                "*optiresponsesetequationdata1 \"" + tf.getName() + "\" 0 0 0 0 1 0 \n" +
                                "*optiresponsesetequationdata2 \"" + tf.getName() + "\" 0 0 1 0\n" +
                                "*optiobjectivecreate 1 " + optType + loadCond;
                    }
                    return createArray +
                            "*createdoublearray 6  0 0 0 0 0 0\n" +
                            optiResponseCreate + "\n" +
                            "*optiresponsesetequationdata1 \"" + tf.getName() + "\" 0 0 0 0 1 0 \n" +
                            "*optiresponsesetequationdata2 \"" + tf.getName() + "\" 0 0 1 0\n" +
                            "*optiobjectivecreate 1 " + optType + loadCond;
                }).collect(Collectors.joining("\n"));
    }


    public String formConstraints(OptimizationEntity optimization, Map<OptistructBaseSettingType, OptistructOptimizationBaseSetting> baseSettings) {
        Integer tfCount = baseSettings.get(OptistructBaseSettingType.NUM_TARGET_FUNCS).getCurrentValue();
        Integer cCount = baseSettings.get(OptistructBaseSettingType.NUM_CONSTRAINTS).getCurrentValue();
        List<OptistructOptimizationConstraint> cs = optistructOptimizationSettingsManager
                .findOptimizationSettings(optimization.getId(), OptistructSettingsType.Constraint, null)
                .stream()
                .limit(cCount)
                .map(abstr -> (OptistructOptimizationConstraint) abstr)
                .collect(toList());
        return zipWithIndex(cs, tfCount + 1)
                .map(pair -> {
                    OptistructOptimizationConstraint c = pair.getValue();


                    String atta = getAtta(c.getAttaId(), c.getAttaValue());

                    List<Integer> attiIdsWithZeros = Optional.ofNullable(c.getAttiId())
                            .map(unused -> numericalInputConverter.convert(c.getAttiValue()))
                            .orElse(new ArrayList<>());

                    List<Integer> attiIds = new ArrayList<>(attiIdsWithZeros);
                    attiIdsWithZeros.addAll(asList(0, 0, 0, 0, 0, 0));

                    String createArray = "*createarray " + attiIdsWithZeros.size() + " " +
                            attiIdsWithZeros.stream().map(String::valueOf).collect(Collectors.joining(" ")) + "\n";

                    String ptype = "0";
                    List<OptistructTFPtype> optistructTFPtypes = optistructTFPtypeDao.findAllowed(c.getTfId());

                    String optiResponseCreate = String.join(" ", "*optiresponsecreate", "\"" + c.getName() + "\"",
                            c.getTfResponse(), ptype, "0", atta, "0", String.valueOf(attiIds.size()), "6 0 0 0 1",
                            String.valueOf(attiIdsWithZeros.size()), "1 6");

                    String constraintCreate = getConstraintCreate(c, pair.getKey());
                    if (c.getAttiId()!=null && c.getAttiValue()!= null) {
                        OptistructLnSet lnSet = optistructOptimizationLnSetDao.findOne(Integer.parseInt(c.getAttiValue()));
                        return  "set temparraysize [llength [hm_getentityarray sets " + lnSet.getName() + " " + lnSet.getLnSetTypeName() + "]]\n" +
                                "set temparraysizewithzeros [expr 6+$temparraysize]\n"+
                                "eval *createarray $temparraysizewithzeros [hm_getentityarray sets " + lnSet.getName() + " " +
                                lnSet.getLnSetTypeName() + "] 0 0 0 0 0 0\n"+
                                "*createdoublearray 6 0 0 0 0 0 0\n"+
                                "*optiresponsecreate " + "\"" + c.getName() + "\" " + c.getTfResponse()+ " " +
                                 optistructTFPtypes.get(0).getPropId()+" 0 "+ atta +
                                " 0 $temparraysize 6 0 0 0 1 $temparraysizewithzeros 1 6\n"+
                                "*optiresponsesetequationdata1 \"" + c.getName() + "\" 0 0 0 0 1 0 \n" +
                                "*optiresponsesetequationdata2 \"" + c.getName() + "\" 0 0 1 0\n" +
                                "*createarray 1  1 \n" +
                                constraintCreate;
                    }
                    return createArray +
                            "*createdoublearray 6  0 0 0 0 0 0\n" +
                            optiResponseCreate + "\n" +
                            "*optiresponsesetequationdata1 \"" + c.getName() + "\" 0 0 0 0 1 0 \n" +
                            "*optiresponsesetequationdata2 \"" + c.getName() + "\" 0 0 1 0\n" +
                            "*createarray 1  1 \n" +
                            constraintCreate;
                }).collect(Collectors.joining("\n"));
    }


    private String getConstraintCreate(OptistructOptimizationConstraint constraint, Integer id) {
        Map<String, String> params = new HashMap<>();
        params.put("name", constraint.getName());
        params.put("id", "" + id);
        params.put("boundType", getBoundType(constraint));
        params.put("integersAmount", " " + optistructTFDao.findOne(constraint.getTfId()).getIntegersAmount());
        params.put("lowerBound", constraint.getLowerBound());
        params.put("upperBound", constraint.getUpperBound());
        StrSubstitutor substitutor = new StrSubstitutor(params, "<", ">");
        return substitutor.replace("*opticonstraintcreate \"<name>\" <id> <boundType> <lowerBound> <upperBound> 1 <integersAmount>");
    }


    private String getBoundType(OptistructOptimizationConstraint constraint) {
        if (!isEmpty(constraint.getUpperBound()) && !isEmpty(constraint.getLowerBound())) {
            return "2";
        } else if (isEmpty(constraint.getUpperBound())) {
            return "0";
        } else {
            return "1";
        }
    }


    private String getAtta(Integer attaId, String attaValue) {
        if (Objects.equals(attaId, PLAIN_TYPE_ID)) {
            return attaValue;
        }
        return Optional.ofNullable(attaId)
                .map(attaIdNotNull -> optistructTFAttaDao.findOne(attaIdNotNull))
                .map(OptistructTFAtta::getPropId)
                .orElse("0");
    }
}
