package ru.crp.lvms.optimizations.optistruct.tcl.tc;

import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructSet;

import java.util.List;
import java.util.stream.Stream;

import static java.util.Arrays.asList;
import static org.apache.batik.svggen.SVGStylingAttributes.set;

public class CastTCConverter extends OptistructTCConverter {
    public static final String SOLID_COMMAND = "TC_CAST_SOLID";
    public static final String FIRST_NODE_ID_COMMAND = "TC_CAST_ID_1";
    public static final String SECOND_NODE_ID_COMMAND = "TC_CAST_ID_2";
    public static final String ENABLE_NO_HOLES_COMMAND = "TC_CAST_NO_HOLES";

    public static final String THICKNESS_COMMAND_NAME = "TC_CAST_DESIRED_THICKNESS_VAL";

    public static final String FIRST_NODE_COORDS_PREFIX = "TC_CAST_FIRST_NODE";
    public static final String SECOND_NODE_COORDS_PREFIX = "TC_CAST_SECOND_NODE";

    public static final int CAST_TC_ID = 8;


    @Override
    public Stream<String> convert(OptistructOptimizationTechnologicalConstraint currentConstraint, List<TechnologicalConstraintTreeNodeValue> currentConstraintValues,
                                  List<OptistructOptimizationTechnologicalConstraint> allConstraints, String setName,
                                  List<String> alreadyConvertedConstraints) {
        String typeArg = currentConstraintValues.stream()
                .filter(c -> c.getName().equals(SOLID_COMMAND))
                .findAny().map(s -> "1").orElse("2");

        String firstNodeId = extractValue(FIRST_NODE_ID_COMMAND, "-1", "0", currentConstraintValues);

        String secondNodeId = extractValue(SECOND_NODE_ID_COMMAND, "-1", "0", currentConstraintValues);

        String enableNoHoles = extractValueForCheckbox(ENABLE_NO_HOLES_COMMAND, "1", "0", currentConstraintValues);

        String thicknessArg = extractValue(THICKNESS_COMMAND_NAME, "0", "0", currentConstraintValues);

        String createCoordsArrayCommand = generateCreateCoordsArrayCommand(asList(FIRST_NODE_COORDS_PREFIX, SECOND_NODE_COORDS_PREFIX), currentConstraintValues);

        String tcCommand = String.format("*topologydrawupdatewithnoholeandstamp \"%s\" %s %s %s 1 6 0 %s %s",
                setName, typeArg, firstNodeId, secondNodeId, enableNoHoles, thicknessArg);

        return Stream.of(createCoordsArrayCommand, tcCommand);
    }

    @Override
    public boolean canConvert(OptistructOptimizationTechnologicalConstraint constraint) {
        return constraint.getTechnologicalConstraintId().equals(CAST_TC_ID);
    }
}
