package ru.crp.lvms.optimizations.optistruct.tcl.tc;

import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructSet;
import ru.crp.lvms.optimizations.optistruct.managers.OptistructTechnologicalConstraintTreeValueManager;

import java.util.List;
import java.util.stream.Stream;

import static java.util.Arrays.asList;
import static java.util.stream.Collectors.toList;
import static org.apache.batik.svggen.SVGStylingAttributes.set;

public class DefaultTCConverter extends OptistructTCConverter {
    public static final List<Integer> DEFAULT_TC_IDS = asList(1, 2, 3, 4, 5, 6);
    public static final String MIN_DIM_COMMAND = "MIN_DIM_VALUE";
    public static final String MAX_DIM_COMMAND = "MAX_DIM_VALUE";
    public static final String TENSION_COMMAND = "TENSION_VALUE";

    public static final String FATIQUE_TYPE_MIN_COMMAND = "FATIQUE_TYPE_MIN";
    public static final String FATIQUE_TYPE_MAX_COMMAND = "FATIQUE_TYPE_MAX";
    public static final String FATIQUE_VAL_COMMAND = "FATIQUE_VALUE";

    public static final String MIN_GAP_COMMAND = "GAP_VALUE";
    public static final String GRID_ACTIVE_COMMAND = "TC_GRID_ACTIVE";
    public static final String GRID_DIR_COMMAND = "TC_GRID_DIR";

    private OptistructTechnologicalConstraintTreeValueManager optistructTechnologicalConstraintTreeValueManager;

    public DefaultTCConverter(OptistructTechnologicalConstraintTreeValueManager optistructTechnologicalConstraintTreeValueManager) {
        this.optistructTechnologicalConstraintTreeValueManager = optistructTechnologicalConstraintTreeValueManager;
    }

    @Override
    public Stream<String> convert(OptistructOptimizationTechnologicalConstraint currentConstraint, List<TechnologicalConstraintTreeNodeValue> currentConstraintValues,
                                  List<OptistructOptimizationTechnologicalConstraint> allConstraints, String setName, List<String> alreadyConvertedConstraints) {
        if (alreadyConvertedConstraints.stream().anyMatch(s -> s.startsWith("*topologyparametersupdatewithmingap"))) {
            return Stream.empty();
        }
        List<TechnologicalConstraintTreeNodeValue> allUsedValues = allConstraints.stream().filter(c -> DEFAULT_TC_IDS.contains(c.getTechnologicalConstraintId()))
                .flatMap(c -> optistructTechnologicalConstraintTreeValueManager.findValuesOfTC(c.getId()).stream())
                .collect(toList());

        String minDim = extractValue(MIN_DIM_COMMAND, "0", "0", allUsedValues);
        String maxDim = extractValue(MAX_DIM_COMMAND, "0", "0", allUsedValues);
        String tension = extractValue(TENSION_COMMAND, "0", "0", allUsedValues);

        String gridType = extractValueForCheckbox(GRID_ACTIVE_COMMAND, "1", "0", allUsedValues);
        if (!gridType.equals("0")) {
            gridType = extractValueForCheckbox(GRID_DIR_COMMAND, "2", "1", allUsedValues);
        }
        String fatiqueType = extractValueForCheckbox(FATIQUE_TYPE_MIN_COMMAND, "2", FATIQUE_TYPE_MAX_COMMAND, "1", "0", allUsedValues);

        String fatiqueVal = extractValue(FATIQUE_VAL_COMMAND, "0", "0", allUsedValues);

        String minGapValue = extractValue(MIN_GAP_COMMAND, "0", "0", allUsedValues);

        return Stream.of(String.format("*topologyparametersupdatewithmingap \"%s\" %s %s %s %s %s %s %s",
                setName, minDim, maxDim, tension, gridType, fatiqueType, fatiqueVal, minGapValue));
    }


    @Override
    public boolean canConvert(OptistructOptimizationTechnologicalConstraint constraint) {
        return DEFAULT_TC_IDS.contains(constraint.getTechnologicalConstraintId());
    }
}
