package ru.crp.lvms.optimizations.tosca.config;

import org.apache.commons.lang3.tuple.Pair;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class Command {
    private String name;
    private List<Pair<String, String>> parameters;

    public Command() {
        parameters = new ArrayList<>();
    }

    public Command(String name) {
        this.name = name;
        parameters = new ArrayList<>();
    }

    public Command(String name, List<Pair<String, String>> parameters) {
        this.name = name;
        this.parameters = new ArrayList<>(parameters);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<Pair<String, String>> getParameters() {
        return parameters;
    }

    public void setParameters(List<Pair<String, String>> parameters) {
        this.parameters = new ArrayList<>(parameters);
    }

    public Command addParameter(String key, String value) {
        parameters.add(Pair.of(key, value));
        return this;
    }

    public Command addParameters(List<Pair<String, String>> params) {
        parameters.addAll(params);
        return this;
    }

    @Override
    public String toString() {
        String params = parameters.stream()
                .map(param -> "\t" + param.getKey() + "\t=\t" + param.getValue())
                .collect(Collectors.joining("\n"));
        return name + "\n" + params + "\nEND_\n";
    }
}
