package ru.crp.lvms.optimizations.tosca.config.argumentConverters;

import org.apache.commons.lang3.tuple.Pair;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;

import java.util.Collections;
import java.util.List;
import java.util.stream.Stream;

import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class MidPlaneArgumentConverter extends CommandArgumentConverter {
    public static final int MID_PLANE_ARGUMENT_ID = 12;
    public static final int POINT_ARGUMENT_ID = 16;
    public static final List<Integer> ANCESTOR_IDS = Collections.singletonList(MID_PLANE_ARGUMENT_ID);
    public static final String MID_PLANE_COMMAND_NAME = "MID_PLANE";

    public MidPlaneArgumentConverter(ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager) {
        super(toscaOptimizationTechnologicalConstraintManager);
    }

    @Override
    public List<Pair<String, String>> convertValue(ToscaOptimizationTechnologicalConstraint constraint,
                                                   TechnologicalConstraintTreeNodeValue value,
                                                   List<TechnologicalConstraintTreeNodeValue> allValues,
                                                   List<Pair<String, String>> alreadyConvertedValues) {
        // if already handled
        if (alreadyConvertedValues.stream().anyMatch(v -> v.getKey().equals(MID_PLANE_COMMAND_NAME))) {
            return Collections.emptyList();
        }
        List<TechnologicalConstraintTreeNode> ascendingBranch = getAscendingBranch(value);
        // grandChildren should be handled inside parent branch, so just skip
        if (ascendingBranch.size() > 2) {
            return Collections.emptyList();
        }
        if (value.getTechnologicalConstraintTreeId() != POINT_ARGUMENT_ID) {
            return Collections.singletonList(Pair.of(MID_PLANE_COMMAND_NAME, value.getName()));
        }
        // POINT
        List<TechnologicalConstraintTreeNodeValue> pointParams = allValues.stream()
                .filter(v -> v.getParentId() != null && v.getParentId().equals(value.getTechnologicalConstraintTreeId()))
                .collect(toList());
        String commandValue = Stream.of(ascendingBranch.get(0).getName(),
                pointParams.stream().filter(v -> v.getName().equals("X")).map(TechnologicalConstraintTreeNodeValue::getValue).findFirst().orElse("0"),
                pointParams.stream().filter(v -> v.getName().equals("Y")).map(TechnologicalConstraintTreeNodeValue::getValue).findFirst().orElse("0"),
                pointParams.stream().filter(v -> v.getName().equals("Z")).map(TechnologicalConstraintTreeNodeValue::getValue).findFirst().orElse("0")
        )
                .collect(joining(","));
        return Collections.singletonList(Pair.of(MID_PLANE_COMMAND_NAME, commandValue));
    }

    @Override
    public boolean canConvert(ToscaOptimizationTechnologicalConstraint constraint, TechnologicalConstraintTreeNodeValue value) {
        return toscaOptimizationTechnologicalConstraintManager.hasAncestors(value.getTechnologicalConstraintTreeId(), ANCESTOR_IDS);
    }
}
