package ru.crp.lvms.optimizations.tosca.config.argumentConverters;

import org.apache.commons.lang3.tuple.Pair;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

import static java.util.stream.Collectors.joining;


public class MultipleChildrenArgumentConverter extends CommandArgumentConverter {
    // todo check CYCLIC_SYM_START
    public static final int CLIENT_DIR_ARGUMENT_ID = 94;
    // CLIENT_DIR, DEMOLD_DIR, TOL, SURF_PARAM, SURF_PARAM, SURF_PARAM, CYCLIC_SYM_START
    public static final List<Integer> PARENT_IDS = Arrays.asList(CLIENT_DIR_ARGUMENT_ID, 98, 103, 136, 125, 149, 114);

    public MultipleChildrenArgumentConverter(ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager) {
        super(toscaOptimizationTechnologicalConstraintManager);
    }

    @Override
    public List<Pair<String, String>> convertValue(ToscaOptimizationTechnologicalConstraint constraint,
                                                   TechnologicalConstraintTreeNodeValue value,
                                                   List<TechnologicalConstraintTreeNodeValue> allValues,
                                                   List<Pair<String, String>> alreadyConvertedValues) {
        // parent will be handled in child branch
        if (PARENT_IDS.contains(value.getId())) {
            return Collections.emptyList();
        }
        TechnologicalConstraintTreeNode parent = toscaOptimizationTechnologicalConstraintManager.getField(value.getParentId());
        // already handled
        if (alreadyConvertedValues.stream().anyMatch(v -> v.getKey().equals(parent.getName()))) {
            return Collections.emptyList();
        }
        String commandValue = allValues.stream()
                .filter(v -> Objects.equals(v.getParentId(), value.getParentId()))
                .map(TechnologicalConstraintTreeNodeValue::getValue)
                .filter(Objects::nonNull)
                .collect(joining(","));
        if (commandValue.isEmpty()) {
            return Collections.emptyList();
        }
        return Collections.singletonList(Pair.of(parent.getName(), commandValue));

    }

    @Override
    public boolean canConvert(ToscaOptimizationTechnologicalConstraint constraint, TechnologicalConstraintTreeNodeValue value) {
        return (value.getParentId() != null && PARENT_IDS.contains(value.getParentId()) || PARENT_IDS.contains(value.getTechnologicalConstraintTreeId()));
    }
}
