package ru.crp.lvms.optimizations.tosca.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.optimizations.tosca.entities.ToscaConstraint;
import ru.crp.lvms.optimizations.tosca.entities.ToscaTargetFuncOrConstraintParameter;
import ru.crp.lvms.optimizations.tosca.enums.NodeOrElem;
import ru.crp.lvms.optimizations.tosca.enums.ToscaBaseSettingType;
import ru.crp.lvms.optimizations.tosca.enums.ToscaConstraintType;
import ru.crp.lvms.optimizations.tosca.enums.ToscaSettingsType;
import ru.crp.lvms.optimizations.tosca.managers.ToscaSuggestManager;
import ru.crp.lvms.utils.URLs;

import java.util.List;
import java.util.Map;

import static java.util.Collections.emptyList;

@RestController
@RequestMapping(value = URLs.TOSCA_CONSTRAINT_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class ToscaConstraintController extends ToscaAbstractOptimizationSettingsController<ToscaConstraint> {

    @Override
    protected ToscaBaseSettingType getToskaOptimizationBaseSettingType() {
        return ToscaBaseSettingType.NUM_CONSTRAINTS;
    }

    @Override
    protected ToscaSettingsType getSettingsType() {
        return ToscaSettingsType.Constraint;
    }

    @Autowired
    private ToscaSuggestManager toscaSuggestManager;

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/parameters", method = RequestMethod.GET)
    public Map<String, List<ToscaTargetFuncOrConstraintParameter>> getAvailableCParameters(@PathVariable Integer optimizationId) {
        return toscaSuggestManager.getAvailableParameters(optimizationId, false);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/parameters", method = RequestMethod.GET, params = {"group"})
    public List<ToscaTargetFuncOrConstraintParameter> getAvailableCParametersForGroup(@PathVariable Integer optimizationId,
                                                                                      @RequestParam("group") String groupName) {
        return toscaSuggestManager.getAvailableParameters(optimizationId, false).getOrDefault(groupName, emptyList());
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/parameters/groups", method = RequestMethod.GET)
    public List<String> getAvailableCParametersGroups(@PathVariable Integer optimizationId) {
        return toscaSuggestManager.getAvailableParametersGroups(optimizationId, false);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/types", method = RequestMethod.GET)
    public ToscaConstraintType[] getAllAvailableCTypes() {
        return ToscaConstraintType.values();
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/nodeOrElement", method = RequestMethod.GET)
    public NodeOrElem[] getAllAvailableNoesForConstraints() {
        return NodeOrElem.values();
    }
}
