package ru.crp.lvms.optimizations.tosca.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.optimizations.tosca.entities.ToscaTargetFuncOrConstraintParameter;
import ru.crp.lvms.optimizations.tosca.entities.ToscaTargetFunction;
import ru.crp.lvms.optimizations.tosca.enums.NodeOrElem;
import ru.crp.lvms.optimizations.tosca.enums.ToscaBaseSettingType;
import ru.crp.lvms.optimizations.tosca.enums.ToscaSettingsType;
import ru.crp.lvms.optimizations.tosca.enums.ToscaTargetFunctionType;
import ru.crp.lvms.optimizations.tosca.managers.ToscaSuggestManager;
import ru.crp.lvms.utils.URLs;

import java.util.List;
import java.util.Map;

import static java.util.Collections.emptyList;

@RestController
@RequestMapping(value = URLs.TOSCA_TARGET_FUNCS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class ToscaTargetFunctionsController extends ToscaAbstractOptimizationSettingsController<ToscaTargetFunction> {

    @Override
    protected ToscaSettingsType getSettingsType() {
        return ToscaSettingsType.TargetFunc;
    }


    @Override
    protected ToscaBaseSettingType getToskaOptimizationBaseSettingType() {
        return ToscaBaseSettingType.NUM_TARGET_FUNCS;
    }

    @Autowired
    private ToscaSuggestManager toscaSuggestManager;

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/parameters", method = RequestMethod.GET)
    public Map<String, List<ToscaTargetFuncOrConstraintParameter>> getAvailableTFParameters(@PathVariable Integer optimizationId) {
        return toscaSuggestManager.getAvailableParameters(optimizationId, true);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/parameters", method = RequestMethod.GET, params = {"group"})
    public List<ToscaTargetFuncOrConstraintParameter> getAvailableTFParametersForGroup(@PathVariable Integer optimizationId,
                                                                                       @RequestParam("group") String groupName) {
        return toscaSuggestManager.getAvailableParameters(optimizationId, true).getOrDefault(groupName, emptyList());
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/parameters/groups", method = RequestMethod.GET)
    public List<String> getAvailableTFParametersGroups(@PathVariable Integer optimizationId) {
        return toscaSuggestManager.getAvailableParametersGroups(optimizationId, true);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/types", method = RequestMethod.GET)
    public ToscaTargetFunctionType[] getAllAvailableTFTypes() {
        return ToscaTargetFunctionType.values();
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/nodeOrElement", method = RequestMethod.GET)
    public NodeOrElem[] getAllAvailableNoes() {
        return NodeOrElem.values();
    }
}
