package ru.crp.lvms.optimizations.tosca.dao;

import org.springframework.stereotype.Repository;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettignsDao;
import ru.crp.lvms.optimizations.tosca.entities.ToscaSet;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

@Repository
public class ToscaOptimizationSetsDao extends AbstractOptimizationSettignsDao<ToscaSet> {

    public List<ToscaSet> findSelectedSets(Integer optimizationId, @Nullable Integer setTypeId) {
        if (setTypeId == null) {
            return find("WHERE optimization_id = ? AND is_selected = TRUE ORDER BY name", optimizationId);
        } else {
            return find("WHERE optimization_id = ? AND is_selected = TRUE AND set_type_id = ? ORDER BY name", optimizationId, setTypeId);
        }
    }

    public void deleteAllForOptimization(Integer optimizationId) {
        jdbcTemplate().update("DELETE FROM " + getTableName() + " WHERE optimization_id = ?", optimizationId);
    }

    public void addSetsForOptimization(Integer optimizationId, List<ToscaSet> newSets) {
        List<Object[]> paramsInsert = new ArrayList<>();
        for (ToscaSet set : newSets) {
            paramsInsert.add(new Object[]{optimizationId, set.getName(), set.getSetTypeId(), set.getIsSelected(), set.getIsOptimizationSpace()});
        }
        if (!paramsInsert.isEmpty()) {
            getJdbcTemplate().batchUpdate(
                    "INSERT INTO " + getTableName() + "(optimization_id, name, set_type_id, is_selected, is_optimization_space) " +
                            "VALUES (?, ?, ?, ?, ?)",
                    paramsInsert);
        }
    }
}
