package ru.crp.lvms.optimizations.tosca.dao;

import org.springframework.stereotype.Repository;
import ru.crp.cmlb.model.utils.EntityUtils;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

@Repository
public class ToscaTechnologicalConstraintTreeDao extends BaseSpaceDao<TechnologicalConstraintTreeNode> {

    public ToscaTechnologicalConstraintTreeDao() {
        setTableName("toska_technological_constraints_tree_extended");
    }

    public List<TechnologicalConstraintTreeNode> getFullTreeForTC(Integer constraintId) {
        return find("WHERE tc_id = ? ORDER BY id", constraintId);
    }

    public List<TechnologicalConstraintTreeNode> getTCTree(Integer constraintId, @Nullable Integer parentId) {
        if (parentId == null) {
            return find("WHERE tc_id = ? AND parent_id IS NULL", constraintId);
        } else {
            return find("WHERE tc_id = ? AND parent_id = ?", constraintId, parentId);
        }
    }

    public boolean hasAncestors(int initialNodeId, List<Integer> ancestorIds) {
        String INSql = EntityUtils.sqlForListParams(ancestorIds);
        List<Object> params = new ArrayList<>(ancestorIds);
        params.add(0, initialNodeId);
        return getJdbcTemplate().queryForObject("WITH RECURSIVE recs(id, parent_id) AS " +
                        "(SELECT r.id, r.parent_id FROM toska_technological_constraints_tree r WHERE r.id = ? " +
                        "UNION ALL SELECT r2.id, r2.parent_id FROM toska_technological_constraints_tree r2, recs AS r WHERE r2.id = r.parent_id) " +
                        "SELECT EXISTS(SELECT id FROM recs WHERE id IN " + INSql + ");",
                Boolean.class, params.toArray());
    }
}
