package ru.crp.lvms.optimizations.tosca.entities;

import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettings;
import ru.crp.lvms.optimizations.tosca.enums.NodeOrElem;
import ru.crp.lvms.optimizations.tosca.enums.ToscaConstraintType;

@Table("toska_target_functions_join")
public class ToscaTargetFunction extends AbstractOptimizationSettings {

    @Column("parameter_id")
    private Integer parameterId;

    @Column("type_id")
    private Integer typeId;

    @Column("scalar")
    private String scalar;

    @Column("el_group")
    private Integer elementsGroupId;

    @Column("nd_group")
    private Integer nodesGroupId;

    @Column("param_name")
    private String paramName;

    @Column("dresp_type")
    private String drespType;

    @Column("node_or_element_id")
    private Integer nodeOrElementId;

    private boolean isElementGroupAllowed;

    private boolean isNodesGroupAllowed;

    private boolean isScalarAllowed;


    public ToscaTargetFunction() {
        typeId = ToscaConstraintType.EQUALS.getId();
    }

    public ToscaTargetFunction(int newOptimizationId, int defaultParameterId, String name) {
        typeId = ToscaConstraintType.EQUALS.getId();
        setOptimizationId(newOptimizationId);
        setParameterId(defaultParameterId);
        setName(name);
    }

    public String getParamName() {
        return paramName;
    }

    public void setParamName(String paramName) {
        this.paramName = paramName;
    }

    public String getDrespType() {
        return drespType;
    }

    public void setDrespType(String drespType) {
        this.drespType = drespType;
    }

    public Integer getNodeOrElementId() {
        return nodeOrElementId;
    }

    public void setNodeOrElementId(Integer nodeOrElementId) {
        this.nodeOrElementId = nodeOrElementId;
        if (nodeOrElementId != null) {
            switch (NodeOrElem.valueOf(nodeOrElementId)) {
                case ELEMENT:
                    isElementGroupAllowed = true;
                    isNodesGroupAllowed = false;
                    isScalarAllowed = false;
                    break;
                case NODE:
                    isElementGroupAllowed = false;
                    isNodesGroupAllowed = true;
                    isScalarAllowed = false;
                    break;
                case NODE_AND_ELEMENT:
                case NODE_OR_ELEM:
                    isElementGroupAllowed = true;
                    isNodesGroupAllowed = true;
                    isScalarAllowed = false;
                    break;
                case SCALAR:
                    isElementGroupAllowed = false;
                    isNodesGroupAllowed = false;
                    isScalarAllowed = true;
                    break;
                case NONE:
                default:
                    isElementGroupAllowed = false;
                    isNodesGroupAllowed = false;
                    isScalarAllowed = false;
            }
        }
    }


    public Integer getTypeId() {
        return typeId;
    }

    public void setTypeId(Integer typeId) {
        this.typeId = typeId;
    }

    public String getScalar() {
        return scalar;
    }

    public void setScalar(String scalar) {
        this.scalar = scalar;
    }

    public Integer getElementsGroupId() {
        return elementsGroupId;
    }

    public void setElementsGroupId(Integer elementsGroupId) {
        this.elementsGroupId = elementsGroupId;
    }

    public Integer getNodesGroupId() {
        return nodesGroupId;
    }

    public void setNodesGroupId(Integer nodesGroupId) {
        this.nodesGroupId = nodesGroupId;
    }

    public Integer getParameterId() {
        return parameterId;
    }

    public void setParameterId(Integer parameterId) {
        this.parameterId = parameterId;
    }

    public boolean getIsElementGroupAllowed() {
        return isElementGroupAllowed;
    }

    public boolean getIsNodesGroupAllowed() {
        return isNodesGroupAllowed;
    }

    public boolean getIsScalarAllowed() {
        return isScalarAllowed;
    }
}
