package ru.crp.lvms.optimizations.tosca.enums;

import com.fasterxml.jackson.annotation.JsonFormat;

import javax.annotation.Nonnull;
import java.util.List;

import static java.util.Arrays.asList;

@JsonFormat(shape = JsonFormat.Shape.OBJECT)
public enum SetType {
    Elements(1),
    Nodes(2);

    public static final List<String> ELEMENTS_SET_NAMES = asList("ELEM", "ELEMENT");

    public static final String NODES_SET_NAME = "NODE";
    private int id;

    SetType(int id) {
        this.id = id;
    }

    @Nonnull
    public static SetType valueOf(int id) {
        for (SetType cur : values()) {
            if (cur.getId() == id) {
                return cur;
            }
        }
        throw new IllegalArgumentException("Enum value for id " + id + " is not found");
    }

    public int getId() {
        return id;
    }

    public String getName() {
        return this.toString();
    }

    public static SetType fromCAOFileValue(String name) {
        name = name.trim();
        if (ELEMENTS_SET_NAMES.contains(name)) {
            return Elements;
        }
        if (NODES_SET_NAME.equals(name)) {
            return Nodes;
        }
        throw new IllegalArgumentException(String.format("Unexpected set name: %s", name));
    }
}
