package ru.crp.lvms.optimizations.tosca.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.lvms.optimizations.base.dao.OptimizationDao;
import ru.crp.lvms.optimizations.base.dao.OptimizationJsonSettingsDao;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.base.entities.OptimizationJsonSetting;
import ru.crp.lvms.optimizations.base.entities.OptimizationListSetting;
import ru.crp.lvms.optimizations.base.enums.OptimizationTypes;
import ru.crp.lvms.optimizations.tosca.enums.ToscaJsonParametersParents;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;

import static java.util.Arrays.asList;
import static ru.crp.lvms.optimizations.base.entities.OptimizationJsonSetting.*;

@Component
public class ToscaDefaultParamsCreator {

    public static final String MULTIPLE_VALUES_SEPARATOR = ";";
    public static final String ANIMATION_NAME = "ANIMATION";
    public static final String SMOOTHING_NAME = "SMOOTHING";
    public static final String FORMAT_SETTING_XML_NAME = "FORMAT";

    @Autowired
    private OptimizationJsonSettingsDao optimizationJsonSettingsDao;

    @Autowired
    private OptimizationDao optimizationDao;

    public void addAdditionalOptimizationParameters(OptimizationEntity newOptimization) {
        int optId = newOptimization.getId();

        List<OptimizationJsonSetting> settings = new ArrayList<>();

        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoController.getId(), "Скорость оптимизации", "SPEED",
                asList(new OptimizationListSetting(false, "Очень медленно", "VERY_SLOW"),
                        new OptimizationListSetting(false, "Медленно", "SLOW"),
                        new OptimizationListSetting(false, "Умеренно", "MODERATE"),
                        new OptimizationListSetting(false, "Средне", "MEDIUM"),
                        new OptimizationListSetting(false, "Быстро", "FAST"),
                        new OptimizationListSetting(true, "Фиксированное количество итераций", "ITER", true, "15"))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoController.getId(), "Начальное изменение плотности", "START_DELETE",
                asList(new OptimizationListSetting(true, "Относительная величина", "PERC", true, null),
                        new OptimizationListSetting(false, "Абсолютная величина", "ABS", true, null))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoController.getId(), "Автоматически замораживать", "AUTO_FROZEN",
                asList(new OptimizationListSetting(true, "Элементы с нагруженными узлами", "LOAD"),
                        new OptimizationListSetting(false, "Элементы с фиксированными узлами", "SPC"),
                        new OptimizationListSetting(false, "Элементы со всеми типами граничных условий", "BOTH"),
                        new OptimizationListSetting(false, "Не замораживать", "OFF"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoController.getId(), "Штрафной фактор", "MAT_PENALTY", "3"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoController.getId(), "Максимальное количество итераций", "ITER_MAX", "1000"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoController.getId(), "Отсечка плотности", "ISO_VALUE", "0.1"));


        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Максимальное количество итераций", "ITER_MAX", "1000"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Минимальное количество итераций", "STOP_CRITERION_ITER", "4"));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Изменение плотности", "DENSITY_UPDATE",
                asList(new OptimizationListSetting(true, "Нормальное", "NORMAL"),
                        new OptimizationListSetting(false, "Консервативное", "CONSERVATIVE"),
                        new OptimizationListSetting(false, "Агрессивное", "AGGRESSIVE"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Начальное значение плотности", "DENSITY_INITIAL", "0.5"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Верхний предел плотности", "DENSITY_UPPER", "1"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Нижний предел плотности", "DENSITY_LOWER", "0.001"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Максимальное значение шага изменения плотности", "DENSITY_MOVE", "0.25"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Радиус фильтрации", "FILTER_RADIUS", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Фильтрация", "FILTER_TYPE",
                asList(new OptimizationListSetting(true, "Стандартная", "STANDART"),
                        new OptimizationListSetting(false, "Слабая", "LOW"))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Интерполяция материала", "MAT_INTERPOLATION",
                asList(new OptimizationListSetting(true, "SIMP", "SIMP"),
                        new OptimizationListSetting(false, "RAMP", "RAMP"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Штрафной фактор", "MAT_PENALTY", "3"));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Многопроцессорность", "PROCESSORS",
                asList(new OptimizationListSetting(true, "SINGLE", "SINGLE"),
                        new OptimizationListSetting(false, "MULTI", "MULTI"))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Сходимость", "STOP_CRITERION_LEVEL",
                asList(new OptimizationListSetting(false, "Один критерий", "SINGLE"),
                        new OptimizationListSetting(true, "Два критерия", "BOTH"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Критерий сходимости целевой функции", "STOP_CRITERION_OBJ", "0.001"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Критерий сходимость по плотности", "STOP_CRITERION_DENSITY", "0.005"));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Метод исключения слабых элементов", "SOFT_DELETE_METHOD",
                asList(new OptimizationListSetting(true, "Стандартный", "STANDART", true, "0.05"),
                        new OptimizationListSetting(false, "Агрессивный", "AGGRESSIVE", true, null),
                        new OptimizationListSetting(false, "Максимальные сдвиговые деформации", "MAX_SHEAR_STRAIN", true, null),
                        new OptimizationListSetting(false, "Минимальные главные напряжения", "MIN_PRINCIPAL_STRAIN", true, null),
                        new OptimizationListSetting(false, "Пластичность", "PLASTICITY", true, null),
                        new OptimizationListSetting(false, "Изменение объёма", "VOLUME_CHANGE", true, null))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Группа исключения слабых элементов", "SOFT_DELETE", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Автоматически замораживать", "AUTO_FROZEN",
                asList(new OptimizationListSetting(true, "Элементы с нагруженными узлами", "LOAD"),
                        new OptimizationListSetting(false, "Элементы с фиксированными узлами", "SPC"),
                        new OptimizationListSetting(false, "Элементы со всеми типами граничных условий", "BOTH"),
                        new OptimizationListSetting(false, "Не замораживать", "OFF"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Отсечка плотности", "ISO_VALUE", "0.1"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.TopoSensitivity.getId(), "Сглаживание кривой", "CURV_SMOOTH", "5"));

        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Максимальное количество итераций", "ITER_MAX", "1000"));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Изменение вектора смещения", "VECTOR",
                asList(new OptimizationListSetting(true, "Первый шаг", "FIRST"),
                        new OptimizationListSetting(false, "Каждый шаг", "EVER"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Направление оптимизации", "SCALE", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Ширина рельефа", "BEAD_WIDTH", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Количество итераций", "BEAD_ITER", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Не оптимизировать, если напряжение по Мизесу меньше", "BEAD_MIN_STRESS", "0.001"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Не оптимизировать, если мембранное напряжение меньше изгибногого", "BEAD_MAX_MEMBRANE", "1"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Сглаживание кривизны", "CURV_SMOOTH", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Сглаживание узлов", "BEAD_NODE_SMOOTH", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadController.getId(), "Отсечка плотности", "ISO_VALUE", "0.1"));

        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "Максимальное количество итераций", "ITER_MAX", "1000"));
        settings.add(newCompositeSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "Радиус фильтрации", "FILTER_RADIUS",
                asList(newPlainSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "", "", "4.0"),
                        newListSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "", "",
                                asList(new OptimizationListSetting(true, "REL", "REL"),
                                        new OptimizationListSetting(false, "ABS", "ABS")))
                )));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "Только пространство оптимизации", "ONLY_DES_NODES",
                asList(new OptimizationListSetting(true, "да", "YES"),
                        new OptimizationListSetting(false, "нет", "NO"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "Максимальное смещение за 1 шаг", "NODAL_MOVE", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "Стратегия оптимизации", "NODAL_UPDATE",
                asList(new OptimizationListSetting(false, "консервативная", "CONSERVATIVE"),
                        new OptimizationListSetting(true, "нормальная", "NORMAL"),
                        new OptimizationListSetting(false, "агрессивная", "AGGRESSIVE"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "Пробный шаг (относ. размера элемента)", "PERTURBATION", "0.0001"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.BeadSensitivity.getId(), "Отсечка плотности", "ISO_VALUE", "0.1"));

        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Максимальное количество итераций", "ITER_MAX", "1000"));
        settings.add(newCompositeSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Масштаб", "SCALE",
                asList(newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Расширение", "Расширение", "1"),
                        newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Сужение", "Сужение", "1"))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Срединные узлы", "MID_NODES",
                asList(new OptimizationListSetting(true, "Линейно", "LINEAR"),
                        new OptimizationListSetting(false, "Интерполяция", "INTERPOLATE"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Сглаживание кривой", "CURV_SMOOTH", "5"));
        settings.add(newCompositeSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Фильтр", "FILTER",
                asList(newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "радиус", "радиус", null),
                        newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "отклонение", "отклонение", null),
                        newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "экспонента", "экспонента", null))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Приращение вектора", "VECTOR",
                asList(new OptimizationListSetting(true, "Первый шаг", "FIRST"),
                        new OptimizationListSetting(false, "Каждый шаг", "EVER"))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Смещение", "DISP",
                asList(new OptimizationListSetting(true, "минимальное", "MINIMUM"),
                        new OptimizationListSetting(false, "среднее", "AVERAGE"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Точность", "TOL_VAL", "0.001"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.ShapeController.getId(), "Отсечка плотности", "ISO_VALUE", "0.1"));

        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Тип постпроцессинга", "TASK",
                asList(new OptimizationListSetting(false, "Изоповерхность", "ISO"),
                        new OptimizationListSetting(false, "Поверхность", "SURFACE"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Отсечка плотности", "ISO_VALUE", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Целевой объем", "TARGET_VOLUME", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Количество циклов", "SMOOTH_CYCLES", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Уменьшение точности", "REDUCTION_RATE", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Удаление углов", "REDUCTION_ANGLE", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Проверять самопересечение", "SELF_INTERSECTION_CHECK",
                asList(new OptimizationListSetting(false, "не проверять", "OFF"),
                        new OptimizationListSetting(false, "проверить один раз", "CHECK"),
                        new OptimizationListSetting(false, "проверять всегда", "RUNTIME"),
                        new OptimizationListSetting(false, "проверять при необходимости", "ITERATIVE"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Минимальный угол элемета", "MIN_ANGLE", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Смешанная сетка", "MIXED_MESH",
                asList(new OptimizationListSetting(false, "Нет", "NO"),
                        new OptimizationListSetting(false, "Да", "YES"))));


        settings.add(newCompositeSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Формат", FORMAT_SETTING_XML_NAME,
                asList(
                        newBoolSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "ABAQUS(inp,iges)", "inp" + MULTIPLE_VALUES_SEPARATOR + "iges", false),
                        newBoolSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "ANSYS(cdb,iges)", "cdb" + MULTIPLE_VALUES_SEPARATOR + "iges3", false),
                        newBoolSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "CATIA(stl,iges)", "stl" + MULTIPLE_VALUES_SEPARATOR + "iges3", false),
                        newBoolSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "MSC NASTRAN(bdf)", "bdf", false),
                        newBoolSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "PRO/E(iges)", "iges2", false),
                        newBoolSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "NX(stl)", "stl", false),
                        newBoolSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Solid Works(iges)", "iges2", false))
        ));

        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Результирующая поверхность", "OUTPUT_SURFACE",
                asList(new OptimizationListSetting(false, "исходная", "ORIGINAL"),
                        new OptimizationListSetting(false, "новая", "NEW"),
                        new OptimizationListSetting(false, "обе", "BOTH"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Количество слоев", "SLICE_NUMBER", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Нормаль слоя", "SLICE_NORMAL", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Формат слоя", "SLICE_FORMAT",
                asList(new OptimizationListSetting(false, "Многоугольник", "IGS_POLYGON"),
                        new OptimizationListSetting(false, "Сплайн", "IGS_CURVES"),
                        new OptimizationListSetting(false, "Общий формат", "CLI"),
                        new OptimizationListSetting(false, "Все форматы", "ALL"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Исключить группу", "GROUP_EXCLUDE", null));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Использовать группу", "GROUP_INCLUDE", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Отсечка плотности", "ISO_SEQUENCE",
                asList(new OptimizationListSetting(false, "Постоянная", "CONST"),
                        new OptimizationListSetting(false, "Линейная", "INCR"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Исключить нагруженные элементы", "GROUP_SURFACE", null));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Сглаживание исходной поверхности", "ORIGINAL_SURFACE_SMOOTHING",
                asList(new OptimizationListSetting(false, "Нет", "OFF"),
                        new OptimizationListSetting(false, "Внутреннее", "SHRINK"),
                        new OptimizationListSetting(false, "Да", "FULL"))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Фильтрация результатов", "RESULT_FILTERING",
                asList(new OptimizationListSetting(false, "Нет", "OFF"),
                        new OptimizationListSetting(false, "Средняя", "MODERATE"),
                        new OptimizationListSetting(false, "Да", "FULL"))));
        settings.add(newListSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Сохранять связанные узлы", "KEEP_CONNECTION_NODES",
                asList(new OptimizationListSetting(false, "Нет", "NO"),
                        new OptimizationListSetting(false, "Да", "YES"))));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.PostprocessingParameters.getId(), "Минимальный размер соединения", "COMPONENT_LIMITING_SIZE", null));


        settings.add(newBoolSetting(optId, ToscaJsonParametersParents.AnimationParameters.getId(), "Сглаженная форма", SMOOTHING_NAME, true));
        settings.add(newBoolSetting(optId, ToscaJsonParametersParents.AnimationParameters.getId(), "Анимация результатов", ANIMATION_NAME, true));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.AnimationParameters.getId(), "Вид OX", "vx", "1"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.AnimationParameters.getId(), "Вид OY", "vy", "1"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.AnimationParameters.getId(), "Вид OZ", "vz", "1"));
        settings.add(newPlainSetting(optId, ToscaJsonParametersParents.AnimationParameters.getId(), "Отсечка плотности", "smooth", "0.05"));


        settings.forEach(optimizationJsonSettingsDao::createSettings);
        OptimizationTypes optType = OptimizationTypes.valueOf(newOptimization.getOptimizationTypeId());
        Stream.of(ToscaJsonParametersParents.values())
                .filter(optMethod -> optMethod.isMethodAvailableForOptimizationType(optType))
                .findAny()
                .ifPresent(tom -> optimizationDao.selectOptimizationMethod(newOptimization.getId(), tom.getId()));
    }
}
