package ru.crp.lvms.optimizations.tosca.managers;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.crp.lvms.managers.misc.BaseManager;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.optimizations.tosca.dao.ToscaOptimizationTechnologicalConstraintDao;
import ru.crp.lvms.optimizations.tosca.dao.ToscaTechnologicalConstraintDao;
import ru.crp.lvms.optimizations.tosca.dao.ToscaTechnologicalConstraintTreeDao;
import ru.crp.lvms.optimizations.tosca.dao.ToscaTechnologicalConstraintTreeValueDao;
import ru.crp.lvms.optimizations.base.entities.tree.DependenciesDescription;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.tosca.entities.ToscaTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static java.util.Arrays.asList;
import static java.util.Collections.singletonList;
import static ru.crp.lvms.optimizations.base.entities.tree.ConditionType.ANY_SELECTED;
import static ru.crp.lvms.optimizations.base.entities.tree.ConditionType.NONE_SELECTED;

@Service
public class ToscaOptimizationTechnologicalConstraintManager extends BaseManager<ToscaOptimizationTechnologicalConstraint> {

    private Map<Integer, DependenciesDescription> dependenciesConfig = new HashMap<>();

    @Autowired
    private ToscaOptimizationTechnologicalConstraintDao toscaOptimizationTechnologicalConstraintDao;

    @Autowired
    private ToscaTechnologicalConstraintTreeValueDao toscaTechnologicalConstraintTreeValueDao;

    @Autowired
    private ToscaTechnologicalConstraintDao toscaTechnologicalConstraintDao;

    @Autowired
    private ToscaTechnologicalConstraintTreeDao toscaTechnologicalConstraintTreeDao;

    @PostConstruct
    public void init() {
        dependenciesConfig.put(83, new DependenciesDescription(asList(81, 78), ANY_SELECTED));
        dependenciesConfig.put(82, new DependenciesDescription(asList(81, 78), ANY_SELECTED));
        // DEMOLD_DIR select dependencies
        dependenciesConfig.put(169, new DependenciesDescription(singletonList(98), ANY_SELECTED));
        dependenciesConfig.put(122, new DependenciesDescription(singletonList(98), ANY_SELECTED));
        // CLIENT_DIR select dependencies
        dependenciesConfig.put(108, new DependenciesDescription(singletonList(94), ANY_SELECTED));
        dependenciesConfig.put(109, new DependenciesDescription(singletonList(94), ANY_SELECTED));
        dependenciesConfig.put(111, new DependenciesDescription(singletonList(94), ANY_SELECTED));
        dependenciesConfig.put(112, new DependenciesDescription(singletonList(94), ANY_SELECTED));
        dependenciesConfig.put(117, new DependenciesDescription(singletonList(94), ANY_SELECTED));
        dependenciesConfig.put(135, new DependenciesDescription(singletonList(94), ANY_SELECTED));
        dependenciesConfig.put(146, new DependenciesDescription(singletonList(94), ANY_SELECTED));
        dependenciesConfig.put(173, new DependenciesDescription(singletonList(94), ANY_SELECTED));

        // none of DEMOLD_DIR and CLIENT_DIR select dependencies
        dependenciesConfig.put(175, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(176, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(177, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(178, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(179, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(181, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(183, new DependenciesDescription(asList(94, 98), NONE_SELECTED));

        dependenciesConfig.put(185, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(186, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(187, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(188, new DependenciesDescription(asList(94, 98), NONE_SELECTED));

        dependenciesConfig.put(189, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(190, new DependenciesDescription(asList(94, 98), NONE_SELECTED));

        dependenciesConfig.put(196, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(197, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(198, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(199, new DependenciesDescription(asList(94, 98), NONE_SELECTED));

        dependenciesConfig.put(205, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(206, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(207, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(208, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(209, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(210, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(211, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(212, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(213, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(214, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(215, new DependenciesDescription(asList(94, 98), NONE_SELECTED));
        dependenciesConfig.put(216, new DependenciesDescription(asList(94, 98), NONE_SELECTED));

        // CS
        dependenciesConfig.put(102, new DependenciesDescription(asList(94, 98), ANY_SELECTED));
        // TOL
        dependenciesConfig.put(103, new DependenciesDescription(asList(94, 98), ANY_SELECTED));
        dependenciesConfig.put(104, new DependenciesDescription(asList(94, 98), ANY_SELECTED));
        dependenciesConfig.put(105, new DependenciesDescription(asList(94, 98), ANY_SELECTED));
        dependenciesConfig.put(106, new DependenciesDescription(asList(94, 98), ANY_SELECTED));
        // UNDERCUT_TOL
        dependenciesConfig.put(171, new DependenciesDescription(asList(122, 146, 169), ANY_SELECTED));
        // UNGLE
        dependenciesConfig.put(170, new DependenciesDescription(asList(122, 146, 169, 109, 112, 117), ANY_SELECTED));

        // CYCLIC_SYM_START
        dependenciesConfig.put(123, new DependenciesDescription(asList(112, 117), ANY_SELECTED));
        dependenciesConfig.put(119, new DependenciesDescription(asList(112, 117), ANY_SELECTED));
        dependenciesConfig.put(120, new DependenciesDescription(asList(112, 117), ANY_SELECTED));
        dependenciesConfig.put(121, new DependenciesDescription(asList(112, 117), ANY_SELECTED));

        // CHECK_GROUP
        dependenciesConfig.put(172, new DependenciesDescription(singletonList(169), ANY_SELECTED));

        // SURF_PARAM
        dependenciesConfig.put(125, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));
        dependenciesConfig.put(126, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));
        dependenciesConfig.put(127, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));

        // SURF_SYM_TYPE
        dependenciesConfig.put(128, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));
        dependenciesConfig.put(129, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));
        dependenciesConfig.put(130, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));
        dependenciesConfig.put(131, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));
        dependenciesConfig.put(132, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));

        // SURF_SYM_CS
        dependenciesConfig.put(133, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));

        // SURF_SYM_NDGR
        dependenciesConfig.put(134, new DependenciesDescription(asList(122, 135, 146), ANY_SELECTED));

    }

    @Override
    protected BaseSpaceDao<ToscaOptimizationTechnologicalConstraint> aliveDao() {
        return toscaOptimizationTechnologicalConstraintDao;
    }

    public List<ToscaOptimizationTechnologicalConstraint> findOptimizationTCs(int optimizationId) {
        return toscaOptimizationTechnologicalConstraintDao.findForOptimization(optimizationId);
    }

    public List<TechnologicalConstraintTreeNode> getFullTreeOfTC(Integer constraintId) {
        return setDependencies(toscaTechnologicalConstraintTreeDao.getFullTreeForTC(constraintId));
    }


    public List<TechnologicalConstraintTreeNode> getTCTree(Integer constraintId, @Nullable Integer parentId) {
        return setDependencies(toscaTechnologicalConstraintTreeDao.getTCTree(constraintId, parentId));
    }

    @Nonnull
    @Override
    public ToscaOptimizationTechnologicalConstraint updateAndRead(ToscaOptimizationTechnologicalConstraint entity, @Nullable UserEntity currentUser) {
        ToscaOptimizationTechnologicalConstraint current = findOne(entity.getId(), currentUser);
        ToscaOptimizationTechnologicalConstraint updated = super.updateAndRead(entity, currentUser);
        if (!Objects.equals(current.getTechnologicalConstraintId(), updated.getTechnologicalConstraintId())) {
            toscaTechnologicalConstraintTreeValueDao.clearSavedValues(updated.getId());
        }
        return updated;
    }

    public List<ToscaTechnologicalConstraint> findAllToscaTechnologicalConstraint(Integer optimizationTypeId, UserEntity user) {
        return toscaTechnologicalConstraintDao.findAllToscaTechnologicalConstraint(optimizationTypeId);
    }

    public TechnologicalConstraintTreeNode getField(Integer fieldId) {
        TechnologicalConstraintTreeNode field = toscaTechnologicalConstraintTreeDao.findOne(fieldId);
        if (field != null) {
            setDependencies(singletonList(field));
        }
        return field;
    }

    public boolean hasAncestors(int initialNodeId, List<Integer> ancestorIds) {
        return toscaTechnologicalConstraintTreeDao.hasAncestors(initialNodeId, ancestorIds);
    }


    public <T extends TechnologicalConstraintTreeNode> List<T> setDependencies(List<T> treeNodes) {
        treeNodes.forEach(node -> node.setDependencies(dependenciesConfig.getOrDefault(node.getId(), null)));
        return treeNodes;
    }
}
