package ru.crp.lvms.tasks;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.cmlb.model.entities.FileData;
import ru.crp.cmlb.model.entities.ResourcePreparationException;
import ru.crp.cmls.calculations.solvers.xml.config.DataType;
import ru.crp.lvms.managers.SimulationManager;
import ru.crp.lvms.managers.solvers.SolverConfigManager;
import ru.crp.lvms.model.entities.simulations.SimulationEntity;
import ru.crp.lvms.model.entities.simulations.SimulationType;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.fileGenerators.ECInputFileGenerator;
import ru.crp.lvms.tasks.fileGenerators.OptimizationInputFileGenerator;
import ru.crp.lvms.tasks.fileGenerators.SimulationInputFileGenerator;

import java.io.IOException;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

@Component
public class TaskFilesCollector {
    @Autowired
    private SolverConfigManager solverConfigManager;

    @Autowired
    private ECInputFileGenerator ecInputFileGenerator;

    @Autowired
    private SimulationManager simulationManager;

    @Autowired
    private SimulationInputFileGenerator simulationInputFileGenerator;

    @Autowired
    private OptimizationInputFileGenerator optimizationInputFileGenerator;

    public List<FileData> collectTaskFiles(TaskEntity task) throws IOException {
        List<FileData> taskFiles = new ArrayList<>();
        Path solverFile = solverConfigManager.getSolverFile(task.getSolverName());
        if (solverFile == null) {
            throw new ResourcePreparationException("Can't find config file for solver: " + task.getSolverName());
        }
        taskFiles.add(FileData.fromFile(task.getSolverName(), DataType.SOLVER_CONFIG, solverFile.toFile()));
        switch (task.getParentType()) {
            case effectiveCharacteristic:
                taskFiles.addAll(ecInputFileGenerator.getInputFilesForEffectiveCharacteristics(task));
                break;
            case simulation:
                SimulationEntity parent = simulationManager.findOne(task.getParentId(), null);
                switch (SimulationType.valueOf(parent.getTypeId())) {
                    case Extension:
                        taskFiles.addAll(simulationInputFileGenerator.getInputFilesForTensionSimulation(task));
                        break;
                    case Bend:
                        taskFiles.addAll(simulationInputFileGenerator.getInputFilesForBendSimulation(task));
                        break;
                    case Compression:
                        taskFiles.addAll(simulationInputFileGenerator.getInputFilesForCompressionSimulation(task));
                        break;
                    case ThermalExpansion:
                        // intentionally same action
                    case ThermalConductivityCoeff:
                        taskFiles.addAll(simulationInputFileGenerator.getInputFilesForThermalSimulation(task));
                        break;
                    default:
                        throw new IllegalArgumentException("No file generator for simulation type: " +
                                SimulationType.valueOf(parent.getTypeId()));
                }
                break;
            case optimization:
                taskFiles.addAll(optimizationInputFileGenerator.getInputFilesForOptimization(task));
                break;
            default:
                throw new IllegalArgumentException("Unknown parent type: " + task.getParentType());
        }
        return taskFiles;
    }
}
