package ru.crp.lvms.tasks.fileGenerators;

import org.apache.commons.lang3.StringUtils;

public class ECParameters {
    private String matrixElacticityModulus;
    private String matrixPoissonRatio;
    private String matrixStrengthLimit;

    private String fiberElacticityModulus;
    private String fiberPoissonRatio;
    private String fiberStrengthLimit;

    private String epoxideElacticityModulus;
    private String epoxidePoissonRatio;
    private String epoxideStrengthLimit;

    private String averageLayerThickness;

    private String fiberConcentration;
    private String fiberDiameter;

    private String valueA;
    private String valueB;
    private String valueL;


    public String getMatrixElacticityModulus() {
        return matrixElacticityModulus;
    }

    public void setMatrixElacticityModulus(String matrixElacticityModulus) {
        this.matrixElacticityModulus = matrixElacticityModulus;
    }

    public String getMatrixPoissonRatio() {
        return matrixPoissonRatio;
    }

    public void setMatrixPoissonRatio(String matrixPoissonRatio) {
        this.matrixPoissonRatio = matrixPoissonRatio;
    }

    public String getFiberElacticityModulus() {
        return fiberElacticityModulus;
    }

    public void setFiberElacticityModulus(String fiberElacticityModulus) {
        this.fiberElacticityModulus = fiberElacticityModulus;
    }

    public String getFiberPoissonRatio() {
        return fiberPoissonRatio;
    }

    public void setFiberPoissonRatio(String fiberPoissonRatio) {
        this.fiberPoissonRatio = fiberPoissonRatio;
    }

    public String getFiberConcentration() {
        return fiberConcentration;
    }

    public void setFiberConcentration(String fiberConcentration) {
        this.fiberConcentration = fiberConcentration;
    }

    public String getFiberDiameter() {
        return fiberDiameter;
    }

    public void setFiberDiameter(String fiberDiameter) {
        this.fiberDiameter = fiberDiameter;
    }

    public boolean isAllParametersPresent() {
        return isAllMatrixParametersPresent() && isAllFiberParametersPresent() && isAllECParametersPresent();
    }

    public boolean isAllMatrixParametersPresent() {
        return !StringUtils.trimToEmpty(matrixElacticityModulus).isEmpty() &&
                !StringUtils.trimToEmpty(matrixPoissonRatio).isEmpty() &&
                !StringUtils.trimToEmpty(matrixStrengthLimit).isEmpty();
    }

    public boolean isAllFiberParametersPresent() {
        return !StringUtils.trimToEmpty(fiberElacticityModulus).isEmpty() &&
                !StringUtils.trimToEmpty(fiberPoissonRatio).isEmpty() &&
                !StringUtils.trimToEmpty(fiberStrengthLimit).isEmpty();
    }

    public boolean isAllEpoxideParametersPresent() {
        return !StringUtils.trimToEmpty(epoxideElacticityModulus).isEmpty() &&
                !StringUtils.trimToEmpty(epoxidePoissonRatio).isEmpty() &&
                !StringUtils.trimToEmpty(epoxideStrengthLimit).isEmpty();
    }

    public boolean isAllECParametersPresent() {
        return !StringUtils.trimToEmpty(fiberConcentration).isEmpty() &&
                !StringUtils.trimToEmpty(fiberDiameter).isEmpty();
    }

    public String getMatrixStrengthLimit() {
        return matrixStrengthLimit;
    }

    public void setMatrixStrengthLimit(String matrixStrengthLimit) {
        this.matrixStrengthLimit = matrixStrengthLimit;
    }

    public String getFiberStrengthLimit() {
        return fiberStrengthLimit;
    }

    public void setFiberStrengthLimit(String fiberStrengthLimit) {
        this.fiberStrengthLimit = fiberStrengthLimit;
    }

    public String getEpoxideElacticityModulus() {
        return epoxideElacticityModulus;
    }

    public void setEpoxideElacticityModulus(String epoxideElacticityModulus) {
        this.epoxideElacticityModulus = epoxideElacticityModulus;
    }

    public String getEpoxidePoissonRatio() {
        return epoxidePoissonRatio;
    }

    public void setEpoxidePoissonRatio(String epoxidePoissonRatio) {
        this.epoxidePoissonRatio = epoxidePoissonRatio;
    }

    public String getEpoxideStrengthLimit() {
        return epoxideStrengthLimit;
    }

    public void setEpoxideStrengthLimit(String epoxideStrengthLimit) {
        this.epoxideStrengthLimit = epoxideStrengthLimit;
    }

    public String getAverageLayerThickness() {
        return averageLayerThickness;
    }

    public void setAverageLayerThickness(String averageLayerThickness) {
        this.averageLayerThickness = averageLayerThickness;
    }

    public String getValueA() {
        return valueA;
    }

    public void setValueA(String valueA) {
        this.valueA = valueA;
    }

    public String getValueB() {
        return valueB;
    }

    public void setValueB(String valueB) {
        this.valueB = valueB;
    }

    public String getValueL() {
        return valueL;
    }

    public void setValueL(String valueL) {
        this.valueL = valueL;
    }

    @Override
    public String toString() {
        return "ECParameters{" +
                "matrixElacticityModulus='" + matrixElacticityModulus + '\'' +
                ", matrixPoissonRatio='" + matrixPoissonRatio + '\'' +
                ", matrixStrengthLimit='" + matrixStrengthLimit + '\'' +
                ", fiberElacticityModulus='" + fiberElacticityModulus + '\'' +
                ", fiberPoissonRatio='" + fiberPoissonRatio + '\'' +
                ", fiberStrengthLimit='" + fiberStrengthLimit + '\'' +
                ", epoxideElacticityModulus='" + epoxideElacticityModulus + '\'' +
                ", epoxidePoissonRatio='" + epoxidePoissonRatio + '\'' +
                ", epoxideStrengthLimit='" + epoxideStrengthLimit + '\'' +
                ", averageLayerThickness='" + averageLayerThickness + '\'' +
                ", fiberConcentration='" + fiberConcentration + '\'' +
                ", fiberDiameter='" + fiberDiameter + '\'' +
                ", valueA='" + valueA + '\'' +
                ", valueB='" + valueB + '\'' +
                ", valueL='" + valueL + '\'' +
                '}';
    }
}
