package ru.crp.lvms.tasks.fileGenerators;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.cmlb.model.entities.FileData;
import ru.crp.cmlb.model.entities.ResourcePreparationException;
import ru.crp.cmls.calculations.solvers.xml.config.DataType;
import ru.crp.cmls.calculations.solvers.xml.config.Type;
import ru.crp.lvms.exceptions.InternalException;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructConvertDirection;
import ru.crp.lvms.optimizations.optistruct.tcl.OptistructBatchGenerator;
import ru.crp.lvms.optimizations.optistruct.tcl.OptistructConvertScriptGenerator;
import ru.crp.lvms.optimizations.optistruct.tcl.OptistructFemScriptGenerator;
import ru.crp.lvms.optimizations.optistruct.tcl.OptistructOssScriptGenerator;
import ru.crp.lvms.tasks.slurm.SlurmPathsResolver;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.Collections.singletonList;
import static ru.crp.lvms.model.mappers.PgArrayPropertyEditor.LOGGER;


@Component
public class NXNastranInputFileGenerator extends BaseInputFileGenerator {

    public static final String NX_NASTRAN_CONVERT_TEMPLATE = "nx-nastran-convert.ftl";
    public static final String TEMP_MODEL_FILE_NAME = "tempmodel.prt";
    public static final String FEM_FILE_NAME = "tempmodel_f.fem";
    public static final String SIM_FILE_NAME = "tempmodel_s.sim";
    public static final String INPUT_DAT_NAME = "Start.dat";
    public static final String NXNASTRAN_BATCH_FILE_NAME = "start.bat";

    public static final String NX_NASTRAN_RUN_JOURNAL = "\"NX_NASTRAN_RUN_JOURNAL_PATH\" INPUT_FILES_PATH/";
    public static final String NX_NASTRAN_SOLVER = "\"NX_NASTRAN_SOLVER\" INPUT_FILES_PATH/";



    @Autowired
    private Configuration configuration;

    @Autowired
    private FileManager fileManager;


    public List<FileData> getInputFilesForOptimization(OptimizationEntity optimization) throws IOException, ResourcePreparationException {
        List<FileData> initialFileData = optimizationManager.getInputFilesForOptimization(optimization, null);
        List<FileData> fileData = optimizationManager.getInputFilesForOptimization(optimization, null);
        Template template = configuration.getTemplate(NX_NASTRAN_CONVERT_TEMPLATE);
        Path tempConfigFile = Files.createTempFile(null, null);
        StringBuilder content = new StringBuilder();
        initialFileData.stream()
                .map(f->f.getName())
                .filter(s->s.contains(".dat"))
                .forEach(fileName-> {
                    try {
                        Path tempFile = getConvertTemplate(fileName,optimization,template);
                        String nameWithoutExt = fileName.replaceFirst("[.][^.]+$", "");
                        fileData.add(FileData.fromFile(nameWithoutExt+"_convert.vb", DataType.CUSTOM_INPUT_FILE, tempFile.toFile()));
                        content.append(NX_NASTRAN_RUN_JOURNAL + nameWithoutExt+"_convert.vb" + "\n");
                    } catch (IOException ex){
                        throw new ResourcePreparationException(String.format("Can't convert .dat file %s", fileName), ex);
                    }
                });
        content.append(NX_NASTRAN_SOLVER + INPUT_DAT_NAME + " prog=bundle"+ "\n");
        content.append(formPostprocCommand(fileData)+ "\n");
        Files.write(tempConfigFile, content.toString().getBytes());
        fileData.add(FileData.fromFile(NXNASTRAN_BATCH_FILE_NAME, DataType.SESSION_FILE, tempConfigFile.toFile()));
        return fileData;
    }

    Path getConvertTemplate (String datFileName, OptimizationEntity optimization, Template template) throws  IOException, ResourcePreparationException {
        Map<String, Object> params = new HashMap<>();
        params.put("tempModelPath", TEMP_MODEL_FILE_NAME);
        params.put("inputFileName", datFileName);
        params.put("tempFemPath", FEM_FILE_NAME);
        params.put("tempSimPath", SIM_FILE_NAME);
        params.put("outputFileName", datFileName);
        try {
            Path tempFile = Files.createTempFile(null, null);
            template.process(params, Files.newBufferedWriter(tempFile));
            return tempFile;
        } catch (IOException | TemplateException ex) {
            throw new ResourcePreparationException(String.format("Can't generate config file for optimisation %d", optimization.getId()), ex);
        }
    }

    private String formPostprocCommand(List<FileData> optimizationFiles) {
        for (FileData f:optimizationFiles) {
            if (f.getName().contains("postproc.ses")) {
                f.setType(DataType.SESSION_FILE);
                return ("\"C:\\Program Files\\GNS mbH\\Animator4_v2.2.4_64\\StartA4_64_osmesa.exe\" -s "+ f.getName());
            }
        }
        return("!No_postrpoc_file_detected");
    }
}


