package ru.crp.lvms.tasks.misc;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.lvms.cluster.model.ClusterType;

import javax.annotation.PostConstruct;
import java.util.EnumMap;
import java.util.Map;
import java.util.Set;

@Component
public class TaskCommandsFactory {
    private Map<ClusterType, ClusterCommandsManager> clusterCommandsManagers;

    @Autowired
    public void setClusterCommandsManagers(Set<ClusterCommandsManager> clusterCommandsManagersSet) {
        clusterCommandsManagers = new EnumMap<>(ClusterType.class);
        clusterCommandsManagersSet.forEach(ccm -> {
            ClusterType clusterType = ccm.getClusterType();
            if (clusterCommandsManagers.putIfAbsent(clusterType, ccm) != null) {
                throw new IllegalArgumentException(String.format("Found more then 1 cluster commands manager for cluster type '%s'", clusterType));
            }
        });
    }

    @PostConstruct
    public void init() {
        if (clusterCommandsManagers.size() != ClusterType.values().length) {
            throw new IllegalStateException("Not enough cluster commands managers");
        }
    }

    public ClusterCommandsManager getClusterCommandManager(ClusterType clusterType) {
        return clusterCommandsManagers.get(clusterType);
    }
}
