package ru.crp.lvms.tasks.slurm.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.lvms.model.entities.tasks.SlurmTaskInfo;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.misc.BaseTaskCommandCallback;
import ru.crp.lvms.tasks.slurm.SlurmTaskInfoDao;
import ru.crp.lvms.tasks.slurm.exceptions.SlurmNonZeroExitCodeException;

@Service
public class SlurmStartTaskCommandCallback extends BaseTaskCommandCallback<Void> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SlurmStartTaskCommandCallback.class);

    @Autowired
    private SlurmTaskInfoDao slurmTaskInfoDao;

    @Override
    public void onSuccess(TaskEntity task, Object unused) {
        // don't change parent status, cause we don't know current task status(waiting or solving)
        SlurmTaskInfo slurmTaskInfo = slurmTaskInfoDao.getSlurmTaskInfo(task.getId());
        LOGGER.info("Task {} successfully sent to slurm. Slurm job id = {}", task.getId(), slurmTaskInfo.getSlurmTaskId());
        // can't change to 'Starting' state because slurm handling tasks queue by itself
        taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Waiting);
    }

    @Override
    public void onFailure(TaskEntity task, Throwable error) {
        // regardless of the exception reason all we can do is change task to 'Failed'
        if (error instanceof SlurmNonZeroExitCodeException) {
            SlurmNonZeroExitCodeException slurmStartError = (SlurmNonZeroExitCodeException) error;
            // intentionally skipped logging exception
            LOGGER.error("Error exit code({}: {}) while starting slurm command for task {}",
                    slurmStartError.getExitStatus(), slurmStartError.getExitErrorMessage(), task.getId());
        } else {
            LOGGER.error("Unexpected exception while sending task {} to slurm", task.getId(), error);
        }
        taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
    }
}
