package ru.crp.lvms.tasks.slurm.commands;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.util.concurrent.ListenableFuture;
import ru.crp.lvms.cluster.model.SlurmClusterConfigEntity;
import ru.crp.lvms.model.entities.tasks.SlurmTaskInfo;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.misc.BaseTaskCommand;
import ru.crp.lvms.tasks.slurm.SlurmConnectionManager;
import ru.crp.lvms.tasks.slurm.requests.CancelTaskRequest;

@Service
public class SlurmCancelTaskCommand extends BaseTaskCommand<Void, Void> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SlurmCancelTaskCommand.class);

    @Autowired
    private SlurmConnectionManager slurmConnectionManager;

    // all exceptions thrown by this method will be caught in ListenableFuture callback
    @Async("slurmStartTaskExecutor")
    @Override
    public ListenableFuture<Void> processTask(TaskEntity task, Object unused) throws Exception {
        if (!isSlurmTaskStarted(task)) {
            return new AsyncResult<>(null);
        }
        SlurmClusterConfigEntity clusterConfig = clustersManager.getSlurmClusterConfig(task.getClusterId());
        SlurmTaskInfo slurmTaskInfo = slurmTaskInfoDao.getSlurmTaskInfo(task.getId());
        Integer slurmTaskId = slurmTaskInfo.getSlurmTaskId();
        LOGGER.info("Cancelling task {}. Slurm job id = {}", task.getId(), slurmTaskId);

        slurmConnectionManager.execCommand(clusterConfig, new CancelTaskRequest(slurmTaskId));
        return new AsyncResult<>(null);
    }
}
