package ru.crp.lvms.tasks.slurm.commands;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.util.concurrent.ListenableFuture;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.lvms.cluster.model.ClusterEntity;
import ru.crp.lvms.cluster.model.SlurmClusterConfigEntity;
import ru.crp.lvms.model.entities.tasks.SlurmTaskInfo;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.misc.BaseTaskCommand;
import ru.crp.lvms.tasks.slurm.SlurmConnectionManager;
import ru.crp.lvms.tasks.slurm.SlurmTaskInfoMapper;
import ru.crp.lvms.tasks.slurm.exceptions.SlurmNonZeroExitCodeException;
import ru.crp.lvms.tasks.slurm.requests.ObjectDetailsRequest;

import java.util.Map;

@Service
public class SlurmCheckTaskCommand extends BaseTaskCommand<Void, Void> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SlurmCheckTaskCommand.class);

    @Autowired
    private SlurmConnectionManager slurmConnectionManager;

    @Autowired
    private SlurmTaskInfoMapper slurmTaskInfoMapper;

    @Override
    public boolean isTaskReadyForProcessing(TaskEntity task, ClusterEntity taskCluster) {
        return super.isTaskReadyForProcessing(task, taskCluster) && isSlurmTaskStarted(task);
    }


    // all exceptions thrown by this method will be caught in ListenableFuture callback
    @Async("slurmTaskProgressExecutor")
    @Override
    public ListenableFuture<Void> processTask(TaskEntity task, Object unused) throws Exception {
        Map<String, String> taskInfo = null;
        CalculationStatus newState = null;
        int previousStatusId = task.getStatusId();
        SlurmTaskInfo slurmTaskInfo = slurmTaskInfoDao.getSlurmTaskInfo(task.getId());
        try {
            SlurmClusterConfigEntity clusterConfig = clustersManager.getSlurmClusterConfig(task.getClusterId());
            Integer slurmTaskId = slurmTaskInfo.getSlurmTaskId();
            taskInfo = slurmConnectionManager.execCommand(clusterConfig, ObjectDetailsRequest.createTaskDetailsCommand(slurmTaskId + ""));
        } catch (SlurmNonZeroExitCodeException ex) {
            LOGGER.warn("Non-zero exit code while checking status for task {}: '{}'", task.getId(), ex.getExitErrorMessage());
            newState = CalculationStatus.Failed;
        }
        if (taskInfo != null) {
            if (slurmTaskInfo.getNodeNames().isEmpty()) {
                slurmTaskInfo.setNodeNames(slurmTaskInfoMapper.extractTaskNodes(taskInfo));
                slurmTaskInfoDao.updateSlurmTaskInfo(task.getId(), slurmTaskInfo);
            }
            newState = slurmTaskInfoMapper.extractTaskStatus(taskInfo);
        }
        if (newState == CalculationStatus.Failed) {
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.CalculationFinished);
            taskStatusUpdater.setCalculationFinishedWithException(task);
        } else {
            taskStatusUpdater.changeTaskStatus(task, newState);
        }

        return new AsyncResult<>(null);
    }
}
