package ru.crp.lvms.tasks.space.callbacks;

import org.apache.http.conn.HttpHostConnectException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.cmls.api.ServiceTask;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.DownloadTaskResultsJob;
import ru.crp.lvms.tasks.StartTasksJob;
import ru.crp.lvms.tasks.misc.BaseTaskCommandCallback;
import ru.crp.lvms.tasks.space.exceptions.NodeNotResponseException;


@Component
public class SpaceCheckTaskCommandCallback extends BaseTaskCommandCallback<ServiceTask> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SpaceCheckTaskCommandCallback.class);

    @Autowired
    private StartTasksJob startTasksJob;

    @Autowired
    private DownloadTaskResultsJob downloadTaskResultsJob;


    @Override
    public void onFailure(TaskEntity task, Throwable error) {
        if (error instanceof HttpHostConnectException) {
            LOGGER.warn("Can't check task because node is unavailable");
            try {
                nodesManager.getNodeStatus(task.getNodeId());
            } catch (NodeNotResponseException e) {
                LOGGER.warn("Error while checking node status", e);
            }
        } else {
            LOGGER.warn("Can't check task", error);
        }
    }


    @Override
    public void onSuccess(TaskEntity task, Object serviceTask) {
    }
}
