package ru.crp.lvms.tasks.space.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.cmlb.model.entities.ResourcePreparationException;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.StartTasksJob;
import ru.crp.lvms.tasks.misc.BaseTaskCommandCallback;
import ru.crp.lvms.tasks.space.exceptions.ResultDownloadingException;
import ru.crp.lvms.tasks.space.exceptions.ResultProcessingException;

@Component
public class SpaceDownloadResultsCommandCallback extends BaseTaskCommandCallback<Boolean> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SpaceDownloadResultsCommandCallback.class);

    @Autowired
    private StartTasksJob startTasksJob;


    @Override
    public void onFailure(TaskEntity task, Throwable ex) {
        int taskId = task.getId();
        if (ex instanceof ResourcePreparationException) {
            LOGGER.error("Resource preparation exception for task {}", taskId, ex);
        } else if (ex instanceof ResultDownloadingException) {
            LOGGER.error("Resource download exception for task {}", taskId, ex);
        } else if (ex instanceof ResultProcessingException) {
            LOGGER.error("Resource processing exception for task {}", taskId, ex);
        } else {
            LOGGER.error("Unexpected exception for task {}", taskId, ex);
        }
        taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
        taskStatusUpdater.setCalculationFinishedWithException(task);
        startTasksJob.checkTaskQueueManually();
    }


    @Override
    public void onSuccess(TaskEntity task, Object additionalArg) {
        taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Finished);
        taskStatusUpdater.setCalculationFinishedWithException(task);
        taskStatusUpdater.findNextTask(task.getId()).ifPresent(nextTask ->
                taskStatusUpdater.changeTaskStatus(nextTask, CalculationStatus.Waiting)
        );
        startTasksJob.checkTaskQueueManually();
    }
}
