package ru.crp.lvms.utils;


import com.google.common.base.CaseFormat;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.cmlb.model.entities.base.Named;

import java.util.List;
import java.util.stream.Collectors;

public final class MessageUtils {
    public static final String WRONG_LOGIN = "wrongLogin";
    public static final String ENTITY_DELETED = "entityDeleted";
    public static final String ENTITY_UPDATED = "entityUpdated";

    public static final String USER_DELETE_ITSELF = "userDeleteItself";
    public static final String ACCESS_DENIED_CODE = "accessDenied";

    public static final String ENTITY_NOT_FOUND = "entity.notFound";

    public static final String PARAMETER_NOT_FOUND = "parameter.notFound";
    public static final String PARAMETER_POINT_NOT_FOUND = "parameter.pointNotFound";
    public static final String PARAMETER_USED_IN_MATERIALS = "parameter.usedInMaterials";
    public static final String PARAMETER_FILE_NOT_FOUND = "parameter.fileNotFound";

    public static final String TECHNOLOGY_NOT_FOUND = "technology.notFound";
    public static final String TECHNOLOGY_USED_IN_MATERIAL = "technology.usedInMaterial";

    public static final String MATERIAL_DELETING_IO = "material.deletingIO";
    public static final String MATERIAL_CREATING_IO = "material.creatingIO";
    public static final String MATERIAL_USED_IN_COMPOSITE_MATERIALS = "material.usedInCompositeMaterials";

    public static final String FILE_EXSIST_WITH_DIFFERENT_ROLE = "file.existsWithDifferentRole";

    public static final String TREE_WRONG_NEW_NODE_TYPE = "tree.wrongNewNodeType";
    public static final String TREE_WRONG_NODE_PARENT = "tree.wrongNodeParent";
    public static final String TREE_NODE_NOT_FOUND = "tree.notFound";
    public static final String TREE_NODE_WITH_CHILDREN_DELETE = "tree.nodeWithChildrenDelete";
    public static final String TREE_UNEXPECTED_NODE = "tree.unexpectedNode";
    public static final String TREE_CANT_CHANGE_PARENT_TYPE = "tree.cantChangeParentType";
    public static final String TREE_NODE_REMOVED = "tree.nodeRemoved";
    public static final String TREE_ROOT_OPTIMIZATION_FOLDER = "tree.rootOptimizationFolder";
    public static final String TREE_OPTIMIZATION_TYPE_ALREADY_USED = "tree.optimizationTypeAlreadyUsed";

    public static final String PREFERENCES_COLUMN_NOT_FOUND = "preferences.columnNotFound";
    public static final String PREFERENCES_NOT_FOUND = "preferences.notFound";

    public static final String VALIDATION_UNIQUE_FILED = "validation.uniqueField";

    public static final String SIMULATION_UPDATE_ERROR = "simulation.updateError";
    public static final String SIMULATION_WRONG_SAMPLE_FILE = "simulation.wrongSampleFile";
    public static final String SIMULATION_WRONG_STANDARD = "simulation.wrongStandard";
    public static final String SIMULATION_ALREADY_EXISTS = "simulation.alreadyExists";

    public static final String STANDARD_HAS_RELATION = "standard.hasRelation";

    public static final String IMAGE_WRONG_FORMAT = "image.wrongFormat";

    public static final String USER_HAS_TECHNOLOGIES = "user.hasTechnologies";
    public static final String USER_HAS_MATERIALS = "user.hasMaterials";
    public static final String USER_HAS_COMPOSITE_MATERIALS = "user.hasCompositeMaterials";
    public static final String USER_HAS_FILES = "user.hasFiles";

    public static final String COMPOSITE_MATERIAL_HAS_MATERIAL_FILES = "compositeMaterial.hasMaterialFiles";
    public static final String COMPOSITE_MATERIAL_HAS_SAMPLE_FILES = "compositeMaterial.hasSampleFiles";
    public static final String COMPOSITE_MATERIAL_HAS_SIMULATIONS = "compositeMaterial.hasSimulations";
    public static final String COMPOSITE_MATERIAL_HAS_EFFECTIVE_CHARACTERISTICS = "compositeMaterial.hasEffectiveCharacteristics";

    public static final String FILE_IS_USED_IN_OPTIMIZATION = "optimisations.fileIsUsed";

    public static final String CE_USED_IN_OPTIMIZATIONS = "constructionElement.usedInOptimizations";
    public static final String OPTIMIZATION_USED_IN_FOLDERS = "optimizations.usedInFolders";

    public static final String FILE_USED_IN_OPTIMIZATIONS = "files.usedInOptimizations";

    public static final String EC_USED_IN_SIMULATION = "ec.usedInSimulations";

    private MessageUtils() {

    }


    public static String getEntityType(Class clazz) {
        if (Entity.class.isAssignableFrom(clazz)) {
            return camelToSpaces(clazz.getSimpleName().replace("Entity", ""));
        }
        return camelToSpaces(clazz.getSimpleName());
    }


    // CamelCase to spaces
    public static String camelToSpaces(String camelCase) {
        return CaseFormat.UPPER_CAMEL.to(CaseFormat.LOWER_HYPHEN, camelCase).replace('-', ' ');
    }

    public static String extractNames(List<? extends Named> entities) {
        return entities.stream()
                .map(Named::getName)
                .collect(Collectors.joining(", "));
    }
}
