package ru.crp.lvms.utils;

import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

@Component
public class NumericalInputConverter {

    /*
        Converts strings like "1-15 23 25 34-36" to list of numbers
    */
    public List<Integer> convert(String numericalInput) {
        return extractValues(numericalInput).collect(Collectors.toList());
    }

    public long count(String numericalInput) {
        return extractValues(numericalInput).count();
    }

    private Stream<Integer> extractValues(String numericalInput) {
        if (numericalInput == null || numericalInput.isEmpty()) {
            return Stream.empty();
        }
        String[] blocks = numericalInput.split(" ");
        return Arrays.stream(blocks).map(block -> {
            if (block.contains("-")) {
                String[] ranges = block.split("-");
                return IntStream.rangeClosed(Integer.valueOf(ranges[0]), Integer.valueOf(ranges[1]));
            } else {
                return IntStream.of(Integer.valueOf(block));
            }
        }).flatMap(IntStream::boxed);
    }

}
