package ru.crp.lvms.validators;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.lvms.model.dao.TreeDao;
import ru.crp.lvms.model.entities.misc.ItemType;
import ru.crp.lvms.model.entities.tree.TreeNode;

import javax.annotation.Nullable;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.util.List;

import static java.util.Arrays.asList;

public class ParentExistsValidator implements ConstraintValidator<ParentExists, Integer> {
    @Autowired
    private TreeDao treeDao;

    private List<ItemType> parentTypes;

    @Override
    public void initialize(ParentExists constraintAnnotation) {
        parentTypes = asList(constraintAnnotation.parentTypes());
    }

    @Transactional(readOnly = true)
    @Override
    public boolean isValid(@Nullable Integer value, ConstraintValidatorContext context) {
        if (value == null) {
            return true;
        }
        TreeNode parent = treeDao.findOne(value);
        return parent != null && parentTypes.contains(parent.getItemType());
    }
}
