package ru.crp.lvms.validators;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.model.dao.BaseSpaceDao;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import static ru.crp.lvms.utils.MessageUtils.VALIDATION_UNIQUE_FILED;

public class UniqueByFieldValidator extends BaseEntityValidator implements ConstraintValidator<Unique, Object> {
    private static final Logger LOGGER = LoggerFactory.getLogger(UniqueByFieldValidator.class);

    @Autowired
    private MessageSource msgSrc;

    private String fieldName;
    private String columnName;

    @Override
    public void initialize(Unique constraintAnnotation) {
        init(constraintAnnotation.entityClass());
        fieldName = constraintAnnotation.fieldName();
        columnName = constraintAnnotation.columnNane();
    }

    @Transactional(readOnly = true)
    @Override
    public boolean isValid(Object value, ConstraintValidatorContext context) {
        BaseSpaceDao<? extends Entity> dao = daoList.get(entityClass);
        if (dao == null) {
            throw new IllegalStateException("Can't validate because No dao found for class " + entityClass);
        }
        try {
            String fieldValue = BeanUtils.getProperty(value, fieldName);
            if (fieldValue == null) {
                return true;
            }
            Integer id = null;
            if (PropertyUtils.isReadable(value, "id")) {
                id = Integer.valueOf(BeanUtils.getProperty(value, "id"));
            }

            List<? extends Entity> existedEntities = dao.findByColumnName(columnName, fieldValue);
            if (existedEntities.isEmpty()) {
                return true;
            }
            if (existedEntities.size() > 1) {
                throw new IllegalStateException(msgSrc.getMessage(VALIDATION_UNIQUE_FILED,
                        new Object[] {fieldName}, null, null));
            }
            Entity conflictingEntity = existedEntities.get(0);
            return id != null && conflictingEntity.getId() == id;
        } catch (IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
            LOGGER.error(e.getMessage(), e);
//            todo throw ex
            return false;
        }
    }
}
