package ru.crp.lvms.web.controllers.misc;

import org.jetbrains.annotations.NotNull;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.cmlb.model.entities.base.Named;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.dto.misc.Auditable;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;

public abstract class CrudControllerNotification<E extends Entity & Named, InVO, OutVO extends Entity & Auditable>
        extends CrudControllerSuggest<E, InVO, OutVO> {

    @NotNull
    @Override
    public OutVO delete(HttpServletRequest request, @PathVariable("id") int id, @CurrentUser PortalUser user) {
        OutVO deleted = super.delete(request, id, user);
        getEventsManager().entityDeleted(deleted, user.getUser());
        return deleted;
    }

    @Override
    public void deleteSeveral(HttpServletRequest request,
                              @RequestParam("ids") List<Integer> ids,
                              @CurrentUser PortalUser currentUser) throws BadRequestException {
        deleteSeveralEntities(entityClass, ids, (id) -> {
            delete(request, id, currentUser);
        });
    }

    @NotNull
    @Override
    public OutVO update(HttpServletRequest request, @PathVariable("id") int id,
                        @Valid @RequestBody InVO updateVO,
                        @CurrentUser PortalUser user) {
        OutVO updated = super.update(request, id, updateVO, user);
        getEventsManager().entityUpdated(updated, user.getUser());
        return updated;
    }

}
