package ru.crp.lvms.web.controllers.technology;

import org.jetbrains.annotations.NotNull;
import org.modelmapper.TypeToken;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.HandlerMapping;
import ru.crp.lvms.managers.CompositeMaterialManager;
import ru.crp.lvms.managers.misc.PreferencesManager;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntity;
import ru.crp.lvms.model.entities.compositeMaterials.TechnologyCompositeMaterial;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.utils.Mapper;
import ru.crp.lvms.web.controllers.misc.ColumnsController;
import ru.crp.lvms.web.dto.pagination.DataRequest;
import ru.crp.lvms.web.dto.pagination.PageingResponse;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

import static ru.crp.lvms.utils.URLs.TECHNOLOGY_MATERIALS_URL;
import static ru.crp.lvms.utils.URLs.TECHNOLOGY_URL;

@RestController
@RequestMapping(value = TECHNOLOGY_MATERIALS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class TechnologyMaterialsController {
    public static final TypeToken<List<TechnologyCompositeMaterial>> LIST_TYPE_TOKEN = new TypeToken<List<TechnologyCompositeMaterial>>() {
    };

    @RestController
    @RequestMapping(value = TECHNOLOGY_URL + "/compositeMaterials", produces = MediaType.APPLICATION_JSON_VALUE)
    public static class TechnologyMaterialsColumnsController implements ColumnsController {
        @Autowired
        private PreferencesManager preferencesManager;

        @Override
        public Class getOutVoClass() {
            return TechnologyCompositeMaterial.class;
        }

        @Override
        public PreferencesManager getPreferencesManager() {
            return preferencesManager;
        }
    }

    @Autowired
    private CompositeMaterialManager compositeMaterialManager;

    @NotNull
    @RequestMapping(method = RequestMethod.GET)
    public List<TechnologyCompositeMaterial> getAll(HttpServletRequest request, @CurrentUser PortalUser currentUser) {
        Map pathVariables = (Map) request.getAttribute(HandlerMapping.URI_TEMPLATE_VARIABLES_ATTRIBUTE);
        int technologyId = Integer.parseInt((String) pathVariables.getOrDefault("technologyId", "-1"));
        return Mapper.map(compositeMaterialManager.getByTechnologyId(technologyId, currentUser.getUser()), LIST_TYPE_TOKEN);
    }


    @NotNull
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public PageingResponse<TechnologyCompositeMaterial> page(HttpServletRequest request,
                                                             @RequestBody DataRequest dataRequest,
                                                             @CurrentUser PortalUser currentUser) {
        Map pathVariables = (Map) request.getAttribute(HandlerMapping.URI_TEMPLATE_VARIABLES_ATTRIBUTE);
        int technologyId = Integer.parseInt((String) pathVariables.getOrDefault("technologyId", "-1"));
        Pageable pageable = dataRequest.getSpringPageable();
        PageImpl<CompositeMaterialEntity> page = compositeMaterialManager
                .getPageByTechnologyId(technologyId, dataRequest.getFilters(), pageable, currentUser.getUser());
        return new PageingResponse<>(Mapper.map(page.getContent(), LIST_TYPE_TOKEN), dataRequest.getFilters(), pageable, page.getTotalElements());
    }
}
