
DROP VIEW optistruct_optimization_tech_constraints_join;
ALTER TABLE optistruct_optimization_tech_constraints DROP COLUMN value;
CREATE VIEW optistruct_optimization_tech_constraints_join AS (
  SELECT
    base.*,
    o.optimization_type_id           AS optimization_type_id,
    optistruct_tech_constraints.placeholder AS tech_constraint_placeholder,
    optistruct_tech_constraints.name AS tech_constraint_name

  FROM optistruct_optimization_tech_constraints base
    JOIN optimizations o ON base.optimization_id = o.id
    LEFT JOIN optistruct_tech_constraints ON base.tech_constr_id = optistruct_tech_constraints.id
);

CREATE RULE optistruct_optimization_tech_constraints_join_CREATE
AS ON INSERT TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tech_constraints (optimization_id, tech_constr_id, name)
  VALUES (NEW.optimization_id, NEW.tech_constr_id, NEW.name)
  RETURNING
    optistruct_optimization_tech_constraints.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS tech_constraint_name,
    NULL :: VARCHAR(255) AS tech_constraint_placeholder
);

CREATE RULE optistruct_optimization_tech_constraints_join_UPDATE
AS ON UPDATE TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  UPDATE optistruct_optimization_tech_constraints
  SET tech_constr_id = NEW.tech_constr_id, name = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_tech_constraints_join_DELETE
AS ON DELETE TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tech_constraints
  WHERE id = OLD.id
);


INSERT INTO optistruct_tech_constraints (id, name, placeholder) VALUES (5, 'Усталость', '');
INSERT INTO optistruct_tech_constraints (id, name, placeholder) VALUES (6, 'Поведение сетки', '');
INSERT INTO optistruct_tech_constraints (id, name, placeholder) VALUES (7, 'Условия симметрии', '');
INSERT INTO optistruct_tech_constraints (id, name, placeholder) VALUES (8, 'Изготовление литьем', '');
INSERT INTO optistruct_tech_constraints (id, name, placeholder) VALUES (9, 'Направление вытяжки - экструзия', '');


CREATE TABLE optistruct_technological_constraints_tree (
  id                SERIAL,
  name              VARCHAR(255),
  display_name      VARCHAR(255),
  is_value_editable BOOLEAN,
  parent_id         INTEGER,
  --   pointer to 'common' tc
  tc_id             INTEGER,
  type              VARCHAR(255),
  tooltip           VARCHAR(255),
  group_name        VARCHAR(255),

  FOREIGN KEY (tc_id) REFERENCES optistruct_tech_constraints (id) ON DELETE CASCADE,
  FOREIGN KEY (parent_id) REFERENCES optistruct_technological_constraints_tree (id) ON DELETE CASCADE,
  PRIMARY KEY (id)
);

CREATE TABLE optistruct_technological_constraints_tree_values (
  id         SERIAL,
  tc_tree_id INTEGER,
  --   pointer to optimization tc
  tc_id      INTEGER,
  value      VARCHAR(255),

  FOREIGN KEY (tc_id) REFERENCES optistruct_optimization_tech_constraints (id) ON DELETE CASCADE,
  FOREIGN KEY (tc_tree_id) REFERENCES optistruct_technological_constraints_tree (id) ON DELETE CASCADE,
  PRIMARY KEY (id)
);


INSERT INTO optistruct_technological_constraints_tree (id, tc_id, name, display_name, is_value_editable, parent_id, type, tooltip, group_name) VALUES
  -- Минимальный диаметр оптимизированного элемента
  (100, 1, 'MIN_DIM_VALUE', 'Значение', TRUE, NULL, NULL, NULL, NULL),

  -- Максимальный диаметр оптимизированного элемента
  (200, 2, 'MAX_DIM_VALUE', 'Значение', TRUE, NULL, NULL, NULL, NULL),

  -- Минимальное расстояние между оптимизированными структурными элементами
  (300, 3, 'GAP_VALUE', 'Значение', TRUE, NULL, NULL, NULL, NULL),

  -- Глобальное максимальное ограничение на напряжения
  (400, 4, 'TENSION_VALUE', 'Значение', TRUE, NULL, NULL, NULL, NULL),

  -- Усталость
  (500, 5, 'TC_FATIGUE_TYPE', 'Тип усталости', FALSE, NULL, NULL, NULL, NULL),
    (501, 5, 'FATIQUE_TYPE_MAX', 'Максимальное значение допустимого разрушения', FALSE, 500, 'select', NULL, 'default'),
        (502, 5, 'FATIQUE_VALUE', 'Значение', TRUE, 501, NULL, NULL, NULL),
    (503, 5, 'FATIQUE_TYPE_MIN', 'Минимальное значение долговечности', FALSE, 500, 'select', NULL, 'default'),
        (504, 5, 'FATIQUE_VALUE', 'Значение', TRUE, 503, NULL, NULL, NULL),

  -- Поведение сетки
  (600, 6, 'TC_GRID_ACTIVE', 'Тип сетки активен', FALSE, NULL, 'checkbox', NULL, NULL),
  (603, 6, 'TC_GRID_DIR', 'Тип сетки сонаправлен с направлением вытяжки или с направлением для снятия литьевой формы', FALSE, NULL, 'checkbox', NULL, NULL),

  -- Условия симметрии
  (700, 7, 'TC_SYMMETRY_CONDITION', 'Тип симметрии', FALSE, NULL, NULL, NULL, NULL),
    (701, 7, 'TC_SYM_TYPE_ONE_PLANE', 'Одна плоскость симметрии', FALSE, 700, 'select', NULL, 'default'),

      (702, 7, 'TC_ONE_PLANE_COORDS', 'Координаты отсчетного узла', FALSE, 701, NULL, NULL, NULL),
        (703, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_X', 'X', TRUE, 702, NULL, NULL, NULL),
        (704, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Y', 'Y', TRUE, 702, NULL, NULL, NULL),
        (705, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Z', 'Z', TRUE, 702, NULL, NULL, NULL),

      (706, 7, 'TC_SYMMETRY_CONDITION_ID_1', 'Номер (ID) отсчетного узла', TRUE, 701, NULL, NULL, NULL),

      (707, 7, 'TC_ONE_PLANE_NORMAL', 'Координаты узла, определяющего направление вектора, перпендикулярного к плоскости симметрии', FALSE, 701, NULL, NULL, NULL),
        (708, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_X', 'X', TRUE, 707, NULL, NULL, NULL),
        (709, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Y', 'Y', TRUE, 707, NULL, NULL, NULL),
        (710, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Z', 'Z', TRUE, 707, NULL, NULL, NULL),

      (711, 7, 'TC_SYMMETRY_CONDITION_ID_2', 'Номер (ID) узла, определяющего направление вектора, перпендикулярног о к плоскости симметрии;', TRUE, 701, NULL, NULL, NULL),

    (712, 7, 'TC_SYM_TYPE_TWO_PLANES', 'Две плоскости симметрии', FALSE, 700, 'select', NULL, 'default'),

      (713, 7, 'TC_TWO_PLANES_COORDS', 'Координаты отсчетного узла', FALSE, 712, NULL, NULL, NULL),
        (714, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_X', 'X', TRUE, 713, NULL, NULL, NULL),
        (715, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Y', 'Y', TRUE, 713, NULL, NULL, NULL),
        (716, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Z', 'Z', TRUE, 713, NULL, NULL, NULL),

      (717, 7, 'TC_SYMMETRY_CONDITION_ID_1', 'Номер (ID) отсчетного узла', TRUE, 712, NULL, NULL, NULL),

      (718, 7, 'TC_TWO_PLANES_COORDS_NORMAL_1', 'Координаты узла, определяющего направление вектора, перпендикулярного к первой плоскости симметрии', FALSE, 712, NULL, NULL, NULL),
        (719, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_X', 'X', TRUE, 718, NULL, NULL, NULL),
        (720, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Y', 'Y', TRUE, 718, NULL, NULL, NULL),
        (721, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Z', 'Z', TRUE, 718, NULL, NULL, NULL),

      (722, 7, 'TC_SYMMETRY_CONDITION_ID_2', 'Номер (ID) узла, определяющего направление вектора, перпендикулярног о к плоскости симметрии;', TRUE, 712, NULL, NULL, NULL),

      (723, 7, '// TC_TWO_PLANES_COORDS_NORMAL_2', 'Координаты, определяющего направление вектора, перпендикулярного ко второй плоскости симметрии', FALSE, 712, NULL, NULL, NULL),
        (724, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_X', 'X', TRUE, 723, NULL, NULL, NULL),
        (725, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_Y', 'Y', TRUE, 723, NULL, NULL, NULL),
        (726, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_Z', 'Z', TRUE, 723, NULL, NULL, NULL),

      (727, 7, 'TC_SYMMETRY_CONDITION_ID_3', 'Номер (ID) узла, определяющего направление вектора, перпендикулярног о к плоскости симметрии;', TRUE, 712, NULL, NULL, NULL),

    (728, 7, 'TC_SYM_TYPE_THREE_PLANES', 'Три плоскости симметрии', FALSE, 700, 'select', NULL, 'default'),
      (729, 7, 'TC_THREE_PLANES_COORDS', 'Координаты отсчетного узла', FALSE, 728, NULL, NULL, NULL),
        (730, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_X', 'X', TRUE, 729, NULL, NULL, NULL),
        (731, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Y', 'Y', TRUE, 729, NULL, NULL, NULL),
        (732, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Z', 'Z', TRUE, 729, NULL, NULL, NULL),

      (733, 7, 'TC_SYMMETRY_CONDITION_ID_1', 'Номер (ID) отсчетного узла', TRUE, 728, NULL, NULL, NULL),

      (734, 7, 'TC_THREE_PLANES_COORDS_NORMAL_1', 'Координаты узла, определяющего направление вектора, перпендикулярного к первой плоскости симметрии', FALSE, 728, NULL, NULL, NULL),
        (735, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_X', 'X', TRUE, 734, NULL, NULL, NULL),
        (736, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Y', 'Y', TRUE, 734, NULL, NULL, NULL),
        (737, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Z', 'Z', TRUE, 734, NULL, NULL, NULL),

      (738, 7, 'TC_SYMMETRY_CONDITION_ID_2', 'Номер (ID) узла, определяющего направление вектора, перпендикулярног о к плоскости симметрии', TRUE, 728, NULL, NULL, NULL),

      (739, 7, 'TC_THREE_PLANES_COORDS_NORMAL_3', 'Координаты узла по оси X, определяющего направление вектора, перпендикулярного ко второй плоскости симметрии', FALSE, 728, NULL, NULL, NULL),
        (740, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_X', 'X', TRUE, 739, NULL, NULL, NULL),
        (741, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_Y', 'Y', TRUE, 739, NULL, NULL, NULL),
        (742, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_Z', 'Z', TRUE, 739, NULL, NULL, NULL),

      (743, 7, 'TC_SYMMETRY_CONDITION_ID_3', 'Номер (ID) узла, определяющего направление вектора, перпендикулярног о к плоскости симметрии;', TRUE, 728, NULL, NULL, NULL),


    (744, 7, 'TC_SYM_TYPE_CYCLIC', 'Циклическая симметрия', FALSE, 700, 'select', NULL, 'default'),

      (745, 7, 'CYCLIC_COORDS', 'Координаты отсчетного узла', FALSE, 744, NULL, NULL, NULL),
        (746, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_X', 'X', TRUE, 745, NULL, NULL, NULL),
        (747, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Y', 'Y', TRUE, 745, NULL, NULL, NULL),
        (748, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Z', 'Z', TRUE, 745, NULL, NULL, NULL),

      (749, 7, 'TC_SYMMETRY_CONDITION_ID_1', 'Номер (ID) отсчетного узла', TRUE, 744, NULL, NULL, NULL),

      (750, 7, 'CYCLIC_COORDS_DIR', 'Координаты узла, определяющего направление оси симметрии', FALSE, 744, NULL, NULL, NULL),
        (751, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_X', 'X', TRUE, 750, NULL, NULL, NULL),
        (752, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Y', 'Y', TRUE, 750, NULL, NULL, NULL),
        (753, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Z', 'Z', TRUE, 750, NULL, NULL, NULL),

      (754, 7, 'TC_SYMMETRY_CONDITION_ID_2', 'Номер (ID) узла, определяющего направление оси симметрии', TRUE, 744, NULL, NULL, NULL),

    (755, 7, 'TC_SYM_TYPE_CYCLIC_AND_ONE_PLANE', 'Циклическая симметрия + одна плоскость симметрии', FALSE, 700, 'select', NULL, 'default'),

      (756, 7, 'CYCLIC_AND_ONE_PLANE_COORDS', 'Координаты отсчетного узла', FALSE, 755, NULL, NULL, NULL),
        (757, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_X', 'X', TRUE, 756, NULL, NULL, NULL),
        (758, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Y', 'Y', TRUE, 756, NULL, NULL, NULL),
        (759, 7, 'TC_SYMMETRY_CONDITION_COORDS_1_Z', 'Z', TRUE, 756, NULL, NULL, NULL),

      (760, 7, 'TC_SYMMETRY_CONDITION_ID_1', 'Номер (ID) отсчетного узла', TRUE, 755, NULL, NULL, NULL),

      (761, 7, 'CYCLIC_AND_ONE_PLANE_COORDS_DIR', 'Координаты узла, определяющего направление оси симметрии', FALSE, 755, NULL, NULL, NULL),
        (762, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_X', 'X', TRUE, 761, NULL, NULL, NULL),
        (763, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Y', 'Y', TRUE, 761, NULL, NULL, NULL),
        (764, 7, 'TC_SYMMETRY_CONDITION_COORDS_2_Z', 'Z', TRUE, 761, NULL, NULL, NULL),

      (765, 7, 'TC_SYMMETRY_CONDITION_ID_2', 'Номер (ID) узла, определяющего направление оси симметрии', TRUE, 755, NULL, NULL, NULL),

      (766, 7, 'TC_SYMMETRY_CONDITION_NUM_OF_LOOPS', 'Число циклов симметрии', TRUE, 755, NULL, NULL, NULL),

      (767, 7, 'CYCLIC_AND_ONE_PLANE_COORDS_NORMAL', 'Координаты второго узла, чья проекция образует вектор, исходящий из отсчетного узла, перпендикулярный оси симметрии', FALSE, 755, NULL, NULL, NULL),
        (768, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_X', 'X', TRUE, 767, NULL, NULL, NULL),
        (769, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_Y', 'Y', TRUE, 767, NULL, NULL, NULL),
        (770, 7, 'TC_SYMMETRY_CONDITION_COORDS_3_Z', 'Z', TRUE, 767, NULL, NULL, NULL),

      (771, 7, 'TC_SYMMETRY_CONDITION_ID_3', 'Номер (ID) второго узла, чья проекция образует вектор, исходящий из отсчетного узла, перпендикулярный оси симметрии', TRUE, 755, NULL, NULL, NULL),

  -- Изготовление литьем
  (800, 8, 'TC_CAST', 'Направление снятия литьевой формы', FALSE, NULL, NULL, NULL, NULL),
    (801, 8, 'TC_CAST_SOLID', 'Цельная литьевая форма', FALSE, 800, 'select', NULL, 'default'),

      (802, 8, 'TC_CAST_SOLID_COORDS_1', 'Координаты отсчетного узла', FALSE, 801, NULL, NULL, NULL),
        (803, 8, 'TC_CAST_FIRST_NODE_X', 'X', TRUE, 802, NULL, NULL, NULL),
        (804, 8, 'TC_CAST_FIRST_NODE_Y', 'Y', TRUE, 802, NULL, NULL, NULL),
        (805, 8, 'TC_CAST_FIRST_NODE_Z', 'Z', TRUE, 802, NULL, NULL, NULL),

      (806, 8, 'TC_CAST_ID_1', 'Номер (ID) отсчетного узла', TRUE, 801, NULL, NULL, NULL),

      (828, 8, 'TC_CAST_SOLID_COORDS_2', 'Координаты узла, определяющего направление снятия литьевой формы с конструкции', FALSE, 801, NULL, NULL, NULL),
        (829, 8, 'TC_CAST_SECOND_NODE_X', 'X', TRUE, 828, NULL, NULL, NULL),
        (830, 8, 'TC_CAST_SECOND_NODE_Y', 'Y', TRUE, 828, NULL, NULL, NULL),
        (831, 8, 'TC_CAST_SECOND_NODE_Z', 'Z', TRUE, 828, NULL, NULL, NULL),

      (832, 8, 'TC_CAST_ID_2', 'Номер (ID) узла, определяющего направление снятия литьевой формы с конструкции', TRUE, 801, NULL, NULL, NULL),

      (807, 8, 'TC_CAST_NO_HOLES', 'Включить отсутствие сквозных отверстий', FALSE, 801, 'checkbox', NULL, NULL),

      (810, 8, 'TC_CAST_DESIRED_THICKNESS', 'Включить опцию для штамповк и с желаемой толщиной полученной оболочки или штампованной части', FALSE, 801, 'checkbox', NULL, NULL),
          (813, 8, 'TC_CAST_DESIRED_THICKNESS_VAL', 'Желаемая толщина', TRUE, 810, NULL, NULL, NULL),

    (814, 8, 'TC_CAST_SECTIONAL', 'Разъемная литьевая форма', FALSE, 800, 'select', NULL, 'default'),

      (815, 8, 'TC_CAST_SECTIONAL_COORDS', 'Координаты отсчетного узла', FALSE, 814, NULL, NULL, NULL),
        (816, 8, 'TC_CAST_FIRST_NODE_X', 'X', TRUE, 815, NULL, NULL, NULL),
        (817, 8, 'TC_CAST_FIRST_NODE_Y', 'Y', TRUE, 815, NULL, NULL, NULL),
        (818, 8, 'TC_CAST_FIRST_NODE_Z', 'Z', TRUE, 815, NULL, NULL, NULL),

      (819, 8, 'TC_CAST_ID_1', 'Номер (ID) отсчетного узла', TRUE, 814, NULL, NULL, NULL),

      (820, 8, 'TC_CAST_SECTIONAL_COORDS_2', 'Координаты узла,определяющего направление снятия пресс-формы с конструкции', FALSE, 814, NULL, NULL, NULL),
        (821, 8, 'TC_CAST_SECOND_NODE_X', 'X', TRUE, 820, NULL, NULL, NULL),
        (822, 8, 'TC_CAST_SECOND_NODE_Y', 'Y', TRUE, 820, NULL, NULL, NULL),
        (823, 8, 'TC_CAST_SECOND_NODE_Z', 'Z', TRUE, 820, NULL, NULL, NULL),

      (824, 8, 'TC_CAST_ID_2', 'Номер (ID) узла,определяющего направление снятия пресс-формы с конструкции', TRUE, 814, NULL, NULL, NULL),

      (825, 8, 'TC_CAST_NO_HOLES', 'Включить отсутствие сквозных отверстий', FALSE, 814, 'checkbox', NULL, NULL),

  -- Направление вытяжки - экструзия
  (900, 9, 'TC_EXTRUSION', 'Тип', TRUE, NULL, NULL, NULL, NULL),
    (901, 9, 'TC_WITH_ROTATION', 'С поворотом поперечного сечения', FALSE, 900, 'select', NULL, 'default'),
      (902, 9, 'TC_ROTATION_NODE_IDS', 'Номера узлов', TRUE, 901, NULL, NULL, NULL),
    (903, 9, 'TC_WITHOUT_ROTATION', 'Без поворота поперечного сечения', FALSE, 900, 'select', NULL, 'default'),
      (904, 9, 'TC_WITHOUT_ROTATION_FIRST_NODE_IDS', 'Номера узлов до поворота сечения', TRUE, 903, NULL, NULL, NULL),
      (905, 9, 'TC_WITHOUT_ROTATION_SECOND_NODE_IDS', 'Номера после поворота сечения', TRUE, 903, NULL, NULL, NULL);


CREATE VIEW optistruct_technological_constraints_extended AS
  SELECT
    otc.*,
    EXISTS(SELECT *
           FROM optistruct_technological_constraints_tree
           WHERE tc_id = otc.id) AS has_fields
  FROM optistruct_tech_constraints otc
  ORDER BY otc.name;

CREATE VIEW optistruct_technological_constraints_tree_extended AS
  SELECT
    otct.*,
    EXISTS(SELECT *
           FROM optistruct_technological_constraints_tree child
           WHERE child.parent_id = otct.id) AS is_parent
  FROM optistruct_technological_constraints_tree otct
  ORDER BY otct.name;

CREATE VIEW optistruct_technological_constraints_tree_values_join AS
  SELECT
    otctv.*,
    otct.display_name,
    otct.is_value_editable,
    otct.parent_id,
    otct.name,
    otct.is_parent,
    otct.type,
    otct.tooltip,
    otct.group_name
  FROM optistruct_technological_constraints_tree_extended otct
    JOIN optistruct_technological_constraints_tree_values otctv
      ON otctv.tc_tree_id = otct.id;

CREATE RULE optistruct_technological_constraints_tree_values_join_CREATE
AS ON INSERT TO optistruct_technological_constraints_tree_values_join DO INSTEAD (
  INSERT INTO optistruct_technological_constraints_tree_values (tc_tree_id, tc_id, value)
  VALUES (NEW.tc_tree_id, NEW.tc_id, NEW.value)
  RETURNING
    optistruct_technological_constraints_tree_values.*,
    NULL :: VARCHAR(255) AS field_name,
    FALSE AS is_value_setable,
    NULL :: INTEGER AS parent_id,
    NULL :: VARCHAR(255) AS name,
    NULL :: BOOLEAN AS is_parent,
    NULL :: VARCHAR(255) AS type,
    NULL :: VARCHAR(255) AS tooltip,
    NULL :: VARCHAR(255) AS group_name
);

CREATE RULE optistruct_technological_constraints_tree_values_join_UPDATE
AS ON UPDATE TO optistruct_technological_constraints_tree_values_join DO INSTEAD (
  UPDATE optistruct_technological_constraints_tree_values
  SET value = NEW.value
  WHERE tc_tree_id = NEW.tc_tree_id);

CREATE RULE optistruct_technological_constraints_tree_values_join_DELETE
AS ON DELETE TO optistruct_technological_constraints_tree_values_join DO INSTEAD (
  DELETE FROM optistruct_technological_constraints_tree_values
  WHERE id = OLD.id
);

-- TRIGGERS
--
--CREATE FUNCTION optistruct_technological_constraints_tree_values_checks()
--  RETURNS TRIGGER AS $body$
--BEGIN
--
--  IF EXISTS(SELECT *
--            FROM optistruct_technological_constraints_tree
--            WHERE parent_id = NEW.tc_tree_id)
--  THEN
--    RAISE EXCEPTION 'Optistruct technological constraints field values must be leafs, tc_tree_id = %', NEW.tc_tree_id;
--  END IF;
--
--  IF NOT EXISTS(SELECT *
--                FROM optistruct_technological_constraints_tree
--                WHERE id = NEW.tc_tree_id AND is_value_editable = TRUE)
--  THEN
--    NEW.value := NULL;
--  END IF;
--
--  RETURN NEW;
--END;
--$body$ LANGUAGE plpgsql;
--
--CREATE TRIGGER optistruct_technological_constraints_tree_values_must_be_leafs_trigger
--BEFORE INSERT OR UPDATE ON optistruct_technological_constraints_tree_values
--FOR EACH ROW
--EXECUTE PROCEDURE optistruct_technological_constraints_tree_values_checks();

