DROP VIEW results_join;

ALTER TABLE results ADD COLUMN value VARCHAR(255);
ALTER TABLE results ADD COLUMN target_value VARCHAR(255);

CREATE VIEW results_join AS (
  SELECT r.*,
    u.name AS owner,

    (SELECT CASE
            WHEN parent_type = 'effectiveCharacteristic' THEN ec.id
            WHEN parent_type = 'simulation' THEN vs.id
            WHEN parent_type = 'optimization' THEN o.id
            ELSE NULL
            END) AS parent_id,

    (SELECT CASE
            WHEN parent_type = 'effectiveCharacteristic' THEN ec.name
            WHEN parent_type = 'simulation' THEN vs.name
            WHEN parent_type = 'optimization' THEN o.name
            ELSE NULL
            END) AS parent_name

  FROM results r
    INNER JOIN users u ON u.id = r.owner_id
    LEFT JOIN effective_characteristics ec ON ec.id = r.ec_id
    LEFT JOIN virtual_simulations vs ON vs.id = r.sim_id
    LEFT JOIN optimizations o ON o.id = r.optimization_id
);


CREATE OR REPLACE RULE results_join_CREATE AS ON INSERT TO results_join DO INSTEAD (
  INSERT INTO results (created_by, type, name, path_to_file, description, owner_id, value, target_value, parent_type, ec_id, sim_id, optimization_id)
  VALUES (NEW.created_by, NEW.type, NEW.name, NEW.path_to_file, NEW.description, NEW.owner_id, New.value, New.target_value, NEW.parent_type,
          CASE WHEN NEW.parent_type = 'effectiveCharacteristic' THEN NEW.parent_id ELSE NULL END,
          CASE WHEN NEW.parent_type = 'simulation' THEN NEW.parent_id ELSE NULL END,
          CASE WHEN NEW.parent_type = 'optimization' THEN NEW.parent_id ELSE NULL END)
  RETURNING
    results.*,
    NULL :: VARCHAR AS owner,
    NULL :: INTEGER AS parent_id,
    NULL :: VARCHAR(255) AS parent_name
);

CREATE OR REPLACE RULE results_join_UPDATE AS ON UPDATE TO results_join DO INSTEAD (
  UPDATE results
  SET created_by = NEW.created_by,
    type = NEW.type,
    path_to_file = NEW.path_to_file,
    name = NEW.name,
    description = NEW.description,
    owner_id = NEW.owner_id,
    value = NEW.value,
    target_value = NEW.target_value
  WHERE id = NEW.id);

CREATE OR REPLACE RULE results_join_DELETE AS ON DELETE TO results_join DO INSTEAD (
  DELETE FROM results WHERE id = OLD.id
);