CREATE TABLE construction_elements (
  id            INTEGER DEFAULT nextval('tree_seq' :: REGCLASS),
  pid           INTEGER                                             NOT NULL,
  owner_id      INTEGER                                             NOT NULL,
  path          VARCHAR []                                          NOT NULL,
  path_names    VARCHAR UNIQUE                                      NOT NULL,
  date_created  TIMESTAMP WITHOUT TIME ZONE DEFAULT now()           NOT NULL,
  date_modified TIMESTAMP WITHOUT TIME ZONE DEFAULT now()           NOT NULL,
  name          VARCHAR UNIQUE                                      NOT NULL,
  description   TEXT,
  image_file_id INTEGER,
  cad_model_id  INTEGER,

  PRIMARY KEY (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (image_file_id) REFERENCES files (id),
  FOREIGN KEY (cad_model_id) REFERENCES files (id),
  FOREIGN KEY (pid) REFERENCES folders (id)
);

CREATE TRIGGER construction_elements_path_create
BEFORE INSERT ON construction_elements
FOR EACH ROW
EXECUTE PROCEDURE create_path('ConstructionElement');


CREATE TRIGGER construction_elements_path_update
BEFORE UPDATE ON construction_elements
FOR EACH ROW
WHEN (OLD.name IS DISTINCT FROM NEW.name OR OLD.pid IS DISTINCT FROM NEW.pid)
EXECUTE PROCEDURE update_path('ConstructionElement');


CREATE VIEW construction_elements_join AS
  SELECT
    ce.*,
    image.name AS image_name,
    cad.name   AS cad_name
  FROM construction_elements ce
    LEFT OUTER JOIN files image
      ON ce.image_file_id = image.id
    LEFT OUTER JOIN files cad
      ON ce.cad_model_id = cad.id;


CREATE TABLE construction_elements_files (
  id                      SERIAL,
  construction_element_id INTEGER NOT NULL,
  file_id                 INTEGER NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (file_id) REFERENCES files (id),
  FOREIGN KEY (construction_element_id) REFERENCES construction_elements (id)
);


CREATE TABLE construction_elements_docs (
  id                      SERIAL,
  construction_element_id INTEGER NOT NULL,
  file_id                 INTEGER NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (file_id) REFERENCES files (id),
  FOREIGN KEY (construction_element_id) REFERENCES construction_elements (id)
);


CREATE VIEW construction_elements_docs_join AS
  SELECT
    ced.*,
    ce.name            AS construction_element_name,
    files.name         AS file_name,
    files.path_to_file AS file_source
  FROM construction_elements_docs ced
    JOIN construction_elements ce
      ON ced.construction_element_id = ce.id
    JOIN files
      ON ced.file_id = files.id;


CREATE TABLE adjacent_elements (
  id                               SERIAL,
  construction_element_id          INTEGER NOT NULL,
  adjacent_construction_element_id INTEGER NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (adjacent_construction_element_id) REFERENCES construction_elements (id),
  FOREIGN KEY (construction_element_id) REFERENCES construction_elements (id)
);


CREATE TABLE construction_element_file_types (
  id            INTEGER DEFAULT nextval('tree_seq' :: REGCLASS),
  pid           INTEGER,
  name          VARCHAR                                                     NOT NULL,
  owner_id      INTEGER                                                     NOT NULL,
  path          VARCHAR [], --NOT NULL,--
  path_names    VARCHAR UNIQUE, --NOT NULL,--
  date_created  TIMESTAMP WITHOUT TIME ZONE DEFAULT now()                   NOT NULL,
  date_modified TIMESTAMP WITHOUT TIME ZONE DEFAULT now()                   NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (pid) REFERENCES construction_elements (id)
);

CREATE TRIGGER construction_element_file_types_path_create
BEFORE INSERT ON construction_element_file_types
FOR EACH ROW
EXECUTE PROCEDURE create_path('ConstructionElementFolder');


CREATE TRIGGER construction_element_file_types_path_update
BEFORE UPDATE ON construction_element_file_types
FOR EACH ROW
WHEN (OLD.name IS DISTINCT FROM NEW.name OR OLD.pid IS DISTINCT FROM NEW.pid)
EXECUTE PROCEDURE update_path('ConstructionElementFolder');


CREATE TRIGGER construction_element_file_types_check_files
BEFORE DELETE ON construction_element_file_types
FOR EACH ROW
EXECUTE PROCEDURE check_files_parent_constraint();