CREATE OR REPLACE VIEW technologies_tree AS
  SELECT
    id,
    NULL :: INTEGER                AS pid,
    name,
    path,
    path_names,
    'Technology' :: tree_node_type AS item_type,
    NULL :: TEXT                   AS children_types
  FROM technologies t;
CREATE OR REPLACE RULE technologies_tree_update AS ON UPDATE TO technologies_tree DO INSTEAD (
  UPDATE technologies
  SET NAME = NEW.name
  WHERE id = NEW.id);


CREATE OR REPLACE VIEW materials_tree AS
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'Folder' :: tree_node_type AS item_type
  FROM folders
  WHERE tree_type = 'materials'
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'Material' :: tree_node_type AS item_type
  FROM materials
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'File' :: tree_node_type AS item_type
  FROM files
  WHERE file_type_id = 0;

CREATE OR REPLACE VIEW materials_tree_join AS
  SELECT
    mt.*,
    (SELECT string_agg(DISTINCT item_type :: TEXT, ',')
     FROM materials_tree mt2
     WHERE mt2.pid = mt.id) AS children_types
  FROM materials_tree mt;


CREATE OR REPLACE RULE materials_tree_update_materials AS ON UPDATE TO materials_tree_join DO INSTEAD (
  UPDATE materials
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE materials_tree_update_folders AS ON UPDATE TO materials_tree_join DO INSTEAD (
  UPDATE folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE materials_tree_update_files AS ON UPDATE TO materials_tree_join DO INSTEAD (
  UPDATE files
  SET NAME = NEW.name
  WHERE id = NEW.id);


CREATE OR REPLACE VIEW composite_materials_tree AS
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'Folder' :: tree_node_type AS item_type
  FROM folders
  WHERE tree_type = 'compositeMaterials'
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'CompositeMaterial' :: tree_node_type AS item_type
  FROM composite_materials
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'File' :: tree_node_type AS item_type
  FROM files
  WHERE file_type_id = 3;

CREATE OR REPLACE VIEW composite_materials_tree_join AS
  SELECT
    cmt.*,
    (SELECT string_agg(DISTINCT item_type :: TEXT, ',')
     FROM composite_materials_tree cmt2
     WHERE cmt2.pid = cmt.id) AS children_types
  FROM composite_materials_tree cmt;


CREATE OR REPLACE RULE composite_materials_tree_update_materials AS ON UPDATE TO composite_materials_tree_join DO INSTEAD (
  UPDATE composite_materials
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE composite_materials_tree_update_folders AS ON UPDATE TO composite_materials_tree_join DO INSTEAD (
  UPDATE folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE composite_materials_tree_update_files AS ON UPDATE TO composite_materials_tree_join DO INSTEAD (
  UPDATE files
  SET NAME = NEW.name
  WHERE id = NEW.id);


CREATE OR REPLACE VIEW construction_elements_tree AS
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'Folder' :: tree_node_type AS item_type
  FROM folders
  WHERE tree_type = 'constructionElements'
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'ConstructionElement' :: tree_node_type AS item_type
  FROM construction_elements
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'ConstructionElementFolder' :: tree_node_type AS item_type
  FROM construction_element_file_types
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'File' :: tree_node_type AS item_type
  FROM files
  WHERE file_type_id IN (8,9,10);

CREATE OR REPLACE VIEW construction_elements_tree_join AS
  SELECT
    cet.*,
    (SELECT string_agg(DISTINCT item_type :: TEXT, ',')
     FROM construction_elements_tree cet2
     WHERE cet2.pid = cet.id) AS children_types
  FROM construction_elements_tree cet;


CREATE OR REPLACE RULE construction_elements_tree_update_elements AS ON UPDATE TO construction_elements_tree_join DO INSTEAD (
  UPDATE construction_elements
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE construction_elements_tree_update_folders AS ON UPDATE TO construction_elements_tree_join DO INSTEAD (
  UPDATE folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE construction_elements_tree_update_files AS ON UPDATE TO construction_elements_tree_join DO INSTEAD (
  UPDATE files
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE VIEW optimizations_tree AS
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'Folder' :: tree_node_type AS item_type,
    NULL :: INTEGER            AS optimization_type_id,
    NULL :: INTEGER            AS construction_element_id
  FROM folders
  WHERE tree_type = 'constructionElements'
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'ConstructionElement' :: tree_node_type AS item_type,
    NULL :: INTEGER                         AS optimization_type_id,
    NULL :: INTEGER                         AS construction_element_id
  FROM construction_elements
  UNION
  SELECT
    id,
    CASE WHEN parent_folder_id IS NULL
      THEN construction_element_id
    ELSE parent_folder_id END              AS pid,
    name,
    path,
    path_names,
    'OptimizationFolder' :: tree_node_type AS item_type,
    optimization_type_id,
    construction_element_id
  FROM optimization_folders;

CREATE OR REPLACE VIEW optimizations_tree_join AS
  SELECT
    ot.*,
    (SELECT string_agg(DISTINCT item_type :: TEXT, ',')
     FROM optimizations_tree ot2
     WHERE ot2.pid = ot.id) AS children_types
  FROM optimizations_tree ot;


CREATE OR REPLACE RULE optimizations_tree_update_elements AS ON UPDATE TO optimizations_tree_join DO INSTEAD (
  UPDATE construction_elements
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE optimizations_tree_update_folders AS ON UPDATE TO optimizations_tree_join DO INSTEAD (
  UPDATE folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE optimizations_tree_update_opt_folders AS ON UPDATE TO optimizations_tree_join DO INSTEAD (
  UPDATE optimization_folders
  SET NAME = NEW.name
  WHERE id = NEW.id);


CREATE OR REPLACE VIEW tree AS
  SELECT *
  FROM materials_tree_join
  UNION
  SELECT *
  FROM composite_materials_tree_join
  UNION
  SELECT *
  FROM technologies_tree
  UNION
  SELECT *
  FROM construction_elements_tree_join
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    item_type,
    children_types
  FROM optimizations_tree_join;


CREATE OR REPLACE RULE tree_path_update_folders AS ON UPDATE TO tree DO INSTEAD (
  UPDATE folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_const_elem AS ON UPDATE TO tree DO INSTEAD (
  UPDATE construction_elements
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_file_types AS ON UPDATE TO tree DO INSTEAD (
  UPDATE construction_element_file_types
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_files AS ON UPDATE TO tree DO INSTEAD (
  UPDATE files
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_materials AS ON UPDATE TO tree DO INSTEAD (
  UPDATE materials
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_comp_materials AS ON UPDATE TO tree DO INSTEAD (
  UPDATE composite_materials
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_technologies AS ON UPDATE TO tree DO INSTEAD (
  UPDATE technologies
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_opt_folders AS ON UPDATE TO tree DO INSTEAD (
  UPDATE optimization_folders
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);
