CREATE TABLE solving_statuses (
  id    INTEGER                NOT NULL,
  name  CHARACTER VARYING(255) NOT NULL,
  uname CHARACTER VARYING(255) NOT NULL UNIQUE,

  PRIMARY KEY (id)
);
INSERT INTO solving_statuses VALUES (0, 'NotExecuted', 'Не выполнялся');

CREATE TABLE postprocessing_statuses (
  id    INTEGER                NOT NULL,
  name  CHARACTER VARYING(255) NOT NULL,
  uname CHARACTER VARYING(255) NOT NULL UNIQUE,

  PRIMARY KEY (id)
);
INSERT INTO postprocessing_statuses VALUES (0, 'NotExecuted', 'Не выполнялся');


ALTER TABLE optimizations ALTER COLUMN name DROP NOT NULL;
ALTER TABLE optimizations ADD COLUMN flag CHARACTER VARYING(255) DEFAULT 'none';
ALTER TABLE optimizations ADD COLUMN solving_status_id INTEGER NOT NULL DEFAULT 0 REFERENCES solving_statuses (id);
ALTER TABLE optimizations ADD COLUMN postprocessing_status_id INTEGER NOT NULL DEFAULT 0 REFERENCES postprocessing_statuses (id);
-- ALTER TABLE optimizations ADD COLUMN path VARCHAR [] DEFAULT NULL;
-- ALTER TABLE optimizations ADD COLUMN path_names VARCHAR DEFAULT NULL;

CREATE VIEW optimizations_join AS
  SELECT
    o.*,
    u.name                                                                                     AS owner,
    ss.uname                                                                                   AS solving_status,
    ps.uname                                                                                   AS postprocessing_status,
    ot.name                                                                                    AS optimization_type,
    opf.path_names || '/' :: VARCHAR || o.name :: VARCHAR                                      AS path_names,
    opf.path || concat('{', '"name":"', o.name, '", "id":', o.id, ', "type":null}') :: VARCHAR AS path,
    o2.name                                                                                    AS reference
  FROM optimizations o
    JOIN users u ON u.id = o.owner_id
    JOIN solving_statuses ss ON ss.id = o.solving_status_id
    JOIN postprocessing_statuses ps ON ps.id = o.postprocessing_status_id
    JOIN optimization_types ot ON ot.id = o.optimization_type_id
    JOIN optimization_folders opf ON opf.id = o.pid
    LEFT JOIN optimizations o2 ON o2.id = o.reference_id;

CREATE TABLE optimization_files (
  id              SERIAL,
  optimization_id INTEGER NOT NULL,
  file_id         INTEGER NOT NULL,
  ordering        INTEGER NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (optimization_id) REFERENCES optimizations (id),
  FOREIGN KEY (file_id) REFERENCES files (id)
);

ALTER TABLE ONLY optimization_files
    ADD CONSTRAINT un_file_id_ordering UNIQUE (file_id, ordering);

CREATE TABLE optimization_results (
  id              SERIAL,
  optimization_id INTEGER                                   NOT NULL,
  created_by      CHARACTER VARYING(255),
  type            CHARACTER VARYING(255)                    NOT NULL,
  name            CHARACTER VARYING(255)                    NOT NULL,
  path_to_file    CHARACTER VARYING,
  value           CHARACTER VARYING,
  date_created    TIMESTAMP WITHOUT TIME ZONE DEFAULT now() NOT NULL,
  owner_id        INTEGER                                   NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (optimization_id) REFERENCES optimizations (id),
  FOREIGN KEY (owner_id) REFERENCES users (id)
);

CREATE TABLE optimization_reports (
  id              SERIAL,
  name            CHARACTER VARYING(255)                    NOT NULL,
  optimization_id INTEGER                                   NOT NULL,
  created_by      CHARACTER VARYING(255),
  date_created    TIMESTAMP WITHOUT TIME ZONE DEFAULT now() NOT NULL,
  owner_id        INTEGER                                   NOT NULL REFERENCES users (id),

  PRIMARY KEY (id),
  FOREIGN KEY (optimization_id) REFERENCES optimizations (id),
  FOREIGN KEY (owner_id) REFERENCES users (id)
);