CREATE TABLE nodes (
    id SERIAL,
    name CHARACTER VARYING(255) NOT NULL,
    num_cores INTEGER,
    memory INTEGER,
    url CHARACTER VARYING(255) NOT NULL,
    description text,
    date_created TIMESTAMP without TIME ZONE DEFAULT now(),
    date_modified TIMESTAMP without TIME ZONE DEFAULT now(),
    owner_id INTEGER NOT NULL,
    alive BOOLEAN DEFAULT TRUE,

    PRIMARY KEY (id),
    FOREIGN KEY (owner_id) REFERENCES users(id)
);--#

-- triggers for auto updating date_modified column
CREATE TRIGGER nodes_last_modified
  BEFORE UPDATE ON nodes
  FOR EACH ROW
  WHEN (OLD.* IS DISTINCT FROM NEW.*)
    EXECUTE PROCEDURE update_last_modified();--#

CREATE TABLE nodes_solver_types (
    id SERIAL,
    node_id INTEGER NOT NULL,
    solver_name CHARACTER VARYING(255) NOT NULL,

    PRIMARY KEY (id),
    FOREIGN KEY (node_id) REFERENCES nodes(id)
);


CREATE VIEW nodes_alive AS
    SELECT o.*,
        users.name AS owner,
        (SELECT ARRAY_AGG(nst.solver_name) FROM nodes_solver_types nst) AS available_solvers
    FROM nodes o
        JOIN users ON o.owner_id = users.id;--#

CREATE OR REPLACE RULE nodes_alive_DELETE AS ON DELETE TO nodes_alive DO INSTEAD (
    DELETE FROM nodes WHERE id = OLD.id);--#

CREATE OR REPLACE RULE nodes_alive_CREATE AS ON INSERT TO nodes_alive DO INSTEAD (
  INSERT INTO nodes (name, num_cores, memory, url, description, date_created, date_modified, owner_id) VALUES
    (NEW.name, NEW.num_cores, NEW.memory, NEW.url, NEW.description, NEW.date_created, NEW.date_modified, NEW.owner_id)
  RETURNING nodes.*, NULL :: VARCHAR AS owner, NULL :: VARCHAR[] AS available_solvers);

CREATE OR REPLACE RULE nodes_alive_UPDATE AS ON UPDATE TO nodes_alive DO INSTEAD (
    UPDATE nodes
      SET name = NEW.name,
          num_cores = NEW.num_cores,
          memory = NEW.memory,
          url = NEW.url,
          description = NEW.description,
          date_created = NEW.date_created,
          date_modified = NEW.date_modified,
          owner_id = NEW.owner_id
    WHERE id = NEW.id);
