CREATE TABLE set_type (
  id SERIAL,
  name VARCHAR(255),

  PRIMARY KEY(id)
);

INSERT INTO set_type(id, name) VALUES (1, 'Elements'), (2, 'Nodes');

CREATE TABLE toska_sets (
  id SERIAL,
  name VARCHAR(255),
  is_optimization_space BOOLEAN,
  optimization_id INTEGER NOT NULL,
  set_type_id INTEGER NOT NULL,
  is_selected BOOLEAN NOT NULL DEFAULT FALSE,

  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(set_type_id) REFERENCES set_type(id) ON DELETE CASCADE,
  PRIMARY KEY(id)
);

CREATE VIEW toska_sets_join AS
  SELECT ts.*, st.name as set_type_name
  FROM toska_sets ts
    JOIN set_type st
      ON st.id = ts.set_type_id;

CREATE RULE toska_sets_join_CREATE AS ON INSERT TO toska_sets_join DO INSTEAD (
  INSERT INTO toska_sets (name, is_optimization_space, optimization_id, set_type_id, is_selected)
  VALUES (NEW.name, NEW.is_optimization_space, NEW.optimization_id, NEW.set_type_id, NEW.is_selected)
  RETURNING
    toska_sets.*,
    NULL :: VARCHAR(255) AS set_type_name
);

CREATE RULE toska_sets_join_UPDATE AS ON UPDATE TO toska_sets_join DO INSTEAD (
  UPDATE toska_sets
  SET name = NEW.name, is_optimization_space = NEW.is_optimization_space, set_type_id = NEW.set_type_id, is_selected = NEW.is_selected
  WHERE id = NEW.id);

CREATE RULE toska_sets_join_DELETE AS ON DELETE TO toska_sets_join DO INSTEAD (
  DELETE FROM toska_sets WHERE id = OLD.id
);
