CREATE TABLE toska_target_function_parameters (
  id SERIAL,
  name VARCHAR(255) NOT NULL,
  dresp_type VARCHAR(255) NOT NULL,
  node_or_element_id INTEGER NOT NULL,
  group_name VARCHAR(255) NOT NULL,

  PRIMARY KEY(id)
);

CREATE VIEW toska_constraints_parameters AS
  SELECT * FROM toska_target_function_parameters as ttfp
  WHERE ttfp.id NOT IN (4);

CREATE TABLE toska_optimization_target_functions (
  id SERIAL,
  name VARCHAR(255),
  parameter_id INTEGER NOT NULL DEFAULT 1,
  optimization_id INTEGER NOT NULL,
  type_id INTEGER, -- in TargetFunctionType enum
  el_group INTEGER,
  nd_group INTEGER,
  scalar VARCHAR(255),

  FOREIGN KEY(parameter_id) REFERENCES toska_target_function_parameters(id) ON DELETE CASCADE,
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(el_group) REFERENCES toska_sets(id) ON DELETE SET NULL,
  FOREIGN KEY(nd_group) REFERENCES toska_sets(id) ON DELETE SET NULL,
  PRIMARY KEY(id)
);

CREATE VIEW toska_target_functions_join AS
  SELECT ttf.*, ttfp.name as param_name, ttfp.dresp_type, ttfp.node_or_element_id
  FROM toska_optimization_target_functions ttf
    JOIN toska_target_function_parameters ttfp
      ON ttfp.id = ttf.parameter_id;

CREATE RULE toska_target_functions_join_CREATE AS ON INSERT TO toska_target_functions_join DO INSTEAD (
  INSERT INTO toska_optimization_target_functions (name, parameter_id, optimization_id, type_id, el_group, nd_group, scalar)
  VALUES (NEW.name, NEW.parameter_id, NEW.optimization_id, NEW.type_id, NEW.el_group, NEW.nd_group, NEW.scalar)
  RETURNING
    toska_optimization_target_functions.*,
    NULL :: VARCHAR(255) AS param_name,
    NULL :: VARCHAR(255) AS dresp_type,
    NULL :: INTEGER AS node_or_element_id
);

CREATE RULE toska_target_functions_join_UPDATE AS ON UPDATE TO toska_target_functions_join DO INSTEAD (
  UPDATE toska_optimization_target_functions
  SET name  = NEW.name, parameter_id = NEW.parameter_id, optimization_id = NEW.optimization_id,
    type_id = NEW.type_id, el_group = NEW.el_group, nd_group = NEW.nd_group, scalar = NEW.scalar
  WHERE id = NEW.id);

CREATE RULE toska_target_functions_join_DELETE AS ON DELETE TO toska_target_functions_join DO INSTEAD (
  DELETE FROM toska_optimization_target_functions WHERE id = OLD.id
);

------------------------------------------------------------------------------------------------------------

CREATE TABLE toska_optimization_constraints (
  id SERIAL,
  name VARCHAR(255),
  parameter_id INTEGER NOT NULL DEFAULT 1,
  optimization_id INTEGER NOT NULL,
  type_id INTEGER,
  value VARCHAR(255),
  isAbsolute BOOLEAN,
  el_group INTEGER,
  nd_group INTEGER,
  scalar VARCHAR(255),

  FOREIGN KEY(parameter_id) REFERENCES toska_target_function_parameters(id) ON DELETE CASCADE,
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(el_group) REFERENCES toska_sets(id) ON DELETE SET NULL,
  FOREIGN KEY(nd_group) REFERENCES toska_sets(id) ON DELETE SET NULL,
  PRIMARY KEY(id)
);

CREATE VIEW toska_constraints_join AS
  SELECT tc.*, ttfp.name as param_name, ttfp.dresp_type, ttfp.node_or_element_id
  FROM toska_optimization_constraints tc
    JOIN toska_constraints_parameters ttfp
      ON ttfp.id = tc.parameter_id;

CREATE RULE toska_constraints_join_CREATE AS ON INSERT TO toska_constraints_join DO INSTEAD (
  INSERT INTO toska_optimization_constraints (name, parameter_id, optimization_id, type_id, el_group, nd_group, scalar, value, isAbsolute)
  VALUES (NEW.name, NEW.parameter_id, NEW.optimization_id, NEW.type_id, NEW.el_group, NEW.nd_group, NEW.scalar, NEW.value, NEW.isAbsolute)
  RETURNING
    toska_optimization_constraints.*,
    NULL :: VARCHAR(255) AS param_name,
    NULL :: VARCHAR(255) AS dresp_type,
    NULL :: INTEGER AS node_or_element_id
);

CREATE RULE toska_constraints_join_UPDATE AS ON UPDATE TO toska_constraints_join DO INSTEAD (
  UPDATE toska_optimization_constraints
  SET name = NEW.name, parameter_id = NEW.parameter_id, optimization_id = NEW.optimization_id,
    type_id = NEW.type_id, el_group = NEW.el_group, nd_group = NEW.nd_group, scalar = NEW.scalar,
    value = NEW.value, isAbsolute = NEW.isAbsolute
  WHERE id = NEW.id);

CREATE RULE toska_constraints_join_DELETE AS ON DELETE TO toska_constraints_join DO INSTEAD (
  DELETE FROM toska_constraints_join WHERE id = OLD.id
);

INSERT INTO toska_target_function_parameters(id, name, dresp_type, node_or_element_id, group_name) VALUES
  (1,'масса конструкции','WEIGHT',1,'Базовые'),
  (2,'объём конструкции','VOLUME',1,'Базовые'),
  (3,'собственная частота','DYN_FREQ',5,'Базовые'),
  (4,'собственная частота по Крейсельмаеру','DYN_FREQ_KREISSEL',5,'Базовые'),
  (5,'средняя плотность','DENSITY_MEASURE',1,'Базовые'),
  (6,'модуль вектора перемещений в узле','DISP_ABS',2,'Смещение конструкции'),
  (7,'перемещение в узле в направлении OX','DISP_X',2,'Смещение конструкции'),
  (8,'перемещение в узле в направлении OY','DISP_Y',2,'Смещение конструкции'),
  (9,'перемещение в узле в направлении OZ','DISP_Z',2,'Смещение конструкции'),
  (10,'модуль перемещения в узле в направлении OX','DISP_X_ABS',2,'Смещение конструкции'),
  (11,'модуль перемещения в узле в направлении OY','DISP_Y_ABS',2,'Смещение конструкции'),
  (12,'модуль перемещения в узле в направлении OZ','DISP_Z_ABS',2,'Смещение конструкции'),
  (13,'Абсолютное значение поворота','ROT_ABS',2,'Смещение конструкции'),
  (14,'Поворот вокруг оси OX','ROT _X',2,'Смещение конструкции'),
  (15,'Поворот вокруг оси OY','ROT _Y',2,'Смещение конструкции'),
  (16,'Поворот вокруг оси OZ','ROT _Z',2,'Смещение конструкции'),
  (17,'Абсолютное значение поворота вокруг оси OX','ROT _X_ABS',2,'Смещение конструкции'),
  (18,'Абсолютное значение поворота вокруг оси OY','ROT _Y_ABS',2,'Смещение конструкции'),
  (19,'Абсолютное значение поворота вокруг оси OZ','ROT _Z_ABS',2,'Смещение конструкции'),
  (20,'Средняя плотность энергии','ENERGY_STIFF_MEASURE',6,'Напряжения и деформации'),
  (21,'Внутренняя энергия деформации','STRAIN_ENERGY',4,'Напряжения и деформации'),
  (22,'Нормированные напряжения по Мизесу','SIG_TOPO_MISES',1,'Напряжения и деформации'),
  (23,'центр тяжести в направлении OX','CENTER_GRAVITY_X',1,'Инерционные и массовые характеристики'),
  (24,'центр тяжести в направлении OY','CENTER_GRAVITY_Y',1,'Инерционные и массовые характеристики'),
  (25,'центр тяжести в направлении OZ','CENTER_GRAVITY_Z',1,'Инерционные и массовые характеристики'),
  (26,'Момент инерции XX','INERTIA_XX',1,'Инерционные и массовые характеристики'),
  (27,'Момент инерции XY','INERTIA_XY',1,'Инерционные и массовые характеристики'),
  (28,'Момент инерции XZ','INERTIA_XZ',1,'Инерционные и массовые характеристики'),
  (29,'Момент инерции YY','INERTIA_YY',1,'Инерционные и массовые характеристики'),
  (30,'Момент инерции YZ','INERTIA_YZ',1,'Инерционные и массовые характеристики'),
  (31,'Момент инерции ZZ','INERTIA_ZZ',1,'Инерционные и массовые характеристики'),
  (32,'Модуль вектора силы реакции','REACTION_FORCE_ABS',2,'Силы и моменты'),
  (33,'Сила реакции в направлении OX','REACTION_FORCE_X',2,'Силы и моменты'),
  (34,'Сила реакции в направлении OY','REACTION_FORCE_Y',2,'Силы и моменты'),
  (35,'Сила реакции в направлении OZ','REACTION_FORCE_Z',2,'Силы и моменты'),
  (36,'Модуль силы реакции в направлении OX','REACTION_FORCE_X_ABS',2,'Силы и моменты'),
  (37,'Модуль силы реакции в направлении OY','REACTION_FORCE_Y_ABS',2,'Силы и моменты'),
  (38,'Модуль силы реакции в направлении OZ','REACTION_FORCE_Z_ABS',2,'Силы и моменты'),
  (39,'Модуль вектора момента реакции','REACTION_MOMENT_ABS',2,'Силы и моменты'),
  (40,'Момент реакции в направлении OX','REACTION_MOMENT_X',2,'Силы и моменты'),
  (41,'Момент реакции в направлении OY','REACTION_MOMENT_Y',2,'Силы и моменты'),
  (42,'Момент реакции в направлении OZ','REACTION_MOMENT_Z',2,'Силы и моменты'),
  (43,'Модуль момента реакции в направлении OX','REACTION_MOMENT_X_ABS',2,'Силы и моменты'),
  (44,'Модуль момента реакции в направлении OY','REACTION_MOMENT_Y_ABS',2,'Силы и моменты'),
  (45,'Модуль момента реакции в направлении OZ','REACTION_MOMENT_Z_ABS',2,'Силы и моменты'),
  (46,'Модуль вектора силы в узле','INTERNAL_FORCE_ABS',3,'Силы и моменты'),
  (47,'Сила в узле в направлении OX','INTERNAL_FORCE_X',3,'Силы и моменты'),
  (48,'Сила в узле в направлении OY','INTERNAL_FORCE_Y',3,'Силы и моменты'),
  (49,'Сила в узле в направлении OZ','INTERNAL_FORCE_Z',3,'Силы и моменты'),
  (50,'Модуль силы в узле в направлении OX','INTERNAL_FORCE_X_ABS',3,'Силы и моменты'),
  (51,'Модуль силы в узле в направлении OY','INTERNAL_FORCE_Y_ABS',3,'Силы и моменты'),
  (52,'Модуль силы в узле в направлении OZ','INTERNAL_FORCE_Z_ABS',3,'Силы и моменты'),
  (53,'Модуль вектора момента в узле','INTERNAL_MOMENT_ABS',3,'Силы и моменты'),
  (54,'Момент в узле в направлении OX','INTERNAL_ MOMENT _X',3,'Силы и моменты'),
  (55,'Момент в узле в направлении OY','INTERNAL_ MOMENT _Y',3,'Силы и моменты'),
  (56,'Момент в узле в направлении OZ','INTERNAL_ MOMENT _Z',3,'Силы и моменты'),
  (57,'Модуль момента в узле в направлении OX','INTERNAL_ MOMENT _X_ABS',3,'Силы и моменты'),
  (58,'Модуль момента в узле в направлении OY','INTERNAL_ MOMENT _Y_ABS',3,'Силы и моменты'),
  (59,'Модуль момента в узле в направлении OZ','INTERNAL_ MOMENT _Z_ABS',3,'Силы и моменты'),
  (60,'АЧХ - ускорение в направлении OX','FS_ACCEL_X',2,'Спектральный и акустический анализ'),
  (61,'АЧХ - ускорение в направлении OY','FS_ACCEL_Y',2,'Спектральный и акустический анализ'),
  (62,'АЧХ - ускорение в направлении OZ','FS_ACCEL_Z',2,'Спектральный и акустический анализ'),
  (63,'давление звука (акустический анализ, дБА)','FS_DBA_PRESSURE',2,'Спектральный и акустический анализ'),
  (64,'давление звука (акустический анализ, дБ)','FS_DB_PRESSURE',2,'Спектральный и акустический анализ'),
  (65,'АЧХ - модуль вектора перемещений','FS_DISP_ABS',2,'Спектральный и акустический анализ'),
  (66,'АЧХ - модуль перемещения в направлении OX','FS_DISP_X_ABS',2,'Спектральный и акустический анализ'),
  (67,'АЧХ - модуль перемещения в направлении OY','FS_DISP_Y_ABS',2,'Спектральный и акустический анализ'),
  (68,'АЧХ - модуль перемещения в направлении OZ','FS_DISP_Z_ABS',2,'Спектральный и акустический анализ'),
  (69,'ФЧХ в направлении OX','FS_PHASE_X',2,'Спектральный и акустический анализ'),
  (70,'ФЧХ в направлении OY','FS_PHASE_Y',2,'Спектральный и акустический анализ'),
  (71,'ФЧХ в направлении OZ','FS_PHASE_Z',2,'Спектральный и акустический анализ'),
  (72,'Мгновенное звуковое давление, Па (акустический анализ)','FS_PRESSURE',2,'Спектральный и акустический анализ'),
  (73,'Эффективное звуковое давление, Па (акустический анализ)','FS_RMS_PRESSURE',2,'Спектральный и акустический анализ'),
  (74,'АЧХ - скорость в направлении OX','FS_VELOCITY_X',2,'Спектральный и акустический анализ'),
  (75,'АЧХ - скорость в направлении OY','FS_VELOCITY_Y',2,'Спектральный и акустический анализ'),
  (76,'АЧХ - скорость в направлении OZ','FS_VELOCITY_Z',2,'Спектральный и акустический анализ');
