CREATE TABLE file_types (
  id   INTEGER,
  name VARCHAR UNIQUE,

  PRIMARY KEY (id)
);

INSERT INTO file_types (id, name) VALUES
  (0, 'MaterialFile'),
  (1, 'SampleFile'),
  (2, 'ParameterFile'),
  (3, 'CompositeMaterialFile'),
  (4, 'ImageFile'),
  (5, 'DescriptionFile'),
  (6, 'StandardFile'),
  (7, 'StandardLoadFile'),
  (8, 'CADFile'),
  (9, 'CAEFile'),
  (10, 'CAOFile'),
  (11, 'DesignSolutionFile'),
  (12, 'LoadCondition');


-- material files
CREATE TABLE files (
  id            INTEGER DEFAULT nextval('tree_seq' :: REGCLASS),
  pid           INTEGER                                    NOT NULL,
  file_type_id  INTEGER                                    NOT NULL,
  reference_id  INTEGER,
  name          VARCHAR                                    NOT NULL,
  path_to_file  VARCHAR,
  path          VARCHAR [], --NOT NULL,--
  path_names    VARCHAR UNIQUE, --NOT NULL,--
  description   TEXT,
  flag          VARCHAR DEFAULT 'none',
  owner_id      INTEGER                                    NOT NULL,
  date_created  TIMESTAMP WITHOUT TIME ZONE DEFAULT now()  NOT NULL,
  date_modified TIMESTAMP WITHOUT TIME ZONE DEFAULT now()  NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (reference_id) REFERENCES files (id),
  FOREIGN KEY (file_type_id) REFERENCES file_types (id)
);

CREATE UNIQUE INDEX unique_name_for_material_files ON files(name) WHERE (file_type_id = 0);

CREATE TRIGGER files_path_create
BEFORE INSERT ON files
FOR EACH ROW
EXECUTE PROCEDURE create_path('File');

CREATE TRIGGER files_path_update
BEFORE UPDATE ON files
FOR EACH ROW
WHEN (OLD.name IS DISTINCT FROM NEW.name OR OLD.pid IS DISTINCT FROM NEW.pid)
EXECUTE PROCEDURE update_path('File');

CREATE TRIGGER files_last_modified
BEFORE UPDATE ON files
FOR EACH ROW
WHEN (OLD.* IS DISTINCT FROM NEW.*)
EXECUTE PROCEDURE update_last_modified();


CREATE VIEW files_join AS
  SELECT
    mf.*,
    u.name   AS owner,
    ft.name  AS file_type,
    mf2.name AS reference
  FROM files mf
    JOIN users u ON mf.owner_id = u.id
    JOIN file_types ft ON mf.file_type_id = ft.id
    LEFT JOIN files mf2 ON mf.reference_id = mf2.id;
