CREATE TABLE optistruct_additional_parameters_types (
  id SERIAL,
  name VARCHAR(255) NOT NULL,
  code_name VARCHAR(255) NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_additional_parameters_types(id, name, code_name) VALUES
  (1, 'Произвольно', 'PLAIN'),
  (2, 'SQP', 'SQP'),
  (3, 'MFD', 'MFD'),
  (4, 'DUAL', 'DUAL'),
  (5, 'BIGOPT', 'BIGOPT'),
  (6, 'Да', '3'),
  (7, 'Нет', '2'),
  (8, 'Треугольники', '0'),
  (9, 'Квадраты', '1'),
  (10, 'Смешанный тип', '2'),
  (11, 'Правильные треугольники', '3');

CREATE TABLE optistruct_additional_parameters_groupes (
  id SERIAL,
  name VARCHAR(255) NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_additional_parameters_groupes VALUES
  (1, 'Optimization'),
  (2, 'Postprocessing');

CREATE TABLE optistruct_additional_parameters (
  id SERIAL,
  name VARCHAR(255) NOT NULL,
  group_id INTEGER NOT NULL,

  PRIMARY KEY(id),
  FOREIGN KEY(group_id) REFERENCES optistruct_additional_parameters_groupes(id)
);

INSERT INTO optistruct_additional_parameters(id, name, group_id) VALUES
  (1, 'Максимальное количество итераций', 1),
  (2, 'Шаг оптимизации', 1),
  (3, 'Критерий сходимости', 1),
--   (4, 'Параметр дискретности', 1),
--   (5, 'Метод оптимизации', 1),
--   (6, 'Минимальная плотность', 1),
--   (7, 'Контроль получения шахматной структуры при оптимизации', 1),
  (8, 'Отсечка плотности', 2),
  (9, 'Число итераций при сглаживании модели', 2),
  (10, 'Угол при сглаживании', 2),
  (11, 'Уточнение сетки', 2),
  (12, 'Размер элемента, при перестроении сетки', 2),
  (13, 'Тип элемента, при перестроении сетки', 2),
  (14, 'Угол, используемый при перестроении сетки', 2)
;

CREATE TABLE optistruct_additional_parameters_allowed_types (
  id SERIAL,
  parameter_id INTEGER NOT NULL,
  type_id INTEGER NOT NULL,

  FOREIGN KEY(parameter_id) REFERENCES optistruct_additional_parameters(id),
  FOREIGN KEY(type_id) REFERENCES optistruct_additional_parameters_types(id)
);

INSERT INTO optistruct_additional_parameters_allowed_types(parameter_id, type_id) VALUES
  (1, 1), (2, 1), (3, 1),
--   (4, 1), (5, 2), (5, 3), (5, 4), (5, 5), (6, 1), (7, 1),
  (8, 1), (9, 1), (10, 1), (11, 6), (11, 7), (12, 1), (13, 8), (13, 9), (13, 10), (13, 11), (14, 1);

CREATE TABLE optistruct_optimization_additional_parameters (
  id SERIAL,
  optimization_id INTEGER NOT NULL,
  parameter_id INTEGER NOT NULL,
  parameter_type_id INTEGER NOT NULL,
  value VARCHAR(255),
  name VARCHAR(255),

  PRIMARY KEY(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(parameter_id) REFERENCES optistruct_additional_parameters(id),
  FOREIGN KEY(parameter_type_id) REFERENCES optistruct_additional_parameters_types(id),
  UNIQUE (optimization_id, parameter_id)
);

CREATE FUNCTION check_optistruct_additional_parameter_allowed()
  RETURNS TRIGGER AS $body$
BEGIN
  IF NOT EXISTS(SELECT *
                FROM optistruct_additional_parameters_allowed_types
                WHERE parameter_id = NEW.parameter_id AND type_id = NEW.parameter_type_id)
  THEN
    RAISE EXCEPTION 'That parameter and type combination is not allowed';
  END IF;
  RETURN NEW;
END;
$body$ LANGUAGE plpgsql;

CREATE TRIGGER optistruct_optimization_additional_parameters_check_parameter_allowed
BEFORE INSERT OR UPDATE ON optistruct_optimization_additional_parameters
FOR EACH ROW
EXECUTE PROCEDURE check_optistruct_additional_parameter_allowed();

CREATE VIEW optistruct_optimization_additional_parameters_join AS (
  SELECT base.*,
    parameter.name AS parameter_name,
    parameter_type.name AS parameter_type_name,
    parameter_type.code_name AS parameter_type_code_name,
    parameter.group_id

  FROM optistruct_optimization_additional_parameters base
    JOIN optistruct_additional_parameters parameter ON base.parameter_id = parameter.id
    JOIN optistruct_additional_parameters_types parameter_type ON base.parameter_type_id = parameter_type.id
);

CREATE RULE optistruct_optimization_additional_parameters_join_CREATE
AS ON INSERT TO optistruct_optimization_additional_parameters_join DO INSTEAD (
  INSERT INTO optistruct_optimization_additional_parameters (optimization_id, parameter_id, parameter_type_id, value, name)
  VALUES (NEW.optimization_id, NEW.parameter_id, NEW.parameter_type_id, NEW.value, NEW.name)
  RETURNING
    optistruct_optimization_additional_parameters.*,
    NULL :: VARCHAR(255) AS parameter_name,
    NULL :: VARCHAR(255) AS parameter_type_name,
    NULL :: VARCHAR(255) AS parameter_type_code_name,
    NULL :: INTEGER AS group_id
);

CREATE RULE optistruct_optimization_additional_parameters_join_UPDATE
AS ON UPDATE TO optistruct_optimization_additional_parameters_join DO INSTEAD (
  UPDATE optistruct_optimization_additional_parameters
  SET parameter_id = NEW.parameter_id, parameter_type_id = NEW.parameter_type_id, value = NEW.value, name = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_additional_parameters_join_DELETE
AS ON DELETE TO optistruct_optimization_additional_parameters_join DO INSTEAD (
  DELETE FROM optistruct_optimization_additional_parameters WHERE id = OLD.id
);
