CREATE TABLE optistruct_component_type (
  id SERIAL,
  name VARCHAR(255),

  PRIMARY KEY(id)
);

INSERT INTO optistruct_component_type(id, name) VALUES (1, 'PSHELL'), (2, 'PSOLID'), (3, 'PCOMP');

CREATE TABLE optistruct_sets (
  id SERIAL,
  name VARCHAR(255),
  is_optimization_space BOOLEAN,
  optimization_id INTEGER NOT NULL,
  component_type_id INTEGER NOT NULL,
  is_selected BOOLEAN NOT NULL DEFAULT FALSE,

  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(component_type_id) REFERENCES optistruct_component_type(id) ON DELETE CASCADE,
  PRIMARY KEY(id)
);

CREATE VIEW optistruct_sets_join AS
  SELECT ts.*, st.name as component_type_name
  FROM optistruct_sets ts
    JOIN optistruct_component_type st
      ON st.id = ts.component_type_id;

CREATE RULE optistruct_sets_join_CREATE AS ON INSERT TO optistruct_sets_join DO INSTEAD (
  INSERT INTO optistruct_sets (name, is_optimization_space, optimization_id, component_type_id, is_selected)
  VALUES (NEW.name, NEW.is_optimization_space, NEW.optimization_id, NEW.component_type_id, NEW.is_selected)
  RETURNING
    optistruct_sets.*,
    NULL :: VARCHAR(255) AS component_type_name
);

CREATE RULE optistruct_sets_join_UPDATE AS ON UPDATE TO optistruct_sets_join DO INSTEAD (
  UPDATE optistruct_sets
  SET name = NEW.name, is_optimization_space = NEW.is_optimization_space, component_type_id = NEW.component_type_id, is_selected = NEW.is_selected
  WHERE id = NEW.id);

CREATE RULE optistruct_sets_join_DELETE AS ON DELETE TO optistruct_sets_join DO INSTEAD (
  DELETE FROM optistruct_sets WHERE id = OLD.id
);