INSERT INTO effective_characteristics_parameters(id, name, type, parent_param_id, is_composite) VALUES
  (13, 'Средняя толщина слоя', 'Mesostructure', null, FALSE);

CREATE FUNCTION check_ec_average_thickness_allowed()
  RETURNS TRIGGER AS $body$
BEGIN
  --  average_width is used only for effective_characteristics with technologies with 'ADDITIVE_3CM' type
  IF NOT EXISTS(
      SELECT *
      FROM effective_characteristics ec
        JOIN composite_materials cm
          ON ec.composite_material_id = cm.id
        JOIN technologies t
          ON cm.technology_id = t.id
      WHERE ec.id = NEW.effective_characteristic_id
            AND cm.id = ec.composite_material_id
            AND t.technology_type = 'ADDITIVE_3CM')
  THEN
    NEW.average_layer_width := NULL;
  END IF;
  RETURN NEW;
END;
$body$ LANGUAGE plpgsql;

CREATE TRIGGER effective_characteristics_parameters_join_check_ec_average_thickness_allowed
BEFORE INSERT OR UPDATE ON effective_characteristics_parameters_join
FOR EACH ROW
EXECUTE PROCEDURE check_ec_average_thickness_allowed();

DROP VIEW effective_characteristics_join;
CREATE VIEW effective_characteristics_join AS (
  SELECT
    ec.*,
    u.name                AS owner,
    cm.name               AS composite_material_name,
    mf.name               AS microstructure_file_name,
    (SELECT COUNT(*)
     FROM results
     WHERE ec_id = ec.id) AS results_count,
    EXISTS(SELECT *
           FROM technologies t
           WHERE cm.technology_id = t.id
                 AND t.technology_type = 'ADDITIVE_3CM') as average_thickness_is_setable
  FROM effective_characteristics ec
    JOIN users u ON u.id = ec.owner_id
    JOIN composite_materials cm ON cm.id = ec.composite_material_id
    LEFT JOIN files mf ON mf.id = ec.microstructure_file_id
);
