(function () {
  "use strict";

  var ColumnRow = Marionette.ItemView.extend({
    tagName: "li",

    attributes: function () {
      var that = this;
      return {
        id: function () {
          return that.model.get('name');
        }
      };
    },

    className: "",

    template: _.template('<a class="fa-icon menu-item <%- checkedClass %>"><%- label %> <i class="fa <%= checkedIcon %> pull-right"></i> </a>'),

    events: {
      "click a": "onClickMenuItem",
      "drop": "dropItem"
    },

    modelEvents: {
      "change renderable": 'toggleState'
    },

    initialize: function (options) {
      this.options = options || {};
    },

    dropItem: function (e, idx) {
      this.trigger('update-sort', {m: this.model, idx: idx});
    },

    toggleState: function () {
      var $columnMenuItem = this.$el.children('a'),
        $icon = $columnMenuItem.children('i'),
        newState = this.model.get('renderable');

      if (newState) {
        $columnMenuItem.addClass("checked", newState);
        $icon.removeClass('fa-square-o').addClass('fa-check-square-o');
      } else {
        $columnMenuItem.removeClass("checked", newState);
        $icon.addClass('fa-square-o').removeClass('fa-check-square-o');
      }
    },

    serializeData: function () {
      return {
        cid: this.model.cid,
        label: this.model.get('label'),
        checkedIcon: this.options.checked ? "fa-check-square-o" : "fa-square-o",
        checkedClass: this.options.checked ? "checked" : ""
      };
    },

    onClickMenuItem: function (e) {
      e.preventDefault();
      e.stopPropagation();

      var $columnMenuItem = $(e.currentTarget);

      var currentState = $columnMenuItem.hasClass('checked'),
        newState = !currentState;

      this.trigger('toggle:column', newState, this.$el);
    }
  });

  var FilterItemView = Marionette.ItemView.extend({

    template: "./views/templates/backgrid/filter_item.html.ejs",

    className: "form-horizontal filter-item col-sm-3",

    ui: {
      filterInput: 'input',
      removeBtn: '.remove-filter',
      clearBtn: '.clearer'
    },

    events: {
      "click @ui.removeBtn": 'onClickRemove',
      "keyup @ui.filterInput": 'onChangeFilter',
      'click @ui.clearBtn': 'onClickClear'
    },

    onClickRemove: function (e) {
      e.preventDefault();

      this.trigger('removeFilter');
    },

    onClickClear: function (e) {
      e.preventDefault();

      this.ui.filterInput.val('');
      this.ui.filterInput.trigger('keyup');
    },

    onChangeFilter: function (e) {
      e.preventDefault();
      this.ui.filterInput.next('span').toggle(Boolean(this.ui.filterInput.val()));

      var len = this.ui.filterInput.val().length;
      if (len > 0 && len < 3) {
        return;
      }
      this.model.set('filterData', this.ui.filterInput.val());
      this.trigger('runFilter');
    }

  });

  var FiltersPanelView = Marionette.CompositeView.extend({

    className: 'filter-items-wrapper',

    template: _.template(''),

    childEvents: {
      'removeFilter': 'onRemoveFilter',
      'runFilter': 'onRunFilter'
    },

    initialize: function (options) {
      this.options = options || {};
    },

    onRemoveFilter: function (view) {
      view.model.set('renderable', false);
      this.collection.remove(view.model);
      this.onRunFilter();
    },

    onRunFilter: function (view, value) {
      var filters = this.collection.filter(function (o, k) {
        return o.get('filterData');
      });

      filters = _.map(filters, function (o, k) {
        var f = [];

        var attrName = o.get('name');
        var attdData = o.get('filterData');
        var column = this.options.backGrid.columns.findWhere({'name': attrName});

        //TODO need get type, but where from?
        attdData = "%" + attdData + "%";

        f.push(attrName);
        f.push(attdData);
        return f;
      }, this);

      filters = _.object(filters);

      this.options.backGrid.collection.setQueryParams({filters: filters});
      this.options.backGrid.collection.fetch();
    },

    childView: FilterItemView
  });

  var FilterChooserView = Marionette.CompositeView.extend({
    template: _.template('<li class="ui-state-disabled drag-disabled"></li>'),

    tagName: 'ul',

    className: "dropdown-menu grid-columns",

    childEvents: {
      'toggle:column': "toggleColumn"
    },

    attributes: {
      "role": 'menu'
    },

    initialize: function (options) {
      this.options = options || {};
    },

    toggleColumn: function (view, checked, $el) {
      var that = this;
      view.model.set('renderable', checked);
      view.model.set('filterData', null);

      if (checked) {
        this.options.panelCollection.add(view.model);
      } else {
        this.options.panelCollection.remove(view.model);
      }
    },

    childViewOptions: function (model, index) {
      return {
        checked: model.get('renderable'),
        childIndex: index
      }
    },

    childView: ColumnRow

  });

  var ColumnChooserView = Marionette.CompositeView.extend({
    template: _.template('<li class="ui-state-disabled drag-disabled"><a class="show-all-columns">Показать все</a></li>'),

    tagName: "ul",

    className: "dropdown-menu grid-columns",

    attributes: {
      "role": "menu"
    },

    childEvents: {
      "update-sort": 'updateSort',
      'toggle:column': "toggleColumn"
    },

    ui: {
      "showAllColumns": ".show-all-columns"
    },

    events: {
      "click @ui.showAllColumns": "showAllTableColumns"
    },

    updateSort: function (e, options) {
      this.collection.remove(options.m);

      this.collection.each(function (model, index) {
        var ordinal = index;

        if (index >= options.idx) {
          ordinal += 1;
        }
        model.set('ordinal', ordinal);
      });

      options.m.set('ordinal', options.idx);

      this.collection.add(options.m, {at: options.idx});

    },
    onRender: function () {
      var that = this;
      this.$el.sortable({
        placeholder: "ui-state-highlight",
        items: "li:not(.drag-disabled)",
        stop: function (e, ui) {
          ui.item.trigger('drop', ui.item.index());
        }
      });
      this.$el.disableSelection();
    },

    initialize: function (options) {
      this.options = options || {};
      this.collection = options.gridView.BackGrid.columns;
      this.listenTo(this.collection, 'add', this.saveOrder);
    },

    saveOrder: function () {
      var sortedNames = _.without(this.collection.invoke('get', 'name'), "");

      $.ajax({
        type: 'PUT',
        contentType: 'application/json',
        url: this.options.gridView.BackGrid.collection.columnsUrl() + "/order",
        data: JSON.stringify(sortedNames)
      })
    },

    childViewOptions: function (model, index) {
      return {
        checked: model.get('renderable'),
        childIndex: index
      }
    },

    addChild: function (child, ChildViewClass, childViewOptions) {
      if (!_.isEmpty(child.get('name'))) {
        Marionette.CollectionView.prototype.addChild.apply(this, arguments);
      }
    },

    buildChildView: function (child, ChildViewClass, childViewOptions) {
      childViewOptions = childViewOptions || {};

      var options = _.extend({model: child}, childViewOptions);
      var view = new ChildViewClass(options);
      return view;
    },

    showAllTableColumns: function (e) {
      e.preventDefault();

      this.$('i.fa').removeClass('fa-square-o').addClass('fa-check-square-o');
      this.$('a.menu-item').addClass("checked", true);

      var that = this;
      $.ajax(this.options.gridView.backgridCollection.columnsUrl(), {
        type: "DELETE",
        success: function (data) {
          that.collection.invoke('set', {'renderable': true})
        }
      })
    },
    toggleColumn: function (view, checked, $el) {
      var that = this;
      view.model.set('renderable', checked);

      var entityModel = new Backbone.Model({
        "name": view.model.get('name'),
        'renderable': checked,
        'label': view.model.get('label'),
        'width': -1
      });
      entityModel.urlRoot = that.options.gridView.BackGrid.collection.columnsUrl();
      entityModel.save([], {
        type: "PUT",
        success: function () {
          view.model.set('renderable', checked);
        }
      });
    },
    childView: ColumnRow
  });

  /**
   * @desc has default values for grid tools
   * @options
   * BackgridCollection 'gridCollection'
   * bool 'showTools' - default true
   * bool 'showPageableTools' - default true
   * bool 'showColumnsTool' - default true
   * bool 'showFiltersTool' - default true
   * bool 'fetchGridData' - default true
   */
  window.App.Views.BackgridBasePageView = Marionette.LayoutView.extend({
    template: "./views/templates/project/project_base_page.html.ejs",
    contextMenuTemplate: null,
    className: "ui-backgrid-table clearfix",
    titlePageSizeTemplate: "./views/templates/backgrid/title_page_size.html.ejs",
    ui: {
      gridFilter: "#grid-filter",
      gridPaginatorTop: "#grid-paginator-top",
      gridPaginatorBottom: "#grid-paginator-bottom",
      createNewBtn: "#create-new",
      tableTools: ".table-tools",
      contextMenu: ".context-menu",
      pageSizeInput: ".js-page-size",
      totalSizeLabel: "#js-total-size",
      titlePageSize: ".js-page-size-wrapper",
      columnsBtn: ".js-grid-columns",
      filtersBtn: ".js-grid-filters",
      columnsDropdown: ".columns-dropdown",
      filtersDropdown: ".js-filters-dropdown"
    },

    regions: {
      createToolRegion: "#create-tool-region",
      gridMain:         "#grid-main",
      filtersPanel:     ".js-filters-panel",
      actionsRegion:    "#js-actions-region"
    },

    serializeData: function () {
      var data = {};
      data.title = this.title;
      data.createButtonTitle = this.createButtonTitle;

      return _.extend(data, this.model ? this.model.toJSON() : {})
    },

    behaviors: {
      PluginsInit: {
        select2: [
          new window.App.Helpers.Select2Config({
            el: "input#js-page-size",
            placeholder: "Set page size",
            local: true,
            minimumInputLength: 0
          })
        ]
      }
    },

    events: {
      "click @ui.columnsBtn":     "onClickColumnsBtn",
      "click @ui.filtersBtn":     "onClickFiltersBtn",
      "click @ui.createNewBtn":   "onClickCreateNew",
      "change @ui.pageSizeInput": "onChangePageSize"
    },

    initialize: function (options) {
      this.options = options || {};
      this.title = this.options.title || this.title;
      this.options.showTools = (options.showTools !== false);
      this.options.showPageableTools = (options.showPageableTools !== false);
      this.options.showColumnsTool = (options.showColumnsTool !== false);
      this.options.showFiltersTool = (options.showFiltersTool !== false);
      this.options.fetchGridData = (options.fetchGridData !== false);
      this.options.showPageSize = (options.showPageSize !== false);

      if (typeof options.gridCollection === "function") {
        this.gridCollection = new (options.gridCollection);
      } else if (typeof options.gridCollection === "object") {
        this.gridCollection = options.gridCollection;
      } else if (typeof options.gridCollection === "undefined") {
        this.gridCollection = new window.App.Collections.GridBaseCollection(options);
      }

      if (this.gridCollection.getModel) {
        this.gridModel = this.gridCollection.getModel();
      } else {
        this.gridModel = Backbone.Model;
      }

      this.actions = this.options.actions || [];
      this.type = this.options.type || this.gridCollection.options.fileType || null;
      this.isFile = this.options.isFile || false;

      this.gridView = new window.App.Views.BackgridCoreView(_.extend(options || {}, {gridCollection: this.gridCollection, columns: this.columns}));
      this.gridView.BackGrid.sortableRows = this.options.sortableRows;

      this.listenTo(this.gridView.BackGrid.collection, "sync", this.setTotalSize);
      this.listenTo(this.gridView.BackGrid.collection, "reset", this.setTotalSize);
      this.listenTo(this.gridView.BackGrid.collection, "add", this.setTotalSize);
      this.listenTo(this.gridView.BackGrid.collection, "remove", this.setTotalSize);

      this.listenTo(this, 'resizable', this.setGridResizable);
      _.bind(this.createFilterDropdown, this);
    },


    onChangePageSize: function (e) {
      this.gridCollection.setDefaultParams(); // Implemented: 27.08: Without this call gridCollection loses it's queryParams on fetch
      this.gridCollection.setPageSize(parseInt($(e.target).val()), {first: false});
    },

    onClickColumnsBtn: function (e) {
      e.preventDefault();
    },

    onClickFiltersBtn: function (e) {
      e.preventDefault();
    },

    createColumnsDropdown: function () {
      var that = this;
      var columns = this.gridView.getColumns();

      var initListener = function (chooser) {
        that.listenTo(chooser, "render", function () {
          that.ui.columnsBtn.removeClass("disabled");
          that.ui.columnsDropdown.append(chooser.$el);
        });

        chooser.render();
      };

      if (columns && !columns.fetched) {
        this.listenToOnce(columns, 'sync', function () {
          initListener(new ColumnChooserView({gridView: that.gridView}));
        })
      } else {
        initListener(new ColumnChooserView({gridView: that.gridView}));
      }

    },

    createFilterDropdown: function () {
      var that = this;
      var panelCollection = new Backbone.Collection([]);

      //set all column models unchecked for filter dropdown
      var initialColumns = _.map(this.gridView.BackGrid.columns.toJSON(), function (o, k) {
        o.renderable = false;
        return o;
      });
      //remove default select(check) column
      var chooserCollection = new Backbone.Collection(_.without(initialColumns, _.findWhere(initialColumns, {name: ""})));

      //ger current collection parameters - filters
      var currentGridFilters = this.gridView.BackGrid.collection.getQueryParams(['filters']);

      //collect filter data from columns(filter name) and collection (filter data)
      _.each(currentGridFilters, function (val, key, o) {
        var filterName = key.substr(0, key.indexOf('Id'));

        var filterModel = chooserCollection.findWhere({'name': filterName});
        //var filterDataModel = this.gridView.BackGrid.collection.findWhere(o);

        if (filterModel) {
          filterModel.set('renderable', true);
          filterModel.set('filterData', val["name"]);
          panelCollection.add(filterModel);
        }

      }, this);

      panelCollection.models = panelCollection.models.reverse();

      var FilterChooser = new FilterChooserView({collection: chooserCollection, panelCollection: panelCollection});
      var FiltersPanel = new FiltersPanelView({collection: panelCollection, backGrid: this.gridView.BackGrid});

      this.listenTo(FilterChooser, "render", function () {
        this.ui.filtersBtn.removeClass("disabled");
      }, this);

      this.ui.filtersDropdown.append(FilterChooser.render().$el);
      this.filtersPanel.show(FiltersPanel);
      this.ui.filtersDropdown.on("hide.bs.dropdown", that.onHideFiltersDropdown);
    },

    //TODO extract to separate add method
    onClickCreateNew: function (e) {
      e.preventDefault();
      this.gridView.insertRow(this.gridModel);
    },

    onRender: function () {
      var that = this;

      this.listenTo(this.gridView.BackGrid, "backgrid:rendered", function (grid) {
        that.setGridResizable();
      });

      this.ui.columnsDropdown.on("hide.bs.dropdown", that.onHideColumnsDropdown);
    },

    getSelectedModels: function () {
      return _.pluck(this.gridView.BackGrid.getSelectedModels(), 'id');
    },

    setTotalSize: function (l) {
      var length;
      if (typeof l == "number") {
        length = l;
      } else {
       if (!_.isUndefined(this.gridView.BackGrid.collection.getTotalRecords)) length = this.gridView.BackGrid.collection.getTotalRecords();
      }
      var length = this.gridView.BackGrid.collection.length ? this.gridView.BackGrid.collection.length : 0;
      this.$("#js-total-size").text(length);
    },

    initContextMenu: function () {
      this.ui.contextMenu.append(new EJS({url: this.contextMenuTemplate}).render());
      this.ui.contextMenu.attr('id', this.gridView.BackGrid.$el.data('menu-id'));
    },

    showTitlePageSize: function () {
      var $titlePageSize = $(new EJS({url: this.titlePageSizeTemplate}).render());
      this.ui.titlePageSize.append($titlePageSize);
      this.ui.titlePageSize.find(".js-page-size").removeClass("hidden").val(this.gridCollection.getQueryParams(["pageable"]).size).select2()//.on("change", this.onChangePageSize.bind(this));
    },

    setGridResizable: function (options) {

      this.gridView.BackGrid.$el.colResizable({
        //liveDrag:true,
        minWidth: 20,
        fixed: false,
        postbackSafe: true,
        partialRefresh: true
      });

    },

    onShow: function () {
      var that = this;

      // Without this special call Firefox doesn't see this.$el.find(".js-total-size") object
      if(navigator.userAgent.toLowerCase().indexOf('firefox') > -1) {
        this.setTotalSize();
      }

      if (!_.isEmpty(this.actions)) {
        this.actionsView = new window.App.Views.TableHeaderActionsView({
          actions: this.actions,
          type: this.type,
          isFile: this.isFile,
          backgrid: this.gridView.BackGrid,
          gridCollection: this.gridCollection
        });
        this.actionsRegion.show(this.actionsView);
      }




      if (that.options.showColumnsTool) {
        that.createColumnsDropdown();
      }

      if (that.options.showFiltersTool && that.options.fetchGridData) {
        if (this.gridView.BackGrid.collection.fetched) {
          that.createFilterDropdown();
        } else {
          this.listenToOnce(this.gridView.BackGrid.collection, 'sync', function (cols) {
            that.createFilterDropdown();
          })
        }
      }

      if (!this.options.showTools) {
        this.ui.tableTools.hide();
      }

      if (this.contextMenuTemplate) {
        this.initContextMenu();
      }

      //Listen grid main region - waiting for bgrid became visible on page
      this.gridMain.on('show', function () {
        that.setGridResizable();
      });

      this.gridMain.show(this.gridView.BackGrid);

      if (that.options.sortableRows) {
        that.$el.find("tbody").sortable();
        that.$el.find("tbody").disableSelection();
      }

      var title = this.$el.find('#grid-title');
      if (this.title) {
        title.text(this.title);
      } else {
        title.parent().hide();
      }

      if (this.options.showPageableTools) {
        var paginatorBottom = this.gridView.renderPaginator();
        var paginatorTop = this.gridView.renderPaginator();
        this.ui.gridPaginatorBottom.append(paginatorBottom);

        this.showTitlePageSize();
      }

      if (this.options.showPageSize && !this.options.showPageableTools) {
        this.showTitlePageSize();
      }

    },

    templateHelpers: function () {
      return {
        //totalRecords : this.gridView.BackGrid.collection.getTotalRecords(),
        actions: this.actions
      }
    }
  });

}());