(function () {

  "use strict";

  window.App.Views._BaseControls = Backbone.Marionette.LayoutView.extend({

    ui: {
      "edit": ".js-edit",
      "view": ".js-view",
      "files": ".js-files",
      "remove": ".js-remove",
      "clipboard": ".js-clipboard",
      "references": ".js-references"
    },

    events: {
      "click @ui.edit": "onEditClick",
      "click @ui.view": "onViewClick",
      "click @ui.remove": "onRemoveClick",
      "click @ui.files": "onFilesClick",
      "click @ui.clipboard": "onClipboardClick",
      "click @ui.references": "onReferencesClick"
    },

    initialize: function (models, options) {
      this.options = options || {};
    },

    onEditClick: function () {
      window.App.navigate("#project/" + this.model.get("objectType") + "/" + this.model.id + "/edit", {trigger: true});
    },

    onViewClick: function () {
      window.App.navigate("#project/" + this.model.get("objectType") + "/" + this.model.id + "/view", {trigger: true});
    },

    onRemoveClick: function () {
      var confStatus = confirm("Are you sure you want to delete " + this.model.get("name") + " " + this.model.get("objectType") + " ?");
      if (confStatus) {
        this.model.destroy({
          success: function (model, response) {
            window.App.navigate("project/home", {trigger: true});
            window.alertify.success(model.get("name") + " " + model.get("objectType") + " deleted");
            if (response.message) window.alertify.warning(response.message);
          },
          error: function (err) {
            console.log(err);
          }
        });
      }
    },

    onReferencesClick: function () {
      window.App.navigate("#project/" + this.model.get("objectType") + "/" + this.model.id + "/references", {trigger: true});
    },

    onFilesClick: function () {
      window.App.navigate("#project/" + this.model.get("objectType") + "/" + this.model.id + "/additionalfiles", {trigger: true});
    },

    onClipboardClick: function () {
      var that = this;
      var action = this.model.get("isInClipboard") ? "REMOVE" : "ADD";
      var message = this.model.get("isInClipboard") ? "Successfully removed from clipboard" : "Successfully added to clipboard";
      window.App.Helpers.ClipboardUpdate({
        action: action,
        data: [{
          type: window.App.Helpers.ClipboardType(this.model.get("objectType")),
          id: this.model.id
        }]
      }).then(function () {
        window.alertify.success(message);
        that.model.set({ "isInClipboard": !that.model.get("isInClipboard") });
        that.render();
      });
    },

    showFlag: function () {
      this.getRegion("flagRegion").show(new window.App.Views.FlagModuleView({ model: this.model }));
    },

    onShow: function () {
      this.$el.append("<div class='ui-controls-object-label " + this.model.get("objectType") + "'>" + this.model.get("objectType").capitalize() + "</div>")
    },

    templateHelpers: function () {
      return {
        isView: this.options.isView,
        inClipboard: this.model.get("isInClipboard")
      }
    }

  });
}());
