(function () {
  "use strict";

  window.App.Views.BaseImageView = Marionette.LayoutView.extend({
    template: "./views/templates/project/base/base_image.html.ejs",
    className: "image-upload-form row",
    tagName: "form",
    attributes: {
      "enctype": "multipart/form-data"
    },

    ui: {
      previewImage: ".js-preview-image",
      imageInput:   ".js-image-input",
      imageRemove:  ".js-remove-image"
    },

    events: {
      "change @ui.imageInput":  "onChangeImageInput",
      "click @ui.imageRemove":  "onClickRemoveImage"
    },

    initialize: function (options) {
      this.options = options || {};
      this.imageUrl = this.options.imageUrl || "rest/" + this.options.objectType + "/" + this.model.id + "/imageFile";
    },

    onChangeImageInput: function () {
      var that = this;
      this.$el.ajaxForm({
        url: this.imageUrl,
        type: "POST",
        success: function (data) {
          window.alertify.success("Изображение успешно загружено.");
          that.model.fetch().done(function () {
            that.render();
          });
        },
        error: function (xhr) {
          window.alertify.error("Не удалось загрузить изображение");
          that.render();
        }
      }).submit();
    },

    onClickRemoveImage: function () {
      var that = this;
      $.ajax({
        type: "DELETE",
        url: this.imageUrl,
        success: function () {
          window.alertify.success("Успешно удалено.");
          that.model.fetch().done(function () {
            that.render();
          });
        }
      })
    },

    templateHelpers: function() {
      return {
        isView: this.options.mode == "view",
        imageUrl: this.imageUrl
      }
    }

  });

}());