(function () {
  "use strict";

  window.App.Views.CompositeMaterialMainView = window.App.Views.MaterialMainView.extend({
    template: "./views/templates/project/composite_material/composite_material_main.html.ejs",

    ui: _.extend({
      technology: ".js-technology",
      equipment: ".js-equipment",
      fiber: ".js-fiber",
      matrix: ".js-matrix",
      epoxide: ".js-epoxide",
      fiberConcentration: ".js-fiber-concentration"
    }, window.App.Views.MaterialMainView.prototype.ui),

    events: _.extend({
      "change @ui.technology": "onChangeSelect",
      "change @ui.equipment": "onChangeSelect",
      "change @ui.fiber": "onChangeSelect",
      "change @ui.matrix": "onChangeSelect",
      "change @ui.epoxide": "onChangeSelect",
      "change @ui.fiberConcentration": "onChangeInput"
    }, window.App.Views.MaterialMainView.prototype.events),

    behaviors: {
      PluginsInit: {
        select2Events: [],
        select2: []
      }
    },

    onShow: function () {
      if (this.options.mode == "edit") {
        this.initTechnologySuggest();
        this.initEquipmentSuggest();
        this.initFiberSuggest();
        this.initMatrixSuggest();
        if (this.model.get("technologyType") === "ADDITIVE_3CM") {
          this.initEpoxideSuggest();
        }
      }
    },

    initTechnologySuggest: function () {
      if (this.ui.technology.data('select2')) this.ui.technology.select2("destroy");
      this.ui.technology.select2(
        new window.App.Helpers.Select2Config({
          el: ".js-technology",
          placeholder: this.model.get("technologyId") ? this.model.get("technology") : "Выберите технологию",
          url: "rest/technologies/suggest",
          isNumber: false,
          allowClear: true,
          minimumInputLength: 0
        })
      );
    },

    initEquipmentSuggest: function () {
      if (!_.isNull(this.model.get("technologyId"))) {
        this.ui.equipment.prop("disabled", false);
        this.ui.equipment.select2(
          new window.App.Helpers.Select2Config({
            el: ".js-equipment",
            placeholder: this.model.get("equipmentId") ? this.model.get("equipment") : "Выберите оборудование",
            url: "rest/technologies/" + this.model.get("technologyId") + "/equipments/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          }));
      } else {
        if (this.ui.equipment.data('select2')) this.ui.equipment.select2("destroy");
        this.ui.equipment.empty();
        this.ui.equipment.prop("disabled", true);
      }
    },

    initFiberSuggest: function () {
      if (this.ui.fiber.data('select2')) this.ui.fiber.select2("destroy");
      this.ui.fiber.select2(
        new window.App.Helpers.Select2Config({
          el: ".js-fiber",
          placeholder: this.model.get("fiberId") ? this.model.get("fiber") : "Волокно",
          url: "rest/materials/suggest",
          isNumber: false,
          allowClear: true,
          minimumInputLength: 0
        })
      );
    },

    initMatrixSuggest: function () {
      if (this.ui.matrix.data('select2')) this.ui.matrix.select2("destroy");
      this.ui.matrix.select2(
        new window.App.Helpers.Select2Config({
          el: ".js-matrix",
          placeholder: this.model.get("matrixId") ? this.model.get("matrix") : "Матрица",
          url: "rest/materials/suggest",
          isNumber: false,
          allowClear: true,
          minimumInputLength: 0
        })
      );
    },

    initEpoxideSuggest: function () {
      if (this.ui.epoxide.data('select2')) this.ui.epoxide.select2("destroy");
      this.ui.epoxide.select2(
        new window.App.Helpers.Select2Config({
          el: ".js-epoxide",
          placeholder: this.model.get("epoxideId") ? this.model.get("epoxide") : "Эпоксид",
          url: "rest/materials/suggest",
          isNumber: false,
          allowClear: true,
          minimumInputLength: 0
        })
      );
    },


    onChangeSelect: function (e) {
      var $target = $(e.target);
      var key = $target.attr("id");
      var value = $target.val();
      this.model.set(key + "Id", value);
      if (key == "technology") {
        this.initEquipmentSuggest();
        if (_.isNull(value)) this.model.set("equipmentId", null);
      }
    },

    onChangeInput: function (e) {
      var $target = $(e.target);
      var key = $target.attr("id");
      var value = $target.val();
      this.model.set(key, value);
    }

  });

}());