(function () {
  "use strict";

  window.App.Views.ConstructionElementMainView = Marionette.LayoutView.extend({
    template: "./views/templates/project/construction_element/construction_element_main.html.ejs",
    className: "row",

    regions: {
      imageRegion: ".js-image-region"
    },

    ui: {
      name:         ".js-name",
      cadFile:      ".js-cadFile",
      description:  ".js-description",
      path:         ".js-path"
    },

    events: {
      "keyup @ui.name":         "onChange",
      "keyup @ui.description":  "onChange",
      "click @ui.path":         "onClickPathLnk",
      "click @ui.cadFile":      "onCadFileLnk"
    },

    initialize: function (options) {
      this.options = options || {};
    },

    onShow: function () {
      this.showImage();
    },

    onClickPathLnk: function () {
      if (this.ui.path.hasClass("disabled")) return false;
      this.changePathModalView = new window.App.Views.ChangePathModal({ treeType: "constructionElements", onlyFolders: true });
      this.changePathModal = new Backbone.BootstrapModal({
        content: this.changePathModalView,
        title: "Выбрать путь",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.listenTo(this.changePathModalView, "path:select:done", this.pathSelectDone);
    },

    pathSelectDone: function (treeNode) {
      this.ui.path.text(window.App.Helpers.ShowPath(treeNode.path));
      this.model.set({ pid: treeNode.id });
    },

    onCadFileLnk: function () {
      window.open("rest/constructionElements/" + this.model.id + "/initialCadModel", "_self");
    },

    onChange: function (e) {
      var $target = $(e.target);
      var key = $target.attr("id");
      var value = this.ui[key].val();
      if (key == "name") {
        if (value.length == 0) {
          this.ui.name.closest(".form-group").addClass("has-error");
          window.alertify.error("Название не может быть пустым.");
          return;
        } else {
          this.ui.name.closest(".form-group").removeClass("has-error");
        }
      }
      this.model.set(key, value);
    },

    showImage: function () {
      this.getRegion("imageRegion").show(new window.App.Views.BaseImageView({
        model: this.model,
        mode: this.options.mode,
        objectType: "constructionElements"
      }));
    },

    templateHelpers: function() {
      return {
        isView: this.options.mode == "view"
      }
    }

  });

}());