(function () {
  "use strict";
  window.App.Views.EffectiveCharacteristicParametersPage = window.App.Views.BasePage.extend({
    template: "./views/templates/project/effective_characteristics/effective_characteristic_parameters_page.html.ejs",

    regions: _.extend({
      "microRegion": ".js-micro-region",
      "mesoRegion": ".js-meso-region"
    }, window.App.Views.BasePage.prototype.regions),

    ui: _.extend({
      "run": ".js-run",
      "results": ".js-results",
      "report": ".js-report"
    }, window.App.Views.BasePage.prototype.ui),

    events: _.extend({
      "click @ui.run": "onRunClick",
      "click @ui.results": "onResultsClick",
      "click @ui.report": "onReportClick"
    }, window.App.Views.BasePage.prototype.events),

    initialize: function (options) {
      window.App.Views.BasePage.prototype.initialize.apply(this, arguments);
      this.controls = ["edit", "run"];
      const FINISHED_STATUS_ID = 4;
      if (this.model.get("statusId") === FINISHED_STATUS_ID) this.controls.push("report");
      if (this.model.get("resultsCount") !== 0) this.controls.push("results");
    },

    onRender: function () {
      this.initAndShowMainView(this.getRegion("mainRegion"));
      this.initAndShowMicroStructureView(this.getRegion("microRegion"));
      this.initAndShowMesoStructureView(this.getRegion("mesoRegion"));
      window.App.Views.BasePage.prototype.onRender.call(this);
    },

    initAndShowMainView: function (region) {
      this.mainView = new window.App.Views.EffectiveCharacteristicMainView({
        layoutView: this,
        mode: this.mode,
        model: this.model
      });
      region.show(this.mainView);
    },

    initAndShowControlsView: function (region) {
      this.controlsView = new window.App.Views.EffectiveCharacteristicControlsView({
        layoutView: this,
        title: this.model.get("name"),
        controls: this.controls,
        mode: this.mode,
        model: this.model
      });
      region.show(this.controlsView);
    },

    initAndShowMicroStructureView: function (region) {
      var that = this;
      this.microstructureParameters = new window.App.Collections.EffectiveCharacteristicMicrostructureParameters({ effectiveCharacteristicId: this.model.id });
      this.microstructureParameters.fetch({
        success: function () {
          that.microStructureView = new window.App.Views.EffectiveCharacteristicMicroStructureView({
            layoutView: that,
            mode: that.mode,
            collection: that.microstructureParameters,
            effectiveCharacteristicId: that.model.id,
            microStructureHasImage: that.model.get("resultsCount") !== 0 && that.model.get("status") === "Completed"
          });
          region.show(that.microStructureView);
        }
      });
    },

    initAndShowMesoStructureView: function (region) {
      var that = this;
      this.mesostructureParameters = new window.App.Collections.EffectiveCharacteristicMesostructureParameters({ effectiveCharacteristicId: this.model.id });
      this.mesostructureParameters.fetch({
        success: function () {
          that.mesoStructureView = new window.App.Views.EffectiveCharacteristicMesoStructureView({
            layoutView: that,
            mode: that.mode,
            collection: that.mesostructureParameters,
            effectiveCharacteristicId: that.model.id,
            averageThicknessIsSetable: that.model.get("averageThicknessIsSetable"),
            mesoStructureHasImage: that.model.get("resultsCount") !== 0 && that.model.get("status") === "Completed"
          });
          region.show(that.mesoStructureView);
        }
      });
    },

    onResultsClick: function () {
      window.App.navigate("project/effectiveCharacteristics/" + this.model.id + "/results/view", { trigger: true });
    },

    onReportClick: function () {
      window.open("rest/effectiveCharacteristics/" + this.model.id + "/report");
    },

    onRunClick: function () {
      this.runModalView = new window.App.Views.BaseRunModal({
        "parentType": "effectiveCharacteristic",
        "parentName": this.model.get("name"),
        "parentId": this.model.id
      });
      this.runModal = new Backbone.BootstrapModal({
        content: this.runModalView,
        title: "Запуск расчета",
        animate: true,
        cancelText: "Отменить",
        okText: "Запустить"
      }).open();
      this.runModalView.passModal(this.runModal);
    }
  });

})();