(function() {
  "use strict";
  window.App.Views.MaterialBasePage = Marionette.LayoutView.extend({
    template: "./views/templates/project/material/material_base_page.html.ejs",
    className: function () {
      return this.options.mode + " material-page";
    },

    regions: {
      controlsRegion:   ".js-controls-region",
      mainRegion:       ".js-main-region",
      defaultsRegion:   ".js-defaults-region",
      plotRegion:       ".js-plot-region"
    },

    ui: {
      navItem: ".js-nav-item"
    },

    events: {
      "click @ui.navItem": "onClickNavItem"
    },

    initialize: function (options) {
      this.options = options || {};
      this.mode = this.options.mode;
      this.controls = ["new", "edit"];
      this.page = "parameters"; // Default
      this.listenTo(this.model, "sync", this.render);
    },

    onRender: function () {
      this.initAndShowControlsView(this.getRegion("controlsRegion"));
    },

    onNewClick: function () {
      window.App.navigate("#project/materials/create", { trigger: true });
    },

    onEditClick: function () {
      window.App.navigate("#project/materials/" + this.model.id + "/" + this.page + "/edit", { trigger: true });
    },

    onSaveClick: function () {
      var that = this;
      this.model.save({}, {
        success: function () {
          window.App.navigate("#project/materials/" + that.model.id + "/" + that.page + "/view", { trigger: true });
        },
        error: function (xhr) {
          console.error("Failed to save: ", xhr);
        }
      });
    },

    onCancelClick: function () {
      window.App.navigate("#project/materials/" + this.model.id + "/" + this.page + "/view", { trigger: true });
    },

    onClickNavItem: function (e) {
      var $target = $(e.target).closest("li");
      if (!$target.hasClass("active"))
        window.App.navigate("#project/" + this.options.modelName + "/" + this.model.id + "/" + $target.data("route") + "/view");
    },

    initAndShowControlsView: function (region) {
      this.controlsView = new window.App.Views.MaterialControlsView({
        layoutView: this,
        model: this.model,
        controls: this.controls
      });
      region.show(this.controlsView);
    },

    templateHelpers: function () {
      return {
        active: this.page
      }
    }

  });

})();