(function () {
  "use strict";

  window.App.Views.MaterialPlotView = Marionette.LayoutView.extend({
    template: "./views/templates/project/material/material_plot.html.ejs",
    className: "ui-material-plot clearfix",

    regions: {
      defaultsRegion:   ".js-defaults-region",
      tableRegion:      ".js-table-region",
      plotRegion:       ".js-plot-region"
    },

    ui: {
      parameterSelect: ".js-select-parameter"
    },

    events: {
      "change @ui.parameterSelect":  "onChangeSelectedParameter"
    },

    initialize: function (options) {
      this.options = options || {};
      this.currentParameter = this.collection.at(0);
      this.listenTo(this.collection, "change:current:parameter", this.onChangeSelectedParameter)
    },

    onRender: function () {
      this.updateTablesAndPlot();
    },

    onChangeSelectedParameter: function (id) {
      id = typeof id == "number" ? id : parseInt(this.ui.parameterSelect.val());
      this.currentParameter = this.collection.findWhere({ id: id });
      this.render();
    },

    updateTablesAndPlot: function () {
      this.drawDefaultParameterValues();
      this.fetchPlotParameterValues();
    },

    fetchPlotParameterValues: function () {
      var that = this;
      this.parameterValues = new window.App.Collections.PlotParameterValues({
        materialId: this.options.materialId,
        parameterId: this.currentParameter.id,
        materialType: "materials"
      });
      this.parameterValues.fetch().done(function () {
        that.drawTableAndPlot();
      });
    },

    drawDefaultParameterValues: function () {
      this.parametersDefaultsView = new window.App.Views.ParametersDefaultValuesView({
        collection: this.collection,
        currentParameter: this.currentParameter
      });
      this.getRegion("defaultsRegion").show(this.parametersDefaultsView);
    },

    drawTableAndPlot: function () {
      this.parameterValuesTableView = new window.App.Views.ParameterValuesTableView({
        collection: this.parameterValues,
        currentParameter: this.currentParameter
      });
      this.parameterValuesPlotView = new window.App.Views.ParameterValuesChartView({
        collection: this.parameterValues,
        currentParameter: this.currentParameter
      });
      if (this.getRegion("tableRegion")) this.getRegion("tableRegion").show(this.parameterValuesTableView);
      if (this.getRegion("plotRegion")) this.getRegion("plotRegion").show(this.parameterValuesPlotView);
    },

    templateHelpers: function () {
      return {
        currentParameterId: this.currentParameter.id
      }
    }

  });

}());