(function () {
  "use strict";

  window.App.Views.CreateOptimizationView = Marionette.LayoutView.extend({
    template: "./views/templates/project/optimization/create_optimization.html.ejs",
    className: "create-optimization row",

    behaviors: {
      PluginsInit: {
        select2Events: [],
        select2: [
          new window.App.Helpers.Select2Config({
            el: ".js-type",
            placeholder: "Тип оптимизации",
            url: "rest/optimizationTypes/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          })
        ]
      }
    },

    regions: {
      mainRegion: ".js-main-region",
      treeRegion: ".js-tree-region"
    },

    ui: {
      name: ".js-name",
      path: ".js-path",
      solverCAO: ".js-solver-cao",
      solverCAE: ".js-solver-cae",
      solverCAD: ".js-solver-cad",
      blockCAO: '.solver-block.cao',
      blockCAE: '.solver-block.cae',
      blockCAD: '.solver-block.cad',
      solverBlock: '.solver-block',
      blockNOSOLVER: '.solver-block.nosolver'
    },

    events: {
      "select2:select select": 'onSelectSolver',
      "select2:unselect select": 'onUnselectSolver',
      "click @ui.path": "onPathClick"
    },

    solverTypes: ['CAE', 'CAO', 'CAD'],

    initialize: function (options) {
      this.options = options || {};
      this.selectedPid = options.params && options.params.id;
      this.path = options.params && options.params.path;
      this.model = new window.App.Models.Optimization();
      this.notInitializedSolvers = this.solverTypes.slice();
      this.initSolversDropdowns();
      this.listenTo(this, "ok", this.onSubmit);
      this.listenTo(this, "cancel", this.onCancel);
    },

    onPathClick: function () {
      this.$(".js-main-region").addClass("hidden");
      this.md.$el.find(".btn").addClass('disabled');
      this.pathModalView = new window.App.Views.CreateModalPath({
        treeType: "optimizations",
        onlyFolders: false
      });
      this.getRegion("treeRegion").show(this.pathModalView);
      this.listenTo(this.pathModalView, "path:select:canceled", this.pathSelectCanceled);
      this.listenTo(this.pathModalView, "path:select:done", this.pathSelectDone);
    },

    pathSelectCanceled: function () {
      this.$(".js-main-region").removeClass("hidden");
      this.md.$el.find(".btn").removeClass("disabled");
      this.getRegion("treeRegion").empty();
    },

    pathSelectDone: function (treeNode) {
      this.notInitializedSolvers = this.solverTypes.slice();
      this.options.params = this.options.params || {};
      this.options.params.optimizationTypeId = treeNode.optimizationTypeId;

      this.pathSelectCanceled();
      this.ui.path.text(window.App.Helpers.ShowPath(treeNode.path));
      this.selectedPid = treeNode.id;

      this.initSolversDropdowns();
    },

    onUnselectSolver: function (e) {
      var $el = this.$(e.currentTarget);
      this.model.set($el.data('id'), null);
    },

    onSelectSolver: function (e) {
      var $el = this.$(e.currentTarget),
        data = e.params && e.params.data || {};

      if (_.isEmpty(data)) {
        return false
      }

      var idx = this.notInitializedSolvers.indexOf(data.type);
      if (idx !== -1) {
        this.notInitializedSolvers.splice(idx, 1)
      }

      this.model.set($el.data('id'), parseInt($el.val()));
      _.each(this.notInitializedSolvers, function (type) {
        if (data['compatible' + type] && data['compatible' + type].length) {
          this.initSolverDropdown(type.toLocaleUpperCase());
        }
      }, this);
    },

    initSolverDropdown: function (type) {
      if (!type) {
        console.warn("Solver type doesn't set!");
        return false;
      }
      console.info('Initialize solver suggest: ' + type);
      type = type.toLocaleUpperCase();
      var optimizationTypeId = this.options.params.optimizationTypeId;

      if (!(this.options.params && optimizationTypeId)) {
        console.warn('No optimizationTypeId found!');
        return false;
      }

      //show solver UI block
      this.ui['block' + type].show();

      var sConfig = new window.App.Helpers.Select2Config({
        el: this.ui['solver' + type],
        placeholder: type,
        url: "rest/optimizationTypes/" + optimizationTypeId + "/" + type.toLocaleLowerCase(),
        isNumber: false,
        allowClear: true,
        minimumInputLength: 0,
        type: 'POST'
      });

      this.ui['solver' + type].select2(sConfig);
    },

    /**
     * Reset dropdowns and hide UI blocks, in other words set them to default values
     */
    resetSolvers: function () {
      this.$('select').each(function () {
        if ($(this).data('select2')) {
          $(this).select2('destroy');
          $(this).empty();
          $(this).parents('.solver-block').hide();
        }
      });
    },

    requestAndIntiSolvers: function (type, solversToCheck) {
      type = type || 'CAO';
      solversToCheck = solversToCheck || this.solverTypes.slice();
      var that = this;
      var optimizationTypeId = this.options.params.optimizationTypeId;
      if (!(this.options.params && optimizationTypeId)) {
        console.warn('No optimizationTypeId found!');
        return false;
      }

      $.post("rest/optimizationTypes/" + optimizationTypeId + "/" + type.toLocaleLowerCase(), onGETSuccessCallback);

      function onGETSuccessCallback(data) {
        data = data || [];
        if (!data.length) {
          solversToCheck.splice(solversToCheck.indexOf(type), 1);
          if (solversToCheck.length) {
            that.requestAndIntiSolvers(solversToCheck[0], solversToCheck);
          } else {
            that.ui.blockNOSOLVER.show();
          }
        } else {
          that.ui.blockNOSOLVER.hide();
          that.initSolverDropdown(type)
        }
      }

    },

    serializeData: function () {
      return {
        'path': this.path && "/" + this.path,
        'optimizationTypeId': this.options.params.optimizationTypeId
      };
    },

    initSolversDropdowns: function () {
      this.resetSolvers();
      this.requestAndIntiSolvers()
    },

    onRender: function () {
      this.initSolversDropdowns()
    },

    passModal: function (md) {
      this.md = md;
    },

    onCancel: function () {
      var fillParams = function (params) {
        var p = "";
        if (params.path && params.optimizationTypeId ) {
          p += "?path=" + encodeURIComponent(params.path) + '&optimizationTypeId=' + params.optimizationTypeId
        }
        p += params.id ? "&id=" + params.id : "";
        return p
      };

      window.App.navigate("#project/optimizations" + fillParams(this.options.params), { trigger: true });
    },

    onSubmit: function (md) {
      md.preventClose();
      var name = this.ui.name.val();
      this.$(".form-group").removeClass('has-error');
      
      if (_.isEmpty(name))
        this.ui.name.closest(".form-group").addClass("has-error");
      else if (!this.selectedPid)
        this.ui.path.closest(".form-group").addClass("has-error");
      else if (!this.model.get('solverCaoId') && !this.model.get('solverCaeId') && !this.model.get('solverCadId')) {
        this.ui.solverBlock.addClass("has-error");
      }
      else {
        this.model.save({
          name: name,
          pid: this.selectedPid,
          optimizationTypeId: this.options.params.optimizationTypeId,
          referenceId: this.options.params.referenceId ? this.options.params.referenceId : null
        }, {
          success: function (model, resp) {
            window.App.navigate("#project/optimizations/" + model.get("id") + "/parameters/edit", {trigger: true});
            md.close();
          },
          error: function () {

          }
        });
      }
    }

  });

}());