let OptimizationFlow = Backbone.Model.extend({
  url: function () {
    return "rest/optimizations/" + this.options.optimizationId + "/flow";
  },
  initialize: function (attrs, options) {
    this.options = options || {};
  }
});

window.OptimizationFlow = React.createClass({
  getInitialState: function () {
    this.flow = new OptimizationFlow(null, {
      optimizationId: this.props.optimization.id, nameCAO: this.props.optimization.attributes.caoName });
    return this.flow.toJSON();
  },

  componentDidMount: function () {
    this.updateModel().then(() => {
      this.setState(this.flow.toJSON())
    }, null);
    window.App.commands.setHandler("optimization:dependency:added", (type) => {
      let state = this.state;
      state[type].total++;
      this.setState(state);
    });
    window.App.commands.setHandler("optimization:dependency:removed", (options) => {
      let state = this.state;
      state[options.type].total = state[options.type].total - options.num;
      this.setState(state);
    });
  },

  onChartClick: function (e) {
    e.preventDefault();
    e.persist();
    this.fetchAttributes("targetFunctions")
      .then((functions) =>
        this.fetchAttributes("cao-borders")
          .then((borders) => {
            this.showChartCreationModal({ borders, functions });
          }, null),
        null);
  },

  showChartCreationModal: function (options) {
    this.createSliceChartView = new window.App.Views.CreateSliceChartView(
        _.extend({ optimizationId: this.props.optimization.id, nameCAO: this.props.optimization.attributes.caoName }, options));
    this.createModal = new Backbone.BootstrapModal({
      content: this.createSliceChartView,
      title: "Построение графика-среза",
      animate: true,
      cancelText: "Отменить",
      okText: "Построить"
    }).open();
    this.createSliceChartView.passModal(this.createModal);
  },

  fetchAttributes: function (type) {
    let url = "rest/optimizations/" + this.props.optimization.id + "/" + this.props.optimization.attributes.caoName +"/" + type;
    return $.get(url);
  },

  updateModel: function () {
    return new Promise((resolve, reject) => {
      this.flow.fetch({
        success: resolve,
        error: reject
      });
    });
  },

  onFlowBlockClick: function (e) {
    e.stopPropagation();
    let $target = $(e.target);

    if (!$target.hasClass("js-flow-btn") && !$target.hasClass("fa")) {
      let type = $target.closest(".ui-flow-block").data("item");
      this.props.onFlowBlockClick(type);
    }
    else {
      let type = $target.closest(".js-flow-btn").data("item");
      this.props.onFlowBlockClick(type);
    }
  },

  render: function () {
    let sliceButtonOpts = {
      toggle: "tooltip",
      title: "Построить график-срез",
      /*style: {
        "margin": "10px"
      }*/
    };
    return (
      <div className="ui-optimization-flow row">
        <div className="col-lg-3">
          <div className="ui-flow-block" data-item="models" onClick={this.onFlowBlockClick}>
            <div className="ui-flow-block-header">Расчетные файлы</div>
            <div className="ui-block-mini clearfix">
              <div className="ui-block-flow-number">
                {this.state.models && this.state.models.total || 0} <span className="ui-block-flow-number-label">моделей</span>
              </div>
            </div>
          </div>
        </div>
        <div className="col-lg-3">
          <div className="ui-flow-block" data-item="dependencies" onClick={this.onFlowBlockClick}>
            <div className="ui-flow-block-header">Зависимости</div>
            <div className="ui-block-mini clearfix">
              <div className="ui-block-flow-number">
              {this.state.materials && this.state.materials.total || 0} <span className="ui-block-flow-number-label">м&#x20;|&#x20;</span>
              {this.state.technologies && this.state.technologies.total || 0} <span className="ui-block-flow-number-label">т&#x20;|&#x20;</span>
              {this.state.loadConditions && this.state.loadConditions.total || 0} <span className="ui-block-flow-number-label">у</span>
              </div>
            </div>
          </div>
        </div>
        <div className="col-lg-3">
          <div className="ui-flow-block cursor-default" data-item="tasks" onClick={this.onFlowBlockClick}>
            <div className="ui-flow-block-header">Проведенные расчеты</div>
            <div className="ui-block-mini clearfix">
              <div className="ui-block-flow-number">{this.state.tasks && this.state.tasks.total || 0} <span className="ui-block-flow-number-label">всего</span></div>
            </div>
          </div>
        </div>
        <div className="col-lg-3">
          <div className="ui-flow-block ui-hide-after" data-item="results-all" onClick={this.onFlowBlockClick}>
            <div className="ui-flow-block-header">Ключевые результаты</div>
                {this.state.results && this.state.results.totalAuto !== 0 ?
                  <div className="ui-block-mini-results clearfix">
                  <div className="ui-block-flow-buttons">
                  <div className="btn btn-info ui-btn-mini js-target-values-btn js-flow-btn" data-item="targetValues" data-toggle="tooltip" title="Целевые показатели" onClick={this.onFlowBlockClick} > {
                    (this.state.results.resultsByType[0].resultType==='values') ? this.state.results.resultsByType[0].totalAuto || 0 :
                    (this.state.results.resultsByType[1].resultType==='values') ? this.state.results.resultsByType[1].totalAuto || 0 :
                    (this.state.results.resultsByType[2].resultType==='values') ? this.state.results.resultsByType[2].totalAuto || 0 :
                    (this.state.results.resultsByType[3].resultType==='values') ? this.state.results.resultsByType[3].totalAuto || 0 :
                    (this.state.results.resultsByType[4].resultType==='values') ? this.state.results.resultsByType[4].totalAuto || 0 :
                    (this.state.results.resultsByType[5].resultType==='values') ? this.state.results.resultsByType[5].totalAuto || 0 :
                    (this.state.results.resultsByType[6].resultType==='values') ? this.state.results.resultsByType[6].totalAuto || 0 :
                    (this.state.results.resultsByType[7].resultType==='values') ? this.state.results.resultsByType[7].totalAuto || 0 :
                    (this.state.results.resultsByType[8].resultType==='values') ? this.state.results.resultsByType[8].totalAuto || 0 :
                    (this.state.results.resultsByType[9].resultType==='values') ? this.state.results.resultsByType[9].totalAuto || 0 :
                    (this.state.results.resultsByType[10].resultType==='values') ? this.state.results.resultsByType[10].totalAuto || 0 :
                    (this.state.results.resultsByType[11].resultType==='values') ? this.state.results.resultsByType[11].totalAuto || 0 :
                    (this.state.results.resultsByType[12].resultType==='values') ? this.state.results.resultsByType[12].totalAuto || 0 :
                    (this.state.results.resultsByType[13].resultType==='values') ? this.state.results.resultsByType[13].totalAuto || 0 :
                    (this.state.results.resultsByType[14].resultType==='values') ? this.state.results.resultsByType[14].totalAuto || 0 :
                    (this.state.results.resultsByType[15].resultType==='values') ? this.state.results.resultsByType[15].totalAuto || 0 :
                    (this.state.results.resultsByType[16].resultType==='values') ? this.state.results.resultsByType[16].totalAuto || 0 :
                    (this.state.results.resultsByType.keys().length > 17 && this.state.results.resultsByType[17].resultType==='values') ?
                        this.state.results.resultsByType[17].totalAuto || 0 : 0} <i className="fa fa-bullseye" aria-hidden="true" onClick={this.onFlowBlockClick}></i></div>
                  <div className="btn btn-info ui-btn-mini js-log-files-btn js-flow-btn" data-item="logFiles" data-toggle="tooltip" title="Информация о расчетах" onClick={this.onFlowBlockClick} > {
                    (this.state.results.resultsByType[0].resultType==='logFiles') ? this.state.results.resultsByType[0].totalAuto || 0 :
                    (this.state.results.resultsByType[1].resultType==='logFiles') ? this.state.results.resultsByType[1].totalAuto || 0 :
                    (this.state.results.resultsByType[2].resultType==='logFiles') ? this.state.results.resultsByType[2].totalAuto || 0 :
                    (this.state.results.resultsByType[3].resultType==='logFiles') ? this.state.results.resultsByType[3].totalAuto || 0 :
                    (this.state.results.resultsByType[4].resultType==='logFiles') ? this.state.results.resultsByType[4].totalAuto || 0 :
                    (this.state.results.resultsByType[5].resultType==='logFiles') ? this.state.results.resultsByType[5].totalAuto || 0 :
                    (this.state.results.resultsByType[6].resultType==='logFiles') ? this.state.results.resultsByType[6].totalAuto || 0 :
                    (this.state.results.resultsByType[7].resultType==='logFiles') ? this.state.results.resultsByType[7].totalAuto || 0 :
                    (this.state.results.resultsByType[8].resultType==='logFiles') ? this.state.results.resultsByType[8].totalAuto || 0 :
                    (this.state.results.resultsByType[9].resultType==='logFiles') ? this.state.results.resultsByType[9].totalAuto || 0 :
                    (this.state.results.resultsByType[10].resultType==='logFiles') ? this.state.results.resultsByType[10].totalAuto || 0 :
                    (this.state.results.resultsByType[11].resultType==='logFiles') ? this.state.results.resultsByType[11].totalAuto || 0 :
                    (this.state.results.resultsByType[12].resultType==='logFiles') ? this.state.results.resultsByType[12].totalAuto || 0 :
                    (this.state.results.resultsByType[13].resultType==='logFiles') ? this.state.results.resultsByType[13].totalAuto || 0 :
                    (this.state.results.resultsByType[14].resultType==='logFiles') ? this.state.results.resultsByType[14].totalAuto || 0 :
                    (this.state.results.resultsByType[15].resultType==='logFiles') ? this.state.results.resultsByType[15].totalAuto || 0 :
                    (this.state.results.resultsByType[16].resultType==='logFiles') ? this.state.results.resultsByType[16].totalAuto || 0 :
                    (this.state.results.resultsByType.keys().length > 17 && this.state.results.resultsByType[17].resultType==='logFiles') ?
                        this.state.results.resultsByType[17].totalAuto || 0 : 0} <i className="fa fa-info" aria-hidden="true" onClick={this.onFlowBlockClick}></i></div>
                  <div className="btn btn-info ui-btn-mini js-pictures-btn js-flow-btn" data-item="pictures" data-toggle="tooltip" title="Изображения" onClick={this.onFlowBlockClick} > {
                    (this.state.results.resultsByType[0].resultType==='pictures') ? this.state.results.resultsByType[0].totalAuto || 0 :
                    (this.state.results.resultsByType[1].resultType==='pictures') ? this.state.results.resultsByType[1].totalAuto || 0 :
                    (this.state.results.resultsByType[2].resultType==='pictures') ? this.state.results.resultsByType[2].totalAuto || 0 :
                    (this.state.results.resultsByType[3].resultType==='pictures') ? this.state.results.resultsByType[3].totalAuto || 0 :
                    (this.state.results.resultsByType[4].resultType==='pictures') ? this.state.results.resultsByType[4].totalAuto || 0 :
                    (this.state.results.resultsByType[5].resultType==='pictures') ? this.state.results.resultsByType[5].totalAuto || 0 :
                    (this.state.results.resultsByType[6].resultType==='pictures') ? this.state.results.resultsByType[6].totalAuto || 0 :
                    (this.state.results.resultsByType[7].resultType==='pictures') ? this.state.results.resultsByType[7].totalAuto || 0 :
                    (this.state.results.resultsByType[8].resultType==='pictures') ? this.state.results.resultsByType[8].totalAuto || 0 :
                    (this.state.results.resultsByType[9].resultType==='pictures') ? this.state.results.resultsByType[9].totalAuto || 0 :
                    (this.state.results.resultsByType[10].resultType==='pictures') ? this.state.results.resultsByType[10].totalAuto || 0 :
                    (this.state.results.resultsByType[11].resultType==='pictures') ? this.state.results.resultsByType[11].totalAuto || 0 :
                    (this.state.results.resultsByType[12].resultType==='pictures') ? this.state.results.resultsByType[12].totalAuto || 0 :
                    (this.state.results.resultsByType[13].resultType==='pictures') ? this.state.results.resultsByType[13].totalAuto || 0 :
                    (this.state.results.resultsByType[14].resultType==='pictures') ? this.state.results.resultsByType[14].totalAuto || 0 :
                    (this.state.results.resultsByType[15].resultType==='pictures') ? this.state.results.resultsByType[15].totalAuto || 0 :
                    (this.state.results.resultsByType[16].resultType==='pictures') ? this.state.results.resultsByType[16].totalAuto || 0 :
                    (this.state.results.resultsByType.keys().length > 17 && this.state.results.resultsByType[17].resultType==='pictures') ?
                        this.state.results.resultsByType[17].totalAuto || 0 : 0} <i className="fa fa-file-image-o" aria-hidden="true" onClick={this.onFlowBlockClick}></i></div>
                  <div className="btn btn-info ui-btn-mini js-db-files-btn js-flow-btn" data-item="dbFiles" data-toggle="tooltip" title="Файлы расчета" onClick={this.onFlowBlockClick} > {
                    (this.state.results.resultsByType[0].resultType==='database') ? this.state.results.resultsByType[0].totalAuto || 0 :
                    (this.state.results.resultsByType[1].resultType==='database') ? this.state.results.resultsByType[1].totalAuto || 0 :
                    (this.state.results.resultsByType[2].resultType==='database') ? this.state.results.resultsByType[2].totalAuto || 0 :
                    (this.state.results.resultsByType[3].resultType==='database') ? this.state.results.resultsByType[3].totalAuto || 0 :
                    (this.state.results.resultsByType[4].resultType==='database') ? this.state.results.resultsByType[4].totalAuto || 0 :
                    (this.state.results.resultsByType[5].resultType==='database') ? this.state.results.resultsByType[5].totalAuto || 0 :
                    (this.state.results.resultsByType[6].resultType==='database') ? this.state.results.resultsByType[6].totalAuto || 0 :
                    (this.state.results.resultsByType[7].resultType==='database') ? this.state.results.resultsByType[7].totalAuto || 0 :
                    (this.state.results.resultsByType[8].resultType==='database') ? this.state.results.resultsByType[8].totalAuto || 0 :
                    (this.state.results.resultsByType[9].resultType==='database') ? this.state.results.resultsByType[9].totalAuto || 0 :
                    (this.state.results.resultsByType[10].resultType==='database') ? this.state.results.resultsByType[10].totalAuto || 0 : (this.state.results.resultsByType[11].resultType==='database') ? this.state.results.resultsByType[11].totalAuto || 0 :
                    (this.state.results.resultsByType[12].resultType==='database') ? this.state.results.resultsByType[12].totalAuto || 0 :
                    (this.state.results.resultsByType[13].resultType==='database') ? this.state.results.resultsByType[13].totalAuto || 0 :
                    (this.state.results.resultsByType[14].resultType==='database') ? this.state.results.resultsByType[14].totalAuto || 0 :
                    (this.state.results.resultsByType[15].resultType==='database') ? this.state.results.resultsByType[15].totalAuto || 0 :
                    (this.state.results.resultsByType[16].resultType==='database') ? this.state.results.resultsByType[16].totalAuto || 0 :
                    (this.state.results.resultsByType.keys().length > 17 && this.state.results.resultsByType[17].resultType==='database') ?
                        this.state.results.resultsByType[17].totalAuto || 0 : 0} <i className="fa fa-database" aria-hidden="true" onClick={this.onFlowBlockClick}></i></div>
                  <div className="btn btn-info ui-btn-mini js-curves-btn js-flow-btn" data-item="curves" data-toggle="tooltip" title="Графики" onClick={this.onFlowBlockClick} > {
                    (this.state.results.resultsByType[0].resultType==='curves') ? this.state.results.resultsByType[0].totalAuto || 0 :
                    (this.state.results.resultsByType[1].resultType==='curves') ? this.state.results.resultsByType[1].totalAuto || 0 :
                    (this.state.results.resultsByType[2].resultType==='curves') ? this.state.results.resultsByType[2].totalAuto || 0 :
                    (this.state.results.resultsByType[3].resultType==='curves') ? this.state.results.resultsByType[3].totalAuto || 0 :
                    (this.state.results.resultsByType[4].resultType==='curves') ? this.state.results.resultsByType[4].totalAuto || 0 :
                    (this.state.results.resultsByType[5].resultType==='curves') ? this.state.results.resultsByType[5].totalAuto || 0 :
                    (this.state.results.resultsByType[6].resultType==='curves') ? this.state.results.resultsByType[6].totalAuto || 0 :
                    (this.state.results.resultsByType[7].resultType==='curves') ? this.state.results.resultsByType[7].totalAuto || 0 :
                    (this.state.results.resultsByType[8].resultType==='curves') ? this.state.results.resultsByType[8].totalAuto || 0 :
                    (this.state.results.resultsByType[9].resultType==='curves') ? this.state.results.resultsByType[9].totalAuto || 0 :
                    (this.state.results.resultsByType[10].resultType==='curves') ? this.state.results.resultsByType[10].totalAuto || 0 :
                    (this.state.results.resultsByType[11].resultType==='curves') ? this.state.results.resultsByType[11].totalAuto || 0 :
                    (this.state.results.resultsByType[12].resultType==='curves') ? this.state.results.resultsByType[12].totalAuto || 0 :
                    (this.state.results.resultsByType[13].resultType==='curves') ? this.state.results.resultsByType[13].totalAuto || 0 :
                    (this.state.results.resultsByType[14].resultType==='curves') ? this.state.results.resultsByType[14].totalAuto || 0 :
                    (this.state.results.resultsByType[15].resultType==='curves') ? this.state.results.resultsByType[15].totalAuto || 0 :
                    (this.state.results.resultsByType[16].resultType==='curves') ? this.state.results.resultsByType[16].totalAuto || 0 :
                    (this.state.results.resultsByType.keys().length > 17 && this.state.results.resultsByType[17].resultType==='curves') ?
                        this.state.results.resultsByType[17].totalAuto || 0 : 0} <i className="fa fa-area-chart" aria-hidden="true" onClick={this.onFlowBlockClick}></i></div>
                  <div className="btn btn-info ui-btn-mini js-animations-btn js-flow-btn" data-item="animations" data-toggle="tooltip" title="Анимация" onClick={this.onFlowBlockClick} > {
                    (this.state.results.resultsByType[0].resultType==='animations') ? this.state.results.resultsByType[0].totalAuto || 0 :
                    (this.state.results.resultsByType[1].resultType==='animations') ? this.state.results.resultsByType[1].totalAuto || 0 :
                    (this.state.results.resultsByType[2].resultType==='animations') ? this.state.results.resultsByType[2].totalAuto || 0 :
                    (this.state.results.resultsByType[3].resultType==='animations') ? this.state.results.resultsByType[3].totalAuto || 0 :
                    (this.state.results.resultsByType[4].resultType==='animations') ? this.state.results.resultsByType[4].totalAuto || 0 :
                    (this.state.results.resultsByType[5].resultType==='animations') ? this.state.results.resultsByType[5].totalAuto || 0 :
                    (this.state.results.resultsByType[6].resultType==='animations') ? this.state.results.resultsByType[6].totalAuto || 0 : (this.state.results.resultsByType[7].resultType==='animations') ? this.state.results.resultsByType[7].totalAuto || 0 :
                    (this.state.results.resultsByType[8].resultType==='animations') ? this.state.results.resultsByType[8].totalAuto || 0 :
                    (this.state.results.resultsByType[9].resultType==='animations') ? this.state.results.resultsByType[9].totalAuto || 0 :
                    (this.state.results.resultsByType[10].resultType==='animations') ? this.state.results.resultsByType[10].totalAuto || 0 :
                    (this.state.results.resultsByType[11].resultType==='animations') ? this.state.results.resultsByType[11].totalAuto || 0 :
                    (this.state.results.resultsByType[12].resultType==='animations') ? this.state.results.resultsByType[12].totalAuto || 0 :
                    (this.state.results.resultsByType[13].resultType==='animations') ? this.state.results.resultsByType[13].totalAuto || 0 :
                    (this.state.results.resultsByType[14].resultType==='animations') ? this.state.results.resultsByType[14].totalAuto || 0 :
                    (this.state.results.resultsByType[15].resultType==='animations') ? this.state.results.resultsByType[15].totalAuto || 0 :
                    (this.state.results.resultsByType[16].resultType==='animations') ? this.state.results.resultsByType[16].totalAuto || 0 :
                    (this.state.results.resultsByType.keys().length > 17 && this.state.results.resultsByType[17].resultType==='animations') ?
                        this.state.results.resultsByType[17].totalAuto || 0 : 0} <i className="fa fa-film" aria-hidden="true" onClick={this.onFlowBlockClick}></i></div>
                  <div className="btn btn-info ui-btn-mini js-stl-visualisation-btn js-flow-btn" data-item="stlVisualisations" data-toggle="tooltip" title="STL-анимация" onClick={this.onFlowBlockClick} > {
                    (this.state.results.resultsByType[0].resultType==='stlVisualisation') ? this.state.results.resultsByType[0].totalAuto || 0 :
                    (this.state.results.resultsByType[1].resultType==='stlVisualisation') ? this.state.results.resultsByType[1].totalAuto || 0 :
                    (this.state.results.resultsByType[2].resultType==='stlVisualisation') ? this.state.results.resultsByType[2].totalAuto || 0 :
                    (this.state.results.resultsByType[3].resultType==='stlVisualisation') ? this.state.results.resultsByType[3].totalAuto || 0 :
                    (this.state.results.resultsByType[4].resultType==='stlVisualisation') ? this.state.results.resultsByType[4].totalAuto || 0 :
                    (this.state.results.resultsByType[5].resultType==='stlVisualisation') ? this.state.results.resultsByType[5].totalAuto || 0 :
                    (this.state.results.resultsByType[6].resultType==='stlVisualisation') ? this.state.results.resultsByType[6].totalAuto || 0 :
                    (this.state.results.resultsByType[7].resultType==='stlVisualisation') ? this.state.results.resultsByType[7].totalAuto || 0 :
                    (this.state.results.resultsByType[8].resultType==='stlVisualisation') ? this.state.results.resultsByType[8].totalAuto || 0 :
                    (this.state.results.resultsByType[9].resultType==='stlVisualisation') ? this.state.results.resultsByType[9].totalAuto || 0 :
                    (this.state.results.resultsByType[10].resultType==='stlVisualisation') ? this.state.results.resultsByType[10].totalAuto || 0 :
                    (this.state.results.resultsByType[11].resultType==='stlVisualisation') ? this.state.results.resultsByType[11].totalAuto || 0 :
                    (this.state.results.resultsByType[12].resultType==='stlVisualisation') ? this.state.results.resultsByType[12].totalAuto || 0 :
                    (this.state.results.resultsByType[13].resultType==='stlVisualisation') ? this.state.results.resultsByType[13].totalAuto || 0 :
                    (this.state.results.resultsByType[14].resultType==='stlVisualisation') ? this.state.results.resultsByType[14].totalAuto || 0 :
                    (this.state.results.resultsByType[15].resultType==='stlVisualisation') ? this.state.results.resultsByType[15].totalAuto || 0 :
                    (this.state.results.resultsByType[16].resultType==='stlVisualisation') ? this.state.results.resultsByType[16].totalAuto || 0 :
                    (this.state.results.resultsByType.keys().length > 17 && this.state.results.resultsByType[17].resultType==='stlVisualisation') ?
                        this.state.results.resultsByType[17].totalAuto || 0 : 0} <i className="fa fa-play-circle" aria-hidden="true" onClick={this.onFlowBlockClick}></i></div>
                  {(this.props.optimization.attributes.nameCAO || this.props.optimization.attributes.caoName === "ModeFrontier") || (this.props.optimization.attributes.nameCAO || this.props.optimization.attributes.caoName === "Optimus") ?
                      <div className="btn btn-success ui-btn-mini js-chart-btn js-flow-btn" onClick={this.onChartClick} {...sliceButtonOpts}>
                          <i className="fa fa-line-chart" aria-hidden="true" onClick={this.onChartClick}></i>
                      </div>
                      : <div></div>}
                  </div>
                  </div>
                : <div className="ui-block-mini clearfix">
                    <div className="ui-block-flow-number"> {this.state.results && this.state.results.totalAuto || 0} <span className="ui-block-flow-number-label">всего</span></div>
                  </div>}
            </div>
          </div>
      </div>
    )
  }
});