(function () {
  "use strict";

  window.App.Views.ParameterValuesTableView = Marionette.CompositeView.extend({
    template: "./views/templates/project/parameter/parameter_values_table.html.ejs",
    childView: window.App.Views.ParameterValuesTableValue,
    childViewContainer: ".js-collection-container",
    className: "ui-parameter-values-table",

    ui: {
      tempIn: ".js-temp",
      valIn: ".js-val",
      addValue: ".js-add-value"
    },

    events: {
      "keyup @ui.tempIn, @ui.valIn": "onInputChange",
      "change @ui.tempIn, @ui.valIn": "onInputChange",
      "click @ui.addValue": "onAddValueClick"
    },

    initialize: function (options) {
      this.options = options || {};
      this.listenTo(this.collection, "add", this.render);
      this.listenTo(this.collection, "model:updated", this.render);
      this.listenTo(this.collection, "remove", this.onRemoveTableValue);
      this.listenTo(this.options.currentParameter, "update:default:value", this.onAddValueClick)
    },

    onInputChange: function () {
      if (!_.isEmpty(this.ui.tempIn.val()) && !_.isEmpty(this.ui.valIn.val())) {
        this.ui.addValue.removeClass('disabled');
        this.temperature = parseFloat(this.ui.tempIn.val());
        this.paramValue = parseFloat(this.ui.valIn.val());
      } else {
        this.ui.addValue.addClass('disabled');
      }
    },

    triggerUpdateDefaults: function (temperature, value) {
      if (temperature == this.options.currentParameter.get("defaultTemperature"))
        this.options.currentParameter.trigger("update:defaults", {
          value: value
        });
    },

    onAddValueClick: function (options) {
      var model = this.collection.findWhere({ temperature: options.temperature || this.temperature });
      if (!_.isUndefined(model)) {
        model.set({paramValue: options.paramValue || this.paramValue});
        model.save();
      } else {
        this.collection.create({
          temperature:  options.temperature || this.temperature,
          paramValue:  options.paramValue || this.paramValue
        });
      }
      this.triggerUpdateDefaults(options.temperature || this.temperature, options.paramValue || this.paramValue);
    },

    onRemoveTableValue: function (model) {
      this.triggerUpdateDefaults(model.get("temperature"), null);
    },

    templateHelpers: function () {
      return {
        parameterName: this.options.currentParameter.get("paramName")
      }
    }
  });

}());