(function() {
  "use strict";

  window.App.Views.ProfileView = Backbone.Marionette.LayoutView.extend({
    className: "user-page",

    template: "./views/templates/project/profile.html.ejs",

    regions: {
      notificationsRegion:  ".js-notifications-region"
    },

    ui: {
      userName:           "#js-name",
      userEmail:          "#js-email",
      userPhone:          "#js-phone",
      userDate:           "#js-date",
      userRole:           ".js-select-role",
      attribute:          ".user-attribute",
      edit:               ".js-edit",
      save:               ".js-save",
      block:              ".js-toggle-block",
      cancel:             ".js-cancel",
      changePassword:     ".js-change-password",
      resetPassword:      ".js-reset-password",
      notifications:      "#js-notifications",
      removeImage:        ".js-close",
      userImageForm:      ".image-upload-form",
      userImage:          "#js-image-upload"
    },

    events: {
      "click @ui.edit":               "onEditClick",
      "click @ui.save":               "onSaveClick",
      "click @ui.block":              "onBlockClick",
      "click @ui.cancel":             "onCancelClick",
      "click @ui.changePassword":     "onChangePasswordClick",
      "click @ui.savePasswowrd":      "onSavePasswordClick",
      "click @ui.notifications":      "onToggleNotificationsClick",
      "click @ui.accessRights":       "onToggleAccessRightsClick",
      "click @ui.removeImage":        "onRemoveImageClick",
      "change @ui.userImage":         "onChangeImage",
      "click @ui.folderLnk":          "onClickFolderLnk",
      "click @ui.addAccessRight":     "onAddAccessRightsClick",
      "click @ui.importAccessRights": "onImportAccessClick",
      "click @ui.resetPassword":      "onResetPassword"
    },

    initialize: function(options) {
      this.options = options || {};
      this.listenTo(this.model, "sync", this.render);
    },

    onRender: function() {
      this.imageWasEdited = false;
    },

    onEditClick: function(e) {
      e.preventDefault();
      window.App.navigate("#profile/" + this.model.id + "/edit", { trigger: true});
    },

    onSaveClick: function() {
      var that = this;
      var bDate = moment(this.ui.userDate.val()).format("YYYY-MM-DDTHH:mm:ss.SSSZ");
      if (bDate == "Invalid date") bDate = null;
      if (this.ui.userName.val() !== "" && this.ui.userEmail.val() !== "") {
        this.model.set({
          "name": this.ui.userName.val(),
          "email": this.ui.userEmail.val(),
          "phone": this.ui.userPhone.val(),
          "birthDate": bDate,
          "roleId": this.ui.userRole.val()
        });

        this.model.save(this.model.toJSON(),{
          success: function() {
            window.alertify.success("Изменения успешно сохранены.");
            window.App.navigate("#profile/" + that.model.id + "/view", { trigger: true });
          }
        });
      } else {
        window.alertify.error("Имя и Почта должны быть заданы.");
      }
    },

    onBlockClick: function() {
      var that = this;
      $.ajax(this.model.url() + "/block", {
        type: "GET",
        contentType: "application/json",
        success: function (res) {
          that.model.fetch();
          window.alertify.success("Изменения успешно сохранены.");
        }
      });
    },

    onCancelClick: function() {
      window.App.navigate("#profile/" + this.model.id + "/view", { trigger: true });
    },

    onResetPassword: function() {
      var c = confirm("Вы действительно хотите сбросить пароль?");
      if (c) {
        $.ajax({
          type: "POST",
          url: "rest/users/" + this.model.id + "/password-reset",
          success: function() {
            window.alertify.success("Пароль успешно изменен.")
          }
        });
      }
    },

    onChangePasswordClick: function (e) {
      var modalView = new window.App.Views.ChangePasswordModal({ model: this.model });
      this.listenTo(modalView, "okClicked", this.onSavePasswordClick);

      this.modalPath = new Backbone.BootstrapModal({
        title: "Изменить пароль",
        content: modalView,
        animate: true,
        okText: "Сохранить",
        cancelText: "Отменить"
      }).open();
    },

    onChangeImage: function(e) {
      e.preventDefault();
      if (e.target.files[0]) {
        this.postImage();
        $('.preview-image').attr('src', URL.createObjectURL(e.target.files[0]) );
      }
    },

    postImage: function () {
      this.ui.userImageForm.ajaxForm({
        url: "rest/users/" + this.model.id + "/avatar",
        type: "POST",
        success: function (data) {
          window.alertify.success("Аватар успешно загружен.");
        },
        error: function (xhr) {
          window.alertify.error("Не удалось загрузить аватар.");
          console.log("upload avatar error ", xhr);
        }
      }).submit();
    },

    onToggleNotificationsClick: function() {
      if (_.isUndefined(this.getRegion("notificationsRegion").currentView)) {
        this.notificationsTableView = new window.App.Views.NotificationsTableView(_.extend(this.options, {
          showColumnsTool: false,
          selectable: false,
          saveToServer: false,
          showTools: false,
          gridCollection: new window.App.Collections.Notifications([], {filters: {sort: [{'field': 'description', direction: 'ASC'}]}, pageable: true})
        }));
        this.getRegion("notificationsRegion").show(this.notificationsTableView);
      } else {
        this.getRegion("notificationsRegion").empty();
      }
    },

    onClickFolderLnk: function (e) {
      e.preventDefault();
      e.stopPropagation();

      var modalView = new window.App.Views.FolderPathModal({ notOnlyLast: true, accessRightsModal: true });
      this.listenTo(modalView, "okClicked", this.getTreeNodeId);

      this.modalPath = new Backbone.BootstrapModal({
        content: modalView,
        animate: true,
        okText: "Сохранить",
        cancelText: "Отменить"
      }).open();
    },

    getTreeNodeId: function(path) {
      this.selectedTreeNodeId = window.App.Helpers.ShowPath(path, 'dir_id');
      this.ui.folderLnk.text(window.App.Helpers.ShowPath(path));
    },

    onRemoveImageClick: function() {
      var c = confirm("Вы действительно хотите удалить изображение?");
      if (c) this.removeImage();
    },

    removeImage: function() {
      var that = this;
      $.ajax({
        type: "DELETE",
        url: "rest/users/" + this.model.id + "/avatar",
        success: function() {
          that.render();
        }
      });
    },


    templateHelpers: function() {
      return {
        isView: this.options.mode == "view",
        parseRole: function (id) {
          var role = "";
          switch(id) {
            case 1:
              role = "Гость";
              break;
            case 2:
              role = "Пользователь";
              break;
            case 3:
              role = "Администратор";
              break;
          };
          return role;
        }
      }
    }

  });

})();
