package ru.crp.lvms;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Assert;
import org.junit.Test;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.model.entities.MaterialEntity;
import ru.crp.lvms.model.entities.ParametersEntityTable;
import ru.crp.lvms.utils.TestUtils;

import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;

import static ru.crp.lvms.utils.URLs.PARAMETERS_URL;

public class ParametersIT extends CRUDIT<ParametersEntityTable, ParametersEntityTable> {

    @Override
    public String setUrl() {
        return PARAMETERS_URL;
    }

    @Override
    public Supplier<ParametersEntityTable> setEntityCreator() {
        return projectCreator::createParameter;
    }

    @Override
    public Supplier<ParametersEntityTable> setEntityInitialisation() {
        return projectCreator::newParameter;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromList() {
        return TestUtils::parametersFromJson;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromPage() {
        return TestUtils::parametersPageFromJson;
    }

    @Override
    public void update(ParametersEntityTable toUpdate) {
        toUpdate.setName("newName");
        toUpdate.setType(2);
        toUpdate.setOwnerId(2);
    }

    @Override
    public void validateAfterUpdate(ParametersEntityTable before, ParametersEntityTable after) {
        Assert.assertEquals(before.getId(), after.getId());
        Assert.assertEquals(before.getType(), after.getType());
        Assert.assertEquals(before.getOwnerId(), after.getOwnerId());
        Assert.assertEquals(before.getName(), after.getName());
    }

    @Test
    public void deleteParameterThatIsUsedMustThrowException() throws Exception {
        MaterialEntity m = projectCreator.createMaterial();
        ParametersEntityTable p = projectCreator.createParameter();
        projectCreator.addParameterToMaterial(p.getId(), m.getId());
        TestUtils.DELETEAndExpectConflict(mockMvc, TestUtils.createObjectUrl(url, p));
    }
}
