package ru.crp.lvms.construction.element;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementDocsEntity;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementEntity;
import ru.crp.lvms.utils.TestUtils;

import java.util.List;

import static ru.crp.lvms.utils.URLs.CONSTRUCTION_ELEMENTS_URL;

public class ConstructionElementDocsIT extends BaseCEIT {

    private ConstructionElementEntity constructionElementEntity;

    private String docsUrl;

    @Before
    public void initTests() {
        constructionElementEntity = projectCreator.createConstructionElementEntity();
        docsUrl = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/descriptionFiles";
    }


    @Test
    public void itMustCreateDocFile() throws Exception {
        List<ConstructionElementDocsEntity> beforeCreate
                = TestUtils.constructionElementDocsFromJson(TestUtils.GETobjects(mockMvc, docsUrl));
        Assert.assertTrue("There should be no description file after creation", beforeCreate.isEmpty());
        ConstructionElementDocsEntity createdDoc = uploadFile(null, docsUrl, ConstructionElementDocsEntity.class);
        List<ConstructionElementDocsEntity> afterCreate
                = TestUtils.constructionElementDocsFromJson(TestUtils.GETobjects(mockMvc, docsUrl));
        Assert.assertEquals("There should be exactly one description file after creation", 1, afterCreate.size());
        Assert.assertEquals("File sources must be the same",
                createdDoc.getFileSource(), afterCreate.get(0).getFileSource());
        Assert.assertEquals("Construction element id must be the same",
                createdDoc.getConstructionElementId(), afterCreate.get(0).getConstructionElementId());
    }


    @Test
    public void itMustUpdateDocFile() throws Exception {
        ConstructionElementDocsEntity createdDoc = uploadFile(null, docsUrl, ConstructionElementDocsEntity.class);
        List<ConstructionElementDocsEntity> afterCreate
                = TestUtils.constructionElementDocsFromJson(TestUtils.GETobjects(mockMvc, docsUrl));
        Integer docId = afterCreate.get(0).getId();
        ConstructionElementDocsEntity updatedDoc = uploadFile(docId, docsUrl, ConstructionElementDocsEntity.class);
        Integer updatedDocId = afterCreate.get(0).getId();
        Assert.assertEquals("Id must not change", docId, updatedDocId);
    }


    @Test
    public void itMustDeleteDocFile() throws Exception {
        ConstructionElementDocsEntity createdDoc = uploadFile(null, docsUrl, ConstructionElementDocsEntity.class);
        TestUtils.DELETEobject(mockMvc, docsUrl + "/" + createdDoc.getId());
        List<ConstructionElementDocsEntity> afterDelete
                = TestUtils.constructionElementDocsFromJson(TestUtils.GETobjects(mockMvc, docsUrl));
        Assert.assertTrue("There should be no description file after deletion", afterDelete.isEmpty());
    }

}
