package ru.crp.lvms.optimizations;

import org.junit.Test;
import ru.crp.lvms.BaseIT;
import ru.crp.lvms.model.entities.tree.OptimizationsTreeNode;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.base.entities.OptimizationType;

import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static ru.crp.lvms.utils.TestUtils.*;
import static ru.crp.lvms.utils.URLs.TREE_URL;

public class OptimizationTreeIT extends BaseIT {
    private static final String OPTIMIZATION_TREE_URL = TREE_URL + "/optimizations";

    @Test
    public void itShouldBeAbleToCreateOptimizationFolder() throws Exception {
        // testing creating "root" folder (first level child of construction element)
        OptimizationsTreeNode rootFolder = projectCreator.newOptimizationFolder();
        rootFolder = CREATEobject(mockMvc, OPTIMIZATION_TREE_URL, rootFolder, OptimizationsTreeNode.class);
        assertNotEquals(0, rootFolder.getId());
        // testing creating 2-nd level child of construction element
        OptimizationsTreeNode childFolder = projectCreator.newOptimizationFolder();

        OptimizationEntity baseOptimization = projectCreator.newOptimization();
        baseOptimization.setPid(rootFolder.getId());
        baseOptimization = projectCreator.createOptimization(baseOptimization);

        childFolder.setPid(rootFolder.getId());
        childFolder.setBaseOptimizationId(baseOptimization.getId());
        List<OptimizationType> optimizationTypes = projectCreator.getOptimizationTypes();
        childFolder.setOptimizationTypeId(optimizationTypes.get(1).getId());

        childFolder = CREATEobject(mockMvc, OPTIMIZATION_TREE_URL, childFolder, OptimizationsTreeNode.class);
        assertNotEquals(0, childFolder.getId());
        assertEquals(Integer.valueOf(rootFolder.getId()), childFolder.getPid());
    }


    @Test
    public void itShouldBeAllowedToCreateOptimizationFolderWithSameTypeAsInParent() throws Exception {
        OptimizationsTreeNode parent = projectCreator.createOptimizationFolder();
        String url = createFolderUrl(parent);
        GETobject(mockMvc, url, OptimizationsTreeNode.class);
        OptimizationsTreeNode child = projectCreator.newOptimizationFolder();

        OptimizationEntity baseOptimization = projectCreator.newOptimization();
        baseOptimization.setPid(parent.getId());
        baseOptimization = projectCreator.createOptimization(baseOptimization);

        child.setPid(parent.getId());
        child.setBaseOptimizationId(baseOptimization.getId());
        child.setOptimizationTypeId(parent.getOptimizationTypeId());
        CREATEobject(mockMvc, OPTIMIZATION_TREE_URL, child, OptimizationsTreeNode.class);
    }


    @Test
    public void itShouldBeAbleToDeleteOptimizationFolder() throws Exception {
        OptimizationsTreeNode folder = projectCreator.createOptimizationFolder();
        String url = createFolderUrl(folder);
        GETobject(mockMvc, url, OptimizationsTreeNode.class);
        DELETEobject(mockMvc, url);
        GETAndExpectNotFound(mockMvc, url);
    }


    private String createFolderUrl(OptimizationsTreeNode folder) {
        return OPTIMIZATION_TREE_URL + "/" + folder.getId();
    }
}
