package ru.crp.lvms.utils;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementDocsEntity;
import ru.crp.lvms.model.entities.misc.FileType;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.function.BiConsumer;

import static org.junit.Assert.*;
import static ru.crp.lvms.utils.TestUtils.*;

@Component()
@Scope("prototype")
public class FilesTester {
    private MockMvc mockMvc;

    @Autowired
    private FileManager fileManager;

    @Value("${storage.root}")
    private String filesFolder;

    @Autowired
    private ObjectMapper jacksonMapper;

    private Path pathToFiles;

    @PostConstruct
    public void init() {
        pathToFiles = Paths.get(filesFolder);
    }

    public void setMockMvc(MockMvc mockMvc) {
        this.mockMvc = mockMvc;
    }

    public void testUpdateFile(FileType fileType, String url) throws Exception {
        String testContent = "test content " + unique("file-content");
        String fileExtension = fileType == FileType.ImageFile ? ".jpeg" : "";

        UPLOADFile(mockMvc, url, testContent.getBytes(), fileExtension, null);
        byte[] savedFile = GETFile(mockMvc, url);
        assertArrayEquals(testContent.getBytes(), savedFile);
    }


    public void assertFileSaved(Integer fileId, byte[] expectedContent) throws IOException {
        assertNotNull(fileId);
        FileEntity file = fileManager.findOne(fileId, null);
        assertNotNull(file);
        assertNotNull(file.getPathToFile());
        Path pathToFile = pathToFiles.resolve(file.getPathToFile());
        assertTrue(Files.exists(pathToFile));
        byte[] fileContent = Files.readAllBytes(pathToFile);
        assertArrayEquals(expectedContent, fileContent);
    }


    public void testDeleteFile(FileType fileType, boolean isTechnology, String url,
                               BiConsumer<FileType, MvcResult> fileRemovedChecker) throws Exception {
        String testContent = "test content";
        String fileExtension = fileType == FileType.ImageFile ? ".jpeg" : "";
        MvcResult mvcResult = UPLOADFile(mockMvc, url, testContent.getBytes(), fileExtension, null);
        // todo definitely need fix
        if (fileType == FileType.DescriptionFile && !isTechnology) {
            ConstructionElementDocsEntity descriptionFile = jacksonMapper
                    .readValue(mvcResult.getResponse().getContentAsString(), ConstructionElementDocsEntity.class);
            url = url + "/" + descriptionFile.getId();

        }
        MvcResult result = DELETEobject(mockMvc, url);
        fileRemovedChecker.accept(fileType, result);
    }
}
