package ru.crp.lvms.tasks.slurm.commands;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.util.concurrent.ListenableFuture;
import ru.crp.cmls.calculations.solvers.xml.config.Type;
import ru.crp.lvms.cluster.ClustersManager;
import ru.crp.lvms.cluster.model.SlurmClusterConfigEntity;
import ru.crp.lvms.managers.ResultsManager;
import ru.crp.lvms.managers.solvers.SolverConfigManager;
import ru.crp.lvms.managers.tasks.ResultsFilesManager;
import ru.crp.lvms.managers.tasks.ResultsParent;
import ru.crp.lvms.managers.tasks.TaskResultsExtractor;
import ru.crp.lvms.model.entities.misc.CreatedBy;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.optimizations.base.managers.OptimizationManager;
import ru.crp.lvms.tasks.misc.BaseTaskCommand;
import ru.crp.lvms.tasks.slurm.SlurmConnectionManager;
import ru.crp.lvms.tasks.slurm.SlurmPathsResolver;
import ru.crp.lvms.utils.Mapper;

import java.nio.file.Path;

@Service
public class SlurmDownloadResultsCommand extends BaseTaskCommand<Void, Void> {
    public static final Logger LOGGER = LoggerFactory.getLogger(SlurmDownloadResultsCommand.class);

    @Autowired
    private SlurmConnectionManager slurmConnectionManager;

    @Autowired
    private SlurmPathsResolver slurmPathsResolver;

    @Autowired
    private ClustersManager clustersManager;

    @Autowired
    private ResultsFilesManager resultsFilesManager;

    @Autowired
    private TaskResultsExtractor taskResultsExtractor;

    @Autowired
    private ResultsManager resultsManager;

    @Autowired
    private OptimizationManager optimizationManager;

    // all exceptions thrown by this method will be caught in ListenableFuture callback
    @Async("slurmDownloadTasksResultExecutor")
    @Override
    public ListenableFuture<Void> processTask(TaskEntity task, Object unused) throws Exception {
        LOGGER.info(String.format("Clearing previous results for task %d", task.getId()));

        if (task.getPrevTaskId() == null && task.getType() != Type.toscaAnimation && task.getType() != Type.toscaSmoothing) {
            resultsManager.deleteResults(Mapper.map(task, ResultsParent.class), true, CreatedBy.Auto);
        }

        SlurmClusterConfigEntity clusterConfig = clustersManager.getSlurmClusterConfig(task.getClusterId());
        String outputDir = slurmPathsResolver.getOutputDirForTask(clusterConfig, task);

        Path resultsDir = resultsFilesManager.getResultsDir(task);

        slurmConnectionManager.downloadResults(clusterConfig, outputDir, resultsDir);
//        LOGGER.info(String.format("Removing work directory on server: %s", slurmPathsResolver.getWorkDirForTask(clusterConfig, task)));
//        slurmConnectionManager.execCommand(clusterConfig, new ActionRequest(String.format("rm -rf %s", slurmPathsResolver.getWorkDirForTask(clusterConfig, task))));
        LOGGER.info(String.format("Extracting results for task %d", task.getId()));
        taskResultsExtractor.extractResults(task);

        if (task.getType() == Type.optistructGenerateFEM) {
            optimizationManager.copyResultsFemToOptimizationFiles(task);
        }
        return new AsyncResult<>(null);
    }
}
